/**
* ColMajorExample
*
* This class illustrates a simple example of using the
* NetSolve Java API with matrices stored in column-major
* order.  This is the exactly the same as MatMulExample
* except that the matrices are stored in column-major 
* instead of row-major order.  
*
* @version 0.0, 24 Sep 1997
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see Netsolve
* @see NetsolveSession
* @see MatMulExample
**/

import java.util.Vector;

public class ColMajorExample extends Example {
  public static void main (String [] args) {
    ColMajorExample mm = new ColMajorExample();
    NetsolveSession ns;
    Netsolve n;
    Netsolve n2;
    double [][] a1 = {     // Initialize the data
      {1.0},
      {2.0},
      {3.0}
    };
    double [][] a2 = {     // Initialize the data
      {1.0, 2.0, 3.0},
      {4.0, 5.0, 6.0}
    };
    double [][] a3 = {     // Initialize the data
      {1.0, 6.0},
      {2.0, 7.0},
      {3.0, 8.0},
      {4.0, 9.0},
      {5.0, 10.0}
    };
    Vector out;

    // Since these variables are all set within the 'try'
    // block, the compiler can't confirm that they are 
    // initialized.  To avoid the subsequent complaints from
    // the compiler, we initialize them to null here.

    n = null;
    n2 = null;
    ns = null;
    out = null;

    // Almost all of the methods in Netsolve and NetsolveSession
    // throw NetSolveException upon some error condition, so it's
    // easier to just enclose the whole transaction in a try-catch
    // block than to try each one individually (although that is
    // possible, too).

    try {
      ns = new NetsolveSession("woodstock.cs.utk.edu");

      n = new Netsolve(ns);     // Specify 'session'
      n.setProblem("dmatmul");  // Specify the problem to solve
      n.setMajor("col");        // Specify column-major order 
      n.pushArg(a1);            // Pass first parameter to dmatmul
      n.pushArg(a2);            // Pass second parameter to dmatmul
      n.submitProb();           // Submit this problem
      out = n.getOutput();      // Get the output item(s)
    }catch(NetSolveException e) {
      System.err.println("Error in first submission:");
      System.err.println(e.getMessage());
      System.exit(1);
    }

    // Print results from first submission.
    // Note that the matrix comes back in column-major order,
    // so we call the appropriate print method.

    System.out.println("Results from first submission (a1 * a1)");
    mm.printMatColMajor( (double [][] ) (out.elementAt(0)) );

    // The second submission is enclosed in a separate try-catch
    // so that we may distinguish between error messages generated
    // during the first and second submissions.  Notice that in
    // initializing this Netsolve object, we reuse the NetsolveSession
    // from the previous submission.  This saves time by only
    // retrieving the problem list once.  However, if you wanted
    // to submit the second problem to a different agent, you 
    // would have to instantiate a new NetsolveSession, specifying
    // that agent.

    try {
      n2 = new Netsolve(ns);     // Specify 'session'
      n2.setProblem("dmatmul");  // Specify the problem to solve
      n2.setMajor("col");        // Specify column-major order 
      n2.pushArg(a2);            // Pass first parameter to dmatmul
      n2.pushArg(a3);            // Pass second parameter to dmatmul
      n2.submitProb();           // Submit this problem
      out = n2.getOutput();      // Get the output item(s)
    }catch(NetSolveException e) {
      System.err.println("Error in second submission:");
      System.err.println(e.getMessage());
      System.exit(1);
    }

    // Print results from second submission.
    // Note that the matrix comes back in column-major order,
    // so we call the appropriate print method.

    System.out.println("Results from second submission (a1 * a2)");
    mm.printMatColMajor( (double [][] ) (out.elementAt(0)) );
  }
}
