/** 
* NonBlockingExample 
*
* This program illustrates using the NetSolve Java API to submit
* problems in non-blocking (or asynchronous) mode.  This allows
* you to submit a problem and immediately continue performing
* other computations while the results are being computed.  In
* this example, we first initialize a large array and submit two
* copies of it to the matrix multiplication routine (dmatmul).  
* Since we have submitted the problem in non-blocking mode, we 
* can immediately attempt to solve another problem, this time a 
* small sorting problem (iqsort).  We then have nothing to do, 
* so we just wait for the sorting job to complete (knowing that 
* it will finish before the matmul).  After the sorting job has 
* completed, we print the vector and then wait for the matmul job 
* to finish.
*
* The following is a sample run of the program.  As you can see,
* the second problem is submitted immediately after the first.
* The second problem (iqsort) does not take long to complete, so
* about a second later, we get the results.  Meanwhile, the 
* matmul is still going and we have nothing better to do than
* just wait for it, so we call waitFor().  Eventually, after a
* little over 15 seconds, the data arrives and we are done.
*
*----
* Thu Sep 25 08:16:25 PDT 1997***Submitted first problem
* Thu Sep 25 08:16:26 PDT 1997***Submitted second problem
* Thu Sep 25 08:16:26 PDT 1997***Waiting for data from second problem...
* Thu Sep 25 08:16:27 PDT 1997***Got it.  The data:
* 3, 4, 4, 7, 43, 45, 54, 234, 434, 543, 867, 5643
* Thu Sep 25 08:16:27 PDT 1997***Waiting for data from first problem...
* Thu Sep 25 08:16:44 PDT 1997***Got it.
* Thu Sep 25 08:16:44 PDT 1997***Done.
*----
*
* @version 0.0, 24 Sep 1997
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see Netsolve
* @see NetsolveSession
**/

import java.util.Vector;
import java.util.Date;

public class NonBlockingExample extends Example {
  public static void main (String [] args) {
    NonBlockingExample nb = new NonBlockingExample();

    NetsolveSession ns = null;   // initialize all to null to avoid
    Netsolve n         = null;   //        the compiler complaints.
    Netsolve n2        = null;
    Vector out_vec     = null;

    // a1 will be the input array and a2 will hold the results (a2=a1*a1).
    double [][] a1;
    double [][] a2;

    // Vector to be sorted:
    int [] int_arr1 = { 43, 234, 543, 4, 4, 3, 5643, 434, 45, 867, 54, 7 };
    int [] int_arr2 = null;

    int i,j,k;

    // Initialize a1 to be 100x100 matrix
    a1 = new double [100][];
    for(i=0;i<a1.length;i++) {
      a1[i] = new double[100];
      for(j=0;j<a1[i].length;j++)
        a1[i][j] = (double)(i*j) * Math.PI;    // use some dummy value
    }

    // Here we submit the first problem (dmatmul):

    try{
      // create new session to woodstock
      ns = new NetsolveSession("woodstock.cs.utk.edu");

      n = new Netsolve(ns);     // Specify 'session'
      n.setProblem("dmatmul");  // Specify the problem to solve
      n.pushArg(a1);            // Pass first parameter to dmatmul
      n.pushArg(a1);            // Pass second parameter to dmatmul
      n.submitProbNB();         // Submit this problem

      System.out.print(new Date().toString());
      System.out.println("***Submitted first problem");
    }catch(NetSolveException e) {
      System.out.println("Error submitting first problem: " + e.getMessage());
      System.exit(1);
    }

    // Now that the first problem has been submitted, we can submit
    // the next one (iqsort).

    try{
      n2 = new Netsolve(ns);    // Specify 'session'
      n2.setProblem("iqsort");  // Specify the problem to solve
      n2.pushArg(int_arr1);     // Pass first parameter to iqsort
      n2.submitProbNB();        // Submit this problem

      System.out.print(new Date().toString());
      System.out.println("***Submitted second problem");
    }catch(NetSolveException e) {
      System.out.println("Error submitting second problem: " + e.getMessage());
      System.exit(1);
    }

    // Now we wait for the sorted vector:

    try{
      System.out.print(new Date().toString());
      System.out.println("***Waiting for data from second problem...");

      n2.waitFor();             // Wait for the data to be available.
      int_arr2 = (int []) n2.getOutput().elementAt(0);

      System.out.print(new Date().toString());
      System.out.println("***Got it.  The data:");
    }catch(NetSolveException e) {
      System.out.println("Error retrieving data from second problem: " +
         e.getMessage());
      System.exit(1);
    }

    // Just for fun, print it out:
    nb.printVec(int_arr2);

    // Now wait for the result of dmatmul to return. 

    try{
      System.out.print(new Date().toString());
      System.out.println("***Waiting for data from first problem...");

      n.waitFor();              // Wait for the data to be available.
      out_vec = n.getOutput();  // Get Output

      System.out.print(new Date().toString());
      System.out.println("***Got it.");
    }catch(NetSolveException e) {
      System.out.println("Error retrieving data from first problem: " +
         e.getMessage());
      System.exit(1);
    }

    // Grab the data but don't print since it's too big.
    a2 = (double [][]) out_vec.elementAt(0); 

    System.out.print(new Date().toString());
    System.out.println("***Done.");
  }
}
