/**
* DgesvExample
*
* This class illustrates a simple example of using the
* NetSolve Java API.  Two linear systems are solved (dgesv)
* using blocking calls.  Then the results are printed out.
*
* @version 1.1, 18 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see Netsolve
* @see NetsolveSession
**/

public class DgesvExample {
  public static void main (String [] args) {
    NetsolveSession ns;
    Netsolve n;
    Netsolve n2;
    double [][] a1 = {     // Initialize the data
      {1.0,   2.0,  3.0,  4.0},
      {5.0,   6.0,  7.0,  8.0},
      {9.0,  10.0, 11.0, 12.0},
      {13.0, 14.0, 15.0, 16.0}
    };
    double [][] a2 = {     // Initialize the data
      {1.0,   2.0,  3.0,  4.0},
      {13.0, 14.0, 15.0, 16.0},
      {9.0,  10.0, 11.0, 12.0},
      {5.0,   6.0,  7.0,  8.0}
    };
    NetSolveObject [] out;
    NetSolveIntScalar outInt;
    NetSolveDoubleMatrix outMat1, outMat2;
    NetSolveIntVector outVec;

    // Since these variables are all set within the 'try'
    // block, the compiler can't confirm that they are
    // initialized.  To avoid the subsequent complaints from
    // the compiler, we initialize them to null here.

    n = null;
    n2 = null;
    ns = null;
    out = null;

    // Almost all of the methods in Netsolve and NetsolveSession
    // throw NetSolveException upon some error condition, so it's
    // easier to just enclose the whole transaction in a try-catch
    // block than to try each one individually (although that is
    // possible, too).

    try {
      ns = new NetsolveSession("woodstock.cs.utk.edu");

      n = new Netsolve(ns);       // Specify 'session'
      n.setProblem("dgesv");      // Specify the problem to solve
      n.pushArg(a1);              // Pass first parameter to dgesv
      n.pushArg(a1);              // Pass second parameter to dgesv
      n.submitProb();             // Submit this problem
      out = n.getOutputObject();  // Get the output item(s)
    }catch(NetSolveException e) {
      System.err.println("Error in first submission:");
      System.err.println(e.getMessage());
      System.exit(1);
    }

    // Print results from first submission.

    System.out.println("Results from first submission:");

    outMat1 = (NetSolveDoubleMatrix) out[0];
    outVec  =    (NetSolveIntVector) out[1];
    outMat2 = (NetSolveDoubleMatrix) out[2];
    outInt  =    (NetSolveIntScalar) out[3];

    outMat1.printData();
    outVec.printData();
    outMat2.printData();
    outInt.printData();

    // The second submission is enclosed in a separate try-catch
    // so that we may distinguish between error messages generated
    // during the first and second submissions.  Notice that in
    // initializing this Netsolve object, we reuse the NetsolveSession
    // from the previous submission.  This saves time by only
    // retrieving the problem list once.  However, if you wanted
    // to submit the second problem to a different agent, you
    // would have to instantiate a new NetsolveSession, specifying
    // that agent.

    try {
      n2 = new Netsolve(ns);        // Specify 'session'
      n2.setProblem("dgesv");       // Specify the problem to solve
      n2.pushArg(a1);               // Pass first parameter to dgesv
      n2.pushArg(a2);               // Pass second parameter to dgesv
      n2.submitProb();              // Submit this problem
      out = n2.getOutputObject();   // Get the output item(s)
    }catch(NetSolveException e) {
      System.err.println("Error in second submission:");
      System.err.println(e.getMessage());
      System.exit(1);
    }

    // Print results from first submission.
    System.out.println("Results from second submission:");

    outMat1 = (NetSolveDoubleMatrix) out[0];
    outVec  =    (NetSolveIntVector) out[1];
    outMat2 = (NetSolveDoubleMatrix) out[2];
    outInt  =    (NetSolveIntScalar) out[3];

    outMat1.printData();
    outVec.printData();
    outMat2.printData();
    outInt.printData();

  }
}
