/**
* MatMulExample
*
* This class illustrates a simple example of using the
* NetSolve Java API.  A matrix of double precision floating
* point numbers is initialized and multiplied by itself.
* Then a second submission is made to illustrate reusing
* the NetsolveSession object.  The results are then printed 
* to the screen.  
*
* @version 0.1, 17 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see Netsolve
* @see NetsolveSession
**/

public class MatMulExample {
  public static void main (String [] args) {
    NetsolveSession ns;
    Netsolve n;
    Netsolve n2;
    double [][] a1 = {     // Initialize the data
      {1.0,   2.0,  3.0,  4.0},
      {5.0,   6.0,  7.0,  8.0},
      {9.0,  10.0, 11.0, 12.0},
      {13.0, 14.0, 15.0, 16.0}
    };
    double [][] a2 = {     // Initialize the data
      {1.0,   2.0,  3.0,  4.0},
      {13.0, 14.0, 15.0, 16.0},
      {9.0,  10.0, 11.0, 12.0},
      {5.0,   6.0,  7.0,  8.0}
    };
    NetSolveObject [] out;
    NetSolveDoubleMatrix product;

    // Since these variables are all set within the 'try'
    // block, the compiler can't confirm that they are 
    // initialized.  To avoid the subsequent complaints from
    // the compiler, we initialize them to null here.

    n = null;
    n2 = null;
    ns = null;
    out = null;
    product = null;

    // Almost all of the methods in Netsolve and NetsolveSession
    // throw NetSolveException upon some error condition, so it's
    // easier to just enclose the whole transaction in a try-catch
    // block than to try each one individually (although that is
    // possible, too).

    try {
      ns = new NetsolveSession("woodstock.cs.utk.edu");

      n = new Netsolve(ns);       // Specify 'session'
      n.setProblem("dmatmul");    // Specify the problem to solve
      n.pushArg(a1);              // Pass first parameter to dmatmul
      n.pushArg(a1);              // Pass second parameter to dmatmul
      n.submitProb();             // Submit this problem
      out = n.getOutputObject();  // Get the output item(s)

      product = (NetSolveDoubleMatrix) out[0]; 
    }catch(NetSolveException e) {
      System.err.println("Error in first submission:");
      System.err.println(e.getMessage());
      System.exit(1);
    }

    // Print results from first submission.

    System.out.println("Results from first submission (a1 * a1)");
    product.printData();

    // The second submission is enclosed in a separate try-catch
    // so that we may distinguish between error messages generated
    // during the first and second submissions.  Notice that in
    // initializing this Netsolve object, we reuse the NetsolveSession
    // from the previous submission.  This saves time by only
    // retrieving the problem list once.  However, if you wanted
    // to submit the second problem to a different agent, you 
    // would have to instantiate a new NetsolveSession, specifying
    // that agent.

    try {
      n2 = new Netsolve(ns);       // Specify 'session'
      n2.setProblem("dmatmul");    // Specify the problem to solve
      n2.pushArg(a1);              // Pass first parameter to dmatmul
      n2.pushArg(a2);              // Pass second parameter to dmatmul
      n2.submitProb();             // Submit this problem
      out = n2.getOutputObject();  // Get the output item(s)

      product = (NetSolveDoubleMatrix) out[0]; 
    }catch(NetSolveException e) {
      System.err.println("Error in second submission:");
      System.err.println(e.getMessage());
      System.exit(1);
    }

    // Print results from first submission.
    System.out.println("Results from second submission (a1 * a2)");
    product.printData();
  }
}
