/**
* UPF_Example
*
* This program illustrates how to submit a problem which requires a 
* User Provided Function (UPF) via the NetSolve Java API.  In this 
* case, we first read the UPFs from files into byte arrays.  Then when
* we are ready to submit the UPFs, we convert the byte arrays to 
* Strings.  The problem submission procedure is essentially the same 
* for UPFs and other data items such as vectors, but with UPFs you must 
* also specify the language (C or FORTRAN).  The appropriate values are
* found in the NetSolve class GlobalDefs:
*
*   GlobalDefs.LANG_FORTRAN  - represents FORTRAN
*   GlobalDefs.LANG_C        - represents C
*
* An example of submitting a UPF follows:
*
*   n.pushArg(upf_string,GlobalDefs.LANG_FORTRAN); 
*
* where upf_string is a String containing the FORTRAN source code. 
*
* @version 0.1, 17 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see Netsolve
* @see NetsolveSession
* @see GlobalDefs
**/

import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

public class UPF_Example {
  public static void main (String [] args) {
    FileInputStream file1 = null, 
                    file2 = null;
    byte [] upf0;
    byte [] upf1;
    NetsolveSession ns    = null;
    NetSolveObject [] obj = null;
    Netsolve n            = null;
    NetSolveIntScalar answer;

    // First attempt to open the files.  Naturally this won't
    // run as an applet.
    try {
      file1 = new FileInputStream("upf0.f");
      file2 = new FileInputStream("upf1.c");
    } catch (FileNotFoundException e) {
      System.err.println("Cannot open file(s)! " + e.getMessage( ));
      System.exit(1);
    }

    upf0 = new byte[400];
    upf1 = new byte[400];
    
    // Now read all the source code.
    try {
      file1.read(upf0);
      file2.read(upf1);
    } catch(IOException e) {
      System.err.println("Error reading from file(s)! " + e.getMessage());
      System.exit(1);
    }

    // The problem upftest requires an integer parameter also:
    int num = 7;

    try {
      ns = new NetsolveSession("woodstock.cs.utk.edu");

      n = new Netsolve(ns);     // Specify 'session'
      n.setProblem("upftest");  // Specify the problem to solve

      // Pass the UPFs to upftest
      n.pushArg(new String(upf0,0),GlobalDefs.LANG_FORTRAN); 
      n.pushArg(new String(upf1,0),GlobalDefs.LANG_C); 
      n.pushArg(num);           // Pass third parameter to upftest

      n.submitProb();           // Submit this problem

      obj = n.getOutputObject();
    }catch (NetSolveException e) {
      System.out.println(e.getMessage());
      System.exit(1);
    }

    // Check to see that we got an answer
    if(obj == null) {
      System.out.println("NO answer");
    } else {
      answer = (NetSolveIntScalar) obj[0];
      System.out.println("The answer is " + answer.getval());
    }
  }
}
