/**
* CheckData
*   
* This class contains methods to parse the user's input, whether the 
* source is from a file, URL, or text typed into the Data Input Window.
* If the data is of the correct format, the data may be sent to the
* server via the SubmitJob class.
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see SubmitJob
**/

import java.io.InputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.ByteArrayInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import java.net.URL;

import java.awt.Choice;
import java.awt.Frame;
import java.awt.AWTEvent;
import java.awt.event.ActionEvent;

import java.util.Vector;
import java.util.StringTokenizer;

public class CheckData extends NetSolveThread {

  private Problem prob;
  private String [] filename;
  private String [] data;
  private StringBuffer error_rpt = new StringBuffer(500);
  private int [] lang_checked;
  private int [] checked;

  public NetSolveObject [] obj;

  private InputData input_frame;
  private int cur_lang = Labels.language;
  private boolean data_err = false;

  /**
   *
   * This is the constructor for the CheckData class.  It just copies the
   * parameters to class variables, starts this class as a thread, and returns
   * to the caller.  The parameters are badly named, but they are only used
   * once each in this constructor to initialize the class variables.
   * 
   * @param a The problem to be solved.
   * @param b Array of filenames containing the data.
   * @param c Array of strings containing the data.
   * @param e Integer array representing the language of the UPFs (C or FORTRAN).
   * @param f Integer array representing the source of the data (file, URL, text).
   * @param g The parent frame.
   *
   **/

  public CheckData(Problem a, String [] b, String [] c,
    int [] e, int [] f, InputData g) {

    prob = a;
    filename = b;
    data = c;
    lang_checked = e;
    checked = f;
    input_frame = g;

    addActionListener(input_frame);

    obj = new NetSolveObject[prob.nb_input];
  } // end constructor CheckData

  /**
   * 
   * This method attempts to open the file and if successful returns
   * a FileInputStream.  If unsuccessful, append an error message to
   * the current error report.
   * 
   * @param datatype The data type of this input object (int, float, etc).
   *                 This lets us know if it's a UPF or not.
   * @param filename The name of the file to open.
   * @param err StringBuffer in which to append error messages.
   *
   * @return Returns a FileInputStream to the opened file.
   *
   **/

  private FileInputStream openFile(NetSolveObject obj, String filename, 
   StringBuffer err) {
    FileInputStream in;
    StringBuffer temp = new StringBuffer(500);
    int resp;

    if(obj instanceof NetSolveUPF) {
      if(filename.length() < 2) {
        // if the filename is less than two characters, it can't possibly
        // end in .c or .f so generate an error

        if(GlobalDefs.DEBUG)
          System.out.println("Invalid filename...Must end in .c or .f");

        err.append(Labels.invalid_file[cur_lang] + "\n");
        return(null);
      }

      if(filename.charAt(filename.length()-2) != '.') {
        // second to last character is not a '.'
        if(GlobalDefs.DEBUG)
          System.out.println("Invalid filename...Must end in .c or .f");

        err.append(Labels.invalid_file[cur_lang] + "\n");
        return(null);
      }
      
      if(filename.charAt(filename.length()-1) == 'c') {
        if(GlobalDefs.DEBUG)
          System.out.println("language is C");
      }
      else if(filename.charAt(filename.length()-1) == 'f') {
        if(GlobalDefs.DEBUG)
          System.out.println("language is FORTRAN");
      }
      else {
        if(GlobalDefs.DEBUG)
          System.out.println("Invalid filename...Must end in .c or .f");
        err.append(Labels.invalid_file[cur_lang] + "\n");
        return(null);
      }
    }

    // The filename looks ok... now try to open it.
    try {
      in = new FileInputStream(filename);
    } catch (FileNotFoundException e) {
      if(GlobalDefs.DEBUG)
        System.out.println("File not found");
      err.append(Labels.no_file[cur_lang]+filename+"\n");
      return(null);
    }
   
    return(in);
  } // end method openFile

  /**
   * 
   * This method returns the current error report - a listing of all
   * the errors in the user's data.
   * 
   * @return Returns a String containing the report.
   *
   **/

  public String getErrorReport()
  {
    return(error_rpt.toString());
  }

  /**
   * 
   * This method returns the status of the user's data.
   * If true, the data was OK.  If false, there were errors
   * in the data.  The error report can be obtained by
   * calling CheckData.getErrorReport().
   * 
   * @return Returns a boolean containing the status..
   *
   * @see #getErrorReport
   *
   **/

  public boolean dataOK()
  {
    return(!data_err);
  }

  /**
   * 
   * Since this class extends Thread, it should have a method, run(),
   * which is called when the thread is started.  This method loops through
   * each input object, parses it, records error information, and starts
   * the data submission (via SubmitJob).
   * 
   * @see SubmitJob
   **/

  public void run() {
    int i,j,k;
    StringBuffer temp = new StringBuffer(100);
    String [] errors;
    int upfCount = 0;
    
    data_err = false;

    input_frame.setStatusText(Labels.checking_data[cur_lang]);
    errors = new String[prob.nb_input];

    for(i=0;i<prob.nb_input;i++) {
      if(checked[i] == GlobalDefs.FILE_INPUT) {
        // this input object is coming from a file
        temp.setLength(0);

        FileInputStream fs = openFile(prob.inputs[i],
                                filename[i],temp);
        if(fs == null) {
          if(GlobalDefs.DEBUG)
            System.out.println("ERROR in your data");
          errors[i] = new String(temp.toString());
          data_err = true;
        }
        else {
          // the file was opened successfully, now check the data

          obj[i] = NetSolveObject.newObject(prob.inputs[i].data_type,
             prob.inputs[i].object_type);
     
          try {
            obj[i].initialize(fs,this);
          }catch (NetSolveException e) {
            if(GlobalDefs.DEBUG)
              System.out.println("ERROR in your data");
            errors[i] = new String(e.getMessage());
            data_err = true;
          }

          if(obj[i] instanceof NetSolveUPF) {
            ((NetSolveUPF) obj[i]).setLang(filename[i]);
            ((NetSolveUPF) obj[i]).setUPFnum(upfCount);
            upfCount++;
          }

          if(obj[i] instanceof NetSolveMatrix)
            ((NetSolveMatrix) obj[i]).setMajor(GlobalDefs.ROW_MAJOR);

          try {
            fs.close();
          }catch(IOException e) {
            // try closing the file, but does it matter if we can't?
            System.err.println("Can't close FileInputStream, but I don't care.");
          }
        }
      }
      else if(checked[i] == GlobalDefs.URL_INPUT) {
        // this input object is coming from a URL

        InputStream url_input = null;
        byte [] head = new byte[4];
        byte b;
        int loop_count=0;

        temp.setLength(0);
        if(GlobalDefs.DEBUG)
          System.out.println("Going to try to read input data from URL: " + filename[i]);
        
        try {
           url_input = new URL(filename[i]).openStream();
          
           /* 
            * Sometimes the http server returns a bunch of header
            * information which will screw up things on the server
            * side if we don't strip it off first.  The following
            * bit of code attempts to determine if there's an http
            * header and if so, strips it off.  I thought the URL
            * class would have a way to return only the content
            * but I couldn't find one (even getContent() returned
            * the headers).
            */

           // mark the current position so that we can return to
           // this point if the first part of the data is not "HTTP"
           if(url_input.markSupported())
             url_input.mark(10);

           url_input.read(head, 0, 4);

           if(head[0]=='H' && head[1]=='T' && head[2]=='T' && head[3]=='P')
           {
             //server sent us some http crap that we need to strip off
             while(true) {
               if(loop_count++ > 200)  // guard against infinite loops
                 throw new IOException("Weird HTTP Header");

               // supposedly the http header stuff will always be followed
               // by two newlines, so that's what we're scanning for here.

               while((b = (byte) url_input.read()) != '\n') {
                 if(b == -1)
                   throw new IOException("Weird HTTP Header");
               }   

               b = (byte) url_input.read();
               if(b == '\n') {
                 // we're done
                 break;
               }
               else if(b == '\r') {
                 // we're almost done
                 url_input.read();
                 break;
               }
             }
           }
           else {
             /*
              * we didn't get HTTP stuff back, so reset the input stream
              * to the beginning, if possible.  otherwise, just reopen the
              * URL.
              */
             if(url_input.markSupported())
               url_input.reset();
             else
               url_input = new URL(filename[i]).openStream();
           }
        } catch (java.net.MalformedURLException e) {
           System.err.println("Bad URL: " + e.getMessage());
           errors[i] = new String(Labels.bad_url[cur_lang]);
           data_err = true;
        } catch (IOException e2) {
           System.err.println("Couldn't open stream to url");
           errors[i] = Labels.stream_err[cur_lang] + e2.getMessage();
           data_err = true;
        }

        if(url_input != null) {
          // the URL has been opened, now check the data

          obj[i] = NetSolveObject.newObject(prob.inputs[i].data_type,
             prob.inputs[i].object_type);
     
          try {
            obj[i].initialize(url_input,this);
          }catch (NetSolveException e2) {
            if(GlobalDefs.DEBUG)
              System.out.println("ERROR in your data");
            errors[i] = new String(e2.getMessage());
            data_err = true;
          }

          if(obj[i] instanceof NetSolveUPF) {
            ((NetSolveUPF) obj[i]).setLang(filename[i]);
            ((NetSolveUPF) obj[i]).setUPFnum(upfCount);
            upfCount++;
          }

          if(obj[i] instanceof NetSolveMatrix)
            ((NetSolveMatrix) obj[i]).setMajor(GlobalDefs.ROW_MAJOR);

          try {
            url_input.close();
          }catch(IOException e3) {
            // try closing the InputStream, but does it matter if we can't?
            System.err.println("Can't close InputStream, but I don't care.");
          }
        }
      }
      else if(checked[i] == GlobalDefs.DATA_INPUT) {

        /* 
         * This input object is coming from the textArea of the GUI
         *
         * Under JDK 1.0.2, we used a StringBufferInputStream, but
         * that class is deprecated under 1.1, so we must find an
         * alternative.  The documentation suggests using the
         * StringReader class, but we need to use a class derived
         * from InputStream, so we choose ByteArrayInputStream.
         */

        ByteArrayInputStream sb = new ByteArrayInputStream(data[i].getBytes());

        obj[i] = NetSolveObject.newObject(prob.inputs[i].data_type,
             prob.inputs[i].object_type);
     
        try {
          obj[i].initialize(sb,this);
        }catch (NetSolveException e3) {
          if(GlobalDefs.DEBUG)
            System.out.println("ERROR in your data");
          errors[i] = new String(e3.getMessage());
          data_err = true;
        }

        if(obj[i] instanceof NetSolveUPF) {
          ((NetSolveUPF) obj[i]).setLang(lang_checked[i]);
          ((NetSolveUPF) obj[i]).setUPFnum(upfCount);
          upfCount++;
        }

        if(obj[i] instanceof NetSolveMatrix)
          ((NetSolveMatrix) obj[i]).setMajor(GlobalDefs.ROW_MAJOR);

        try {
          sb.close();
        }catch(IOException e) {
          // try closing the StringReader, but does it matter if we can't?
          System.err.println("Can't close StringReader, but I don't care.");
        }
      }
      else {
        // data comes from nowhere, flag this as an error

        if(GlobalDefs.DEBUG)
          System.out.println(
            "Error - you must enter a file or some data for input "+i);
        errors[i] = new String(Labels.no_input[cur_lang]+i);
        data_err = true;
      }
    }

    input_frame.setStatusText(Labels.good_data[cur_lang]);

    if(GlobalDefs.DEBUG) {    // When debugging print all the data.
      System.out.println("dimensions: ");
      for(i=0;i<prob.nb_input;i++) {
        System.out.println("Object " + i + ":");
        System.out.println("    m = " + obj[i].m);
        System.out.println("    n = " + obj[i].n);
      }

      System.out.println("DATA:");
      for(i=0;i<prob.nb_input;i++)
        obj[i].printData();
    }    // end if(DEBUG)
    
    input_frame.setStatusText(Labels.submitting[cur_lang]);

    if(data_err)
    {
      // There were errors in the data, generate an error report.

      if(GlobalDefs.DEBUG)
        System.out.println(
          "Not submitting your data because of the following errors:");

      error_rpt.append(Labels.no_submit[cur_lang] + "\n\n");

      for(i=0;i<prob.nb_input;i++)
        if(errors[i] != null) {
          if(GlobalDefs.DEBUG) {
            System.out.println("Input "+i+": ");
            System.out.println(errors[i]);
          }
          error_rpt.append(Labels.input[cur_lang]+i+": \n");
          error_rpt.append(errors[i] + "\n");
        }
    }

    // Tell the input screen that we're finished checking the data.

/*
    synchronized (input_frame) { input_frame.notify(); }
*/
    
    if(GlobalDefs.DEBUG)
      System.out.println("CheckData done...calling processEvent");

    this.setCompleted(true);
    processEvent(new ActionEvent(this,(int)AWTEvent.ACTION_EVENT_MASK,"hi there"));
  }
} // end class CheckData
