/**
* FirstFrame
*
* This class controls the graphical user interface for the Initial
* Screen.
* 
* @version 1.1, 26 Jan 1998 
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.util.Vector;

public class FirstFrame extends Frame
 implements ActionListener, ItemListener, KeyListener {

  private int reqID = 0;

  private Button quit_button;
  private Button help_button;
  private Button solve_button;
  private Button update_button;
  private Button servers_button;

  private List input_list;
  private List output_list;
  private List problem_list;

  private TextArea in_desc;
  private TextArea out_desc;
  private TextArea prob_desc;

  private TextField agent;

  private MenuBar mb;
  private RadioMenu lang_menu;
  private Menu option_menu;
  private CheckboxMenuItem [] lang_item;
  private MenuItem quit_item;

  private int current_problem = 0;
  private int cur_lang = Labels.language;

  private NetSolveClient parent;

  private Label prob_label;
  private Label desc_label;
  private Label in_label;
  private Label out_label;
  private Label in_desc_label;
  private Label out_desc_label;
  private Label agent_label;

  private static final int INPUT_LIST_SIZE     = 5;
  private static final int OUTPUT_LIST_SIZE    = 5;
  private static final int PROBLEM_LIST_SIZE   = 14;
  private static final int IN_DESC_WIDTH       = 30;
  private static final int IN_DESC_HEIGHT      = 5;
  private static final int OUT_DESC_WIDTH      = 30;
  private static final int OUT_DESC_HEIGHT     = 5;
  private static final int PROB_DESC_WIDTH     = 70;
  private static final int PROB_DESC_HEIGHT    = 5;

  private static final int SMALL               = 10;
  private static final int MEDIUM              = 12;

  private static final int LABEL_FONT_SIZE     = MEDIUM;
  private static final int BUTTON_FONT_SIZE    = MEDIUM;
  private static final int PROBLIST_FONT_SIZE  = SMALL;
  private static final int PROBDESC_FONT_SIZE  = SMALL;
  private static final int INDESC_FONT_SIZE    = SMALL;
  private static final int OUTDESC_FONT_SIZE   = SMALL;
  private static final int INLIST_FONT_SIZE    = SMALL;
  private static final int OUTLIST_FONT_SIZE   = SMALL;
  private static final int AGENT_FONT_SIZE     = SMALL;

  public ScreenList screen_list;

  private boolean inApplet;

  private Panel prob_panel;

  /**
   * 
   * This is the constructor for the FirstFrame class.  The purpose of
   * this method is to initialize and layout all the graphical components 
   * in the Initial Screen.  This is all done with standard AWT components
   * with the exception of the RadioMenu class.
   * 
   * @param f The parent frame.
   * @param hostname The hostname of the agent.
   *
   * @see RadioMenu
   **/

  public FirstFrame(NetSolveClient f, String hostname) {

    super("NetSolve Client 0.0"); 

    ImagePanel c = null;
    parent = f;

    inApplet = parent.inApplet;
    if(inApplet)
      screen_list = new ScreenList(20);

    this.setLayout(new BorderLayout(10,10)); 

    /*
     * Initialize all the labels for this screen.
     */
    prob_label = new Label(Labels.problems[cur_lang]);
    prob_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
 
    desc_label = new Label(Labels.desc[cur_lang]);
    desc_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    in_label = new Label(Labels.inputs[cur_lang]);
    in_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    out_label = new Label(Labels.outputs[cur_lang]);
    out_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    in_desc_label = new Label(Labels.in_desc[cur_lang]);
    in_desc_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    out_desc_label = new Label(Labels.out_desc[cur_lang]);
    out_desc_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    agent_label = new Label(Labels.agent[cur_lang]);
    agent_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    /* 
     * Initialize the lists.
     */
    problem_list = new List(PROBLEM_LIST_SIZE,false);
    problem_list.addItemListener(this);
    problem_list.addActionListener(this);
    problem_list.setFont(new Font("Helvetica",Font.PLAIN,PROBLIST_FONT_SIZE));

    input_list = new List(INPUT_LIST_SIZE,false);
    input_list.addItemListener(this);
    input_list.setFont(new Font("Helvetica",Font.PLAIN,INLIST_FONT_SIZE));

    output_list = new List(OUTPUT_LIST_SIZE,false);
    output_list.addItemListener(this);
    output_list.setFont(new Font("Helvetica",Font.PLAIN,OUTLIST_FONT_SIZE));

    /* 
     * Initialize the description textareas.
     */
    in_desc = new TextArea(IN_DESC_HEIGHT,IN_DESC_WIDTH);
    in_desc.setFont(new Font("Helvetica",Font.BOLD,INDESC_FONT_SIZE));
    in_desc.setEditable(false);
  
    out_desc = new TextArea(OUT_DESC_HEIGHT,OUT_DESC_WIDTH);
    out_desc.setFont(new Font("Helvetica",Font.BOLD,OUTDESC_FONT_SIZE));
    out_desc.setEditable(false);

    prob_desc = new TextArea(PROB_DESC_HEIGHT,PROB_DESC_WIDTH);
    prob_desc.setFont(new Font("Helvetica",Font.BOLD,PROBDESC_FONT_SIZE));
    prob_desc.setEditable(false);

    /*
     * Initialize the Options->Language menu which allows selecting
     * the language in which the labels should be displayed.
     */
    lang_item = new CheckboxMenuItem[Labels.NUM_LANGS];
    lang_menu = new RadioMenu("Language",false);

    // add the available languages to the language menu
    for(int i=0;i < Labels.NUM_LANGS; i++) {
      lang_item[i] = new CheckboxMenuItem(Labels.lang[i]);
      lang_item[i].addItemListener(this);
      lang_menu.add(lang_item[i]);
    }

    // select the default lanuage
    lang_menu.selectItem(lang_item[cur_lang]);

    Menu option_menu = new Menu("Options");
    option_menu.add(lang_menu);

    Menu file_menu = new Menu("File");
    quit_item = new MenuItem("Quit");
    quit_item.addActionListener(this);
    file_menu.add(quit_item);

    MenuBar mb = new MenuBar();
    mb.add(file_menu);
    mb.add(option_menu);

    this.setMenuBar(mb);

    /* 
     * Initialize the buttons.
     */
    solve_button = new Button(Labels.solve[cur_lang]);
    solve_button.addActionListener(this);
    solve_button.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    solve_button.setEnabled(false);

    help_button = new Button(Labels.help[cur_lang]);
    help_button.addActionListener(this);
    help_button.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));

    quit_button = new Button(Labels.quit[cur_lang]);
    quit_button.addActionListener(this);
    quit_button.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));

    servers_button = new Button(Labels.serv_info[cur_lang]);
    servers_button.addActionListener(this);
    servers_button.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));

    update_button = new Button(Labels.contact[cur_lang]);
    update_button.addActionListener(this);
    update_button.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));

    /* 
     * Load the logo image.  If we're running as an applet we will have
     * to load the image differently than if we're running as an
     * application.
     */

    if(inApplet) {
      ; // no logo
    }
    else {
      ; // c = new ImagePanel("logo2.gif");
    }

    /*
     * Initialize the agent selection textfield.
     */
    agent = new TextField(hostname,40);
    agent.addKeyListener(this);
    agent.setFont(new Font("Helvetica",Font.BOLD,AGENT_FONT_SIZE));

    /* 
     * Set up all Panels.
     */

    // define panel holding problem description
    Panel desc_panel = new Panel();
    desc_panel.setLayout(new BorderLayout(5,5));

    desc_panel.add("South",prob_desc);
    desc_panel.add("North",desc_label);

    // define panel to hold buttons (solve, help, quit, etc.)
    Panel button_panel = new Panel();
    button_panel.setLayout(new GridLayout(0,2,5,5));

    button_panel.add(solve_button);
    button_panel.add(help_button);
    button_panel.add(servers_button);
    button_panel.add(quit_button);
 
    Panel bottom_right_panel = new Panel();
    bottom_right_panel.setLayout(new BorderLayout(5,5));

    if(c != null) 
      bottom_right_panel.add("Center",c);

    bottom_right_panel.add("South",button_panel);

    Panel bottom_panel = new Panel();
    bottom_panel.setLayout(new BorderLayout(5,5));

    bottom_panel.add("West",desc_panel);
    bottom_panel.add("East",bottom_right_panel);

    GridBagLayout gbl = new GridBagLayout();

// set up constraints...

  // problem list
    GridBagConstraints probListCon = new GridBagConstraints();
    probListCon.fill       = GridBagConstraints.BOTH;
    probListCon.gridheight = 3;
    probListCon.gridwidth  = 2;
    probListCon.gridx      = GridBagConstraints.RELATIVE;
    probListCon.gridy      = GridBagConstraints.RELATIVE;
    probListCon.weightx    = 300.0;
    probListCon.weighty    = 1.0;
    probListCon.insets     = new Insets(0,0,30,30);

  // problem list label
    GridBagConstraints probLabelCon = new GridBagConstraints();
    probLabelCon.fill       = GridBagConstraints.BOTH;
    probLabelCon.gridheight = 1;
    probLabelCon.gridwidth  = 2;
    probLabelCon.gridx      = GridBagConstraints.RELATIVE;
    probLabelCon.gridy      = GridBagConstraints.RELATIVE;
    probLabelCon.weightx    = 0.0;
    probLabelCon.weighty    = 0.0;

  // inputs list
    GridBagConstraints inListCon = new GridBagConstraints();
    inListCon.fill       = GridBagConstraints.BOTH;
    inListCon.gridheight = 1;
    inListCon.gridwidth  = 1;
    inListCon.gridx      = GridBagConstraints.RELATIVE;
    inListCon.gridy      = GridBagConstraints.RELATIVE;
    inListCon.weightx    = 0.0;
    inListCon.weighty    = 0.0;
    inListCon.insets     = new Insets(0,0,30,30);

  // inputs list label
    GridBagConstraints inLabelCon = new GridBagConstraints();
    inLabelCon.fill       = GridBagConstraints.BOTH;
    inLabelCon.gridheight = 1;
    inLabelCon.gridwidth  = 1;
    inLabelCon.gridx      = GridBagConstraints.RELATIVE;
    inLabelCon.gridy      = GridBagConstraints.RELATIVE;
    inLabelCon.weightx    = 0.0;
    inLabelCon.weighty    = 0.0;

  // input description
    GridBagConstraints inDescCon = new GridBagConstraints();
    inDescCon.fill       = GridBagConstraints.BOTH;
    inDescCon.gridheight = 1;
    inDescCon.gridwidth  = GridBagConstraints.REMAINDER;
    inDescCon.gridx      = GridBagConstraints.RELATIVE;
    inDescCon.gridy      = GridBagConstraints.RELATIVE;
    inDescCon.weightx    = 0.0;
    inDescCon.weighty    = 0.0;

  // input description label
    GridBagConstraints inDescLabelCon = new GridBagConstraints();
    inDescLabelCon.fill       = GridBagConstraints.BOTH;
    inDescLabelCon.gridheight = 1;
    inDescLabelCon.gridwidth  = GridBagConstraints.REMAINDER;
    inDescLabelCon.gridx      = GridBagConstraints.RELATIVE;
    inDescLabelCon.gridy      = GridBagConstraints.RELATIVE;
    inDescLabelCon.weightx    = 0.0;
    inDescLabelCon.weighty    = 0.0;

  // outputs list
    GridBagConstraints outListCon = new GridBagConstraints();
    outListCon.fill       = GridBagConstraints.BOTH;
    outListCon.gridheight = 1;
    outListCon.gridwidth  = 1;
    outListCon.gridx      = GridBagConstraints.RELATIVE;
    outListCon.gridy      = GridBagConstraints.RELATIVE;
    outListCon.weightx    = 0.0;
    outListCon.weighty    = 0.0;
    outListCon.insets     = new Insets(0,0,30,30);

  // outputs list label
    GridBagConstraints outLabelCon = new GridBagConstraints();
    outLabelCon.fill       = GridBagConstraints.BOTH;
    outLabelCon.gridheight = 1;
    outLabelCon.gridwidth  = 1;
    outLabelCon.gridx      = GridBagConstraints.RELATIVE;
    outLabelCon.gridy      = GridBagConstraints.RELATIVE;
    outLabelCon.weightx    = 0.0;
    outLabelCon.weighty    = 0.0;

  // output description
    GridBagConstraints outDescCon = new GridBagConstraints();
    outDescCon.fill       = GridBagConstraints.BOTH;
    outDescCon.gridheight = 1;
    outDescCon.gridwidth  = GridBagConstraints.REMAINDER;
    outDescCon.gridx      = GridBagConstraints.RELATIVE;
    outDescCon.gridy      = GridBagConstraints.RELATIVE;
    outDescCon.weightx    = 0.0;
    outDescCon.weighty    = 0.0;

  // output description label
    GridBagConstraints outDescLabelCon = new GridBagConstraints();
    outDescLabelCon.fill       = GridBagConstraints.BOTH;
    outDescLabelCon.gridheight = 1;
    outDescLabelCon.gridwidth  = GridBagConstraints.REMAINDER;
    outDescLabelCon.gridx      = GridBagConstraints.RELATIVE;
    outDescLabelCon.gridy      = GridBagConstraints.RELATIVE;
    outDescLabelCon.weightx    = 0.0;
    outDescLabelCon.weighty    = 0.0;

    gbl.setConstraints(problem_list, probListCon);
    gbl.setConstraints(prob_label, probLabelCon);
    gbl.setConstraints(input_list, inListCon);
    gbl.setConstraints(in_label, inLabelCon);
    gbl.setConstraints(in_desc, inDescCon);
    gbl.setConstraints(in_desc_label, inDescLabelCon);
    gbl.setConstraints(output_list, outListCon);
    gbl.setConstraints(out_label, outLabelCon);
    gbl.setConstraints(out_desc, outDescCon);
    gbl.setConstraints(out_desc_label, outDescLabelCon);

    Panel top_panel = new Panel();
    top_panel.setLayout(gbl);

    top_panel.add(prob_label);
    top_panel.add(in_label);
    top_panel.add(in_desc_label);
    top_panel.add(problem_list);
    top_panel.add(input_list);
    top_panel.add(in_desc);
    top_panel.add(out_label);
    top_panel.add(out_desc_label);
    top_panel.add(output_list);
    top_panel.add(out_desc);

    Panel main_panel = new Panel();
    main_panel.setLayout(new BorderLayout(5,5));

    main_panel.add("North",top_panel);
    main_panel.add("South",bottom_panel);

    Panel agent_select = new Panel();
    agent_select.setLayout(new BorderLayout(5,5));

    agent_select.add("West",agent_label);
    agent_select.add("Center",agent);
    agent_select.add("East",update_button);

    this.add("North",agent_select);
    this.add("South",main_panel);

    /*
     * The following three lines are added for the benefit of
     * Netscape 3.x browsers which will not initialize lists to
     * their proper size unless there's at least one element
     * added prior to showing the screen.  We will promptly
     * remove these dummy elements so as not to interfere with
     * the operation of the interface.
     */

    problem_list.add("alskdjflaksjdf lkasjdflkajsd fasdf hi");
    input_list.add("fasdf hi");
    output_list.add("fasdf hi");

    this.pack();
    this.setVisible(true);

    // Here we remove the dummy elements from the three lists
    // as described above.

    problem_list.remove(0);
    input_list.remove(0);
    output_list.remove(0);
  }  

  /**
   *
   * If this method returns true, this program is running as 
   * an applet.  If false, this program is running as an application.
   *
   * @return Returns boolean representing whether this program
   *    is running as an applet or as an application.
   *
   **/

  public boolean in_applet() {
    return inApplet;
  }

  /**
   *
   * The purpose of this method is to increment and return the request 
   * ID.  The request ID is an integer that is used to uniquely identify
   * each Input Window that is created as a result of clicking the
   * 'Solve' button.  Each time a new window is created, the
   * request ID should be incremented.  This method is synchronized, 
   * just in case.
   *
   * @return Returns integer representing the request ID.
   *
   **/

  public synchronized int getReqID() {
    reqID++;
    return reqID;
  }

  /**
   * 
   * This method updates the problem list (in the Initial Screen) with
   * values from the array of Problems passed in.  This is usually only
   * called after contacting an agent and retrieving its problem list.
   * 
   * @param problems Array of Problems.
   *
   * @see Problem
   **/

  public void updateProbList(Problem [] problems) {
    int i,j;

    problem_list.removeAll();   // First clear the old list

    // then for each problem, add the problem name to the list
    for(i=0;i<problems.length;i++) {
      problem_list.add(problems[i].path + problems[i].name);
    }
    solve_button.setEnabled(true);
  }

  /**
   * 
   * This method updates the input and output lists (in the Initial
   * Screen) with values from the array of Problems (retrieved from the
   * agent earlier).  This is usually called after the user selects a 
   * problem in the Problem List.
   * 
   * @param idx The index of the selected problem.
   *
   **/

  private void updateIOLists(int idx) {
    int i;

    input_list.removeAll();   // Clear the list of input objects

    // then, for each input object required by this problem,
    // add the data type and object type to the input list

    for(i=0;i<parent.gp.problems[idx].nb_input;i++) {

      input_list.add(
        NetSolveObject.datatype2str(parent.gp.problems[idx].inputs[i].data_type)
        + " - " +
        NetSolveObject.objtype2str(parent.gp.problems[idx].inputs[i].object_type));
    }

    output_list.removeAll();   // Clear the list of output objects

    // then, for each output object returned by this problem,
    // add the data type and object type to the output list

    for(i=0;i<parent.gp.problems[idx].nb_output;i++) {
      output_list.add(
        NetSolveObject.datatype2str(parent.gp.problems[idx].outputs[i].data_type)
        + " - " +
        NetSolveObject.objtype2str(parent.gp.problems[idx].outputs[i].object_type));
    }
  }

  /**
   * 
   * This method attempts to contact the agent specified in the TextField of
   * the Initial Screen.  It will then retrieve the list of problems from the
   * agent and clear all the lists.
   * 
   * @see GetProblems
   **/

  private void contactAgent() {
    String agent_name;

    // get the text from the agent selection field
    agent_name = agent.getText();   

    if(agent_name.length() == 0) {
      // nothing typed in the agent selection text field
      NetSolveWarning no_agent = new NetSolveWarning(this,
        Labels.no_agent[cur_lang]);
      return;
    }

    // attempt to get problem list from the specified agent and display it
    try {
      parent.gp = new GetProblems(agent_name);
      updateProbList(parent.gp.problems);

      if(parent.gp.problems.length == 0)  // the agent has no problems
        new NetSolveWarning(this, Labels.no_problems[cur_lang]);
    }catch(NetSolveException e) {
      // Could not make connection
      NetSolveWarning conn_err = new NetSolveWarning(this,
        Labels.no_connection[cur_lang] + e.getMessage());

      // since we could not connect to the specified agent,
      // clear the list of problems and disable the solve button
      problem_list.removeAll();
      solve_button.setEnabled(false);
    }

    // clear the various lists and description boxes
    input_list.removeAll();
    output_list.removeAll();
    prob_desc.setText("");
    in_desc.setText("");
    out_desc.setText("");
  }

  /**
   * 
   * This function 'resets' all the labels on the screen.  This is
   * called when the user changes the language selector.
   *
   * @see Labels
   **/

  private void updateLabels() {

    // update labels using the current language
    prob_label.setText(Labels.problems[cur_lang]);
    desc_label.setText(Labels.desc[cur_lang]);
    in_label.setText(Labels.inputs[cur_lang]);
    out_label.setText(Labels.outputs[cur_lang]);
    in_desc_label.setText(Labels.in_desc[cur_lang]);
    out_desc_label.setText(Labels.out_desc[cur_lang]);
    agent_label.setText(Labels.agent[cur_lang]);

    // update buttons using the current language
    solve_button.setLabel(Labels.solve[cur_lang]);
    help_button.setLabel(Labels.help[cur_lang]);
    quit_button.setLabel(Labels.quit[cur_lang]);
    servers_button.setLabel(Labels.serv_info[cur_lang]);
    update_button.setLabel(Labels.contact[cur_lang]);
  }

  /**
   * 
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    if(inApplet)
      screen_list.disposeAll();
    super.dispose();
  }

  /**
   * 
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   * 
   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    if (source == quit_button) {
      // user clicked the "quit" button
      if(inApplet)
        this.dispose();
      else
        System.exit(0);
    } 
    else if( source == quit_item )
    {
      // user selected "quit" from the menu
      if(inApplet)
        this.dispose();
      else
        System.exit(0);
    }
    else if (source == servers_button) {
      // user clicked the "Server Info" button

      try {
        GetServers gs = new GetServers(agent.getText());
        ServerList sl = new ServerList(gs,agent.getText());

        if(inApplet)
          screen_list.addElement(sl);
      } catch (NetSolveException e) {
        System.err.println("Error getting server information: " + e);
        NetSolveWarning ns = new NetSolveWarning(this,
          Labels.info_error[cur_lang] + e);
      }
    }
    else if (source == help_button) {
      // user clicked on "Help" button

      HelpScreen hlp = new HelpScreen();

      if(inApplet)
        screen_list.addElement(hlp);
    } 
    else if (source == update_button) {
      // user clicked on "Contact/Update" button

      contactAgent();
    }
    else if((source == solve_button) || 
            (source == problem_list)) 
    {
      // the user either clicked "solve" or double-clicked a problem

      int selected;

      // get the index of the selected problem
      selected = problem_list.getSelectedIndex();

      if(selected == -1) {
        // if there's no problem selected, warn the user that they
        // must select one

        NetSolveWarning solve_warn = new NetSolveWarning(this,
          Labels.no_selected[cur_lang]);
      }
      else {
        String agent_name = agent.getText();
        Problem prob_clone = new Problem();

        // make a copy of this problem so that if we need to
        // perform any modifications, it won't affect the original.
        prob_clone.copy(parent.gp.problems[selected]);

        // user wants to solve a problem, start up the Input Screen
        InputData di = new InputData(this,agent_name,prob_clone);

        // add this object to the list of input screens so that when we 
        // quit this screen, we'll know which screens we need to kill off.
        // but we only need to worry about this if we're running as an
        // applet.
        if(inApplet)
          screen_list.addElement(di);

      }
    } 
  }

  /**
   * 
   * This method is called when the user selects a problem from
   * the problem list.  First, get the index of the selected
   * problem.  Then, initialize the input list, the output list,
   * and the problem description.
   * 
   **/

  private void handleProbListSelection()
  {
    int selected = problem_list.getSelectedIndex();

    current_problem = selected;
    prob_desc.setText(parent.gp.problems[selected].description);
    updateIOLists(selected);

    in_desc.setText("");    // clear input description
    out_desc.setText("");   // clear output description
  }

  /**
   * 
   * This method is called when the user selects an input object
   * from the input list.  First, get the index of the selected
   * input object.  Then, initialize the input description box.
   *
   **/

  private void handleInputListSelection()
  {
    int selected = input_list.getSelectedIndex();

    in_desc.setText(
      parent.gp.problems[current_problem].inputs[selected].description);
  }

  /**
   * 
   * This method is called when the user selects an output object
   * from the output list.  First, get the index of the selected
   * output object.  Then, initialize the output description box.
   *
   **/

  private void handleOutputListSelection()
  {
    int selected = output_list.getSelectedIndex();
    
    out_desc.setText(
      parent.gp.problems[current_problem].outputs[selected].description);
  }

  /**
   * 
   * This method is called when the user selects an item from
   * the 'Language' menu.  Loop through each menu item until the
   * selected one is found.  Then, set the current language to
   * the index of the selected item and update all the labels.
   * 
   **/

  private void handleMenuSelection(Object source)
  {
    for(int i=0;i < Labels.NUM_LANGS; i++)
      if(source == lang_item[i])
      {
        Labels.setLang(i);
        cur_lang = Labels.language;
        lang_menu.selectItem(lang_item[i]);
        updateLabels();
      }
  }

  /**
   * 
   * This is the method which is called whenever an item changes state.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   * 
   * @param e The event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
    Object source = e.getSource();

    if(e.getStateChange() == ItemEvent.SELECTED)
    {
      if (source == problem_list)
        handleProbListSelection();
      else if (source == input_list)
        handleInputListSelection();
      else if (source == output_list)
        handleOutputListSelection();
      else if( source instanceof MenuItem )
        handleMenuSelection(source);
    }
  }

  /**
   * 
   * This is the method which is called whenever a key press event is
   * detected.  First, determine the source of the event.  Then execute 
   * the code associated with this event.
   * 
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)
  {
    if(e.getSource() == agent)
      if (e.getKeyCode() == KeyEvent.VK_ENTER)
        contactAgent();    // user hit the 'return' key in the 
                            // agent selection text field, contact
                            // the agent and update the lists
  }

  /**
   * 
   * Does nothing.
   * 
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   * 
   * Does nothing.
   * 
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }
}
