/**
* GetProblems
* 
* This class provides the mechanism to retrieve a list of problems
* from a NetSolve agent.
* 
* @version 1.1, 26 Jan 1998 
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.io.IOException;

public class GetProblems extends GlobalDefs {
  private NetSolveStream str;
  public Problem [] problems;
  private final int DETAIL = 3;
  public int cur_lang = Labels.language;

  /**
   * 
   * This method reads all the information related to each problem 
   * that the NetSolve agent returns.  This information is stored in
   * an array of Problems.
   * 
   * @param num_prob An integer representing the number of problems.
   *
   * @exception java.io.IOException Throws this exception when it 
   *    encounters problems reading from the socket (e.g., a broken pipe).
   *
   * @see Problem
   **/

  private void readProblems(int num_prob) throws java.io.IOException{  
    int i,j, resp, mid;
    problems = new Problem [num_prob];
    String comp;

    int temp1, temp2, k, dt, ot;
    String temp_str;

     for(i=0;i<num_prob;i++) {
       problems[i] = new Problem();

       problems[i].name = str.getString();        // read name
       problems[i].setpath(str.getString());      // read path
       problems[i].description = str.getString(); // read description

       resp = str.server_in.readInt();   // read the number of inputs

       problems[i].setinput(resp);

       // for each input object...
       for(j=0;j< resp ;j++) {
         dt = str.server_in.readInt();
         ot = str.server_in.readInt();

         problems[i].inputs[j] = NetSolveObject.newObject(dt,ot);

         problems[i].inputs[j].setDataType();

         if(problems[i].inputs[j] instanceof NetSolveUPF)
           problems[i].upf++;

         problems[i].inputs[j].description = str.getString();
       }

       resp = str.server_in.readInt();  // read the number of outputs
       problems[i].setoutput(resp);

       // for each input object...
       for(j=0;j<problems[i].nb_output;j++) {
         dt = str.server_in.readInt();
         ot = str.server_in.readInt();

         problems[i].outputs[j] = NetSolveObject.newObject(dt,ot);         

         problems[i].outputs[j].setDataType();

         problems[i].outputs[j].description = str.getString();
       }

       comp = str.getString();  // read problem complexity

       mid = parseComplexity(comp);
   
       if(mid == -1) {
         // Could not parse the complexity string
         problems[i].complexity_a = 0;
         problems[i].complexity_b = 0;
         System.err.println("Warning: Trouble parsing complexity string");
       }
       else {
         try {
           problems[i].complexity_a = 
                new Integer(comp.substring(0,mid)).intValue();
           problems[i].complexity_b = 
                new Integer(comp.substring(mid+1)).intValue();
         }catch(NumberFormatException e) {
           System.err.println("Warning: Trouble parsing complexity string");
           problems[i].complexity_a = 0;
           problems[i].complexity_b = 0;
         }
       }

/***************************************************
 **** This stuff for detail level 4 is untested ****
 **** and probably doesn't work.                ****
 *                                                 *
 *     if(DETAIL == 4) {                           *
 *       resp = str.server_in.readInt();           *
 *       for(j=0;j<resp;j++) {                     *
 *         temp1 = str.server_in.readInt();        *
 *         for(k=0;k<temp1;k++) {                  *
 *           temp_str = str.getString();           *
 *         }                                       *
 *         temp1 = str.server_in.readInt();        *
 *         for(k=0;k<temp1;k++) {                  *
 *           temp_str = str.getString();           *
 *           temp2 = str.server_in.readInt();      *
 *         }                                       *
 *         temp1 = str.server_in.readInt();        *
 *         for(k=0;k<temp1;k++) {                  *
 *           temp_str = str.getString();           *
 *           temp_str = str.getString();           *
 *         }                                       *
 *       }                                         *
 *     }                                           *
 ***************************************************/

     }

     if(GlobalDefs.DEBUG)
       System.out.println("done.");
  }

  /**
   * 
   * This is where we call the sorting routine to order the problems.
   * In this case, we're using a Quicksort routine modified to sort
   * the problems.
   *
   * @see QSortAlgorithm
   **/

  private void sortProblems() {
    QSortAlgorithm qs;

    qs = new QSortAlgorithm();
    qs.sort(problems);
  }

  /**
   * 
   * This method prints all the information about every problem that
   * the agent returned.  This is used for debugging and isn't normally
   * called.
   *
   **/

  private void printProblems() {  
    int i,j;

     for(i=0;i<problems.length;i++) {
       System.out.println("Name:");
       System.out.println(problems[i].name);
  
       System.out.println("Description:");
       System.out.println(problems[i].description);

       System.out.println("UPFs:");
       System.out.println(problems[i].upf);

       System.out.println("Number of Inputs:");
       System.out.println(problems[i].nb_input);

       for(j=0;j<problems[i].nb_input;j++) {
         System.out.println("Input "+j+" Data Type:");
         System.out.println(problems[i].inputs[j].data_type + " - " +
          NetSolveObject.datatype2str(problems[i].inputs[j].data_type));
         System.out.println("Input "+j+" Object Type:");
         System.out.println(problems[i].inputs[j].object_type + " - " +
          NetSolveObject.objtype2str(problems[i].inputs[j].object_type));
         System.out.println("Input "+j+" Description:");
         System.out.println(problems[i].inputs[j].description);
       }

       System.out.println("Number of Outputs:");
       System.out.println(problems[i].nb_output);

       for(j=0;j<problems[i].nb_output;j++) {
         System.out.println("Output "+j+" Data Type:");
         System.out.println(problems[i].outputs[j].data_type + " - " +
          NetSolveObject.datatype2str(problems[i].outputs[j].data_type));
         System.out.println("Output "+j+" Object Type:");
         System.out.println(problems[i].outputs[j].object_type + " - " +
          NetSolveObject.objtype2str(problems[i].outputs[j].object_type));
         System.out.println("Output "+j+" Description:");
         System.out.println(problems[i].outputs[j].description);
       }
       
       System.out.println("Complexity:");
       System.out.println(problems[i].complexity_a + "," + 
                          problems[i].complexity_b);
     }
  }

  /**
   * 
   * This method parses the complexity string returned by the agent. 
   * The string should be a pair of integers separated by a comma.
   * Here we dont check that the string actually contains numbers,
   * just that it has some characters followed by a comma followed
   * by some more characters.
   * 
   * @param cstr The complexity string.
   *
   * @return The index of the beginning of the second number in 
   *   the string.
   *
   **/

  private int parseComplexity(String cstr) {
    int start=0, end=0;
 
    // if the length of the complexity string is less than 3, it
    // can't be correct (the correct form is "x,y").

    if(cstr.length() < 3) 
      return -1;

    while((end < cstr.length()) && (cstr.charAt(end) != ',')) 
      end++;  // skip until we hit a comma or the end of the string

    if((end == cstr.length()) || (start==end)) 
      return -1;  // no comma or no characters following the comma
    
    return end;
  }

  /**
   *
   * This is the constructor for this class.  Its purpose is to
   * contact the agent, retrieve the list of problems from the
   * agent, and sort the list.
   * 
   * @param hostname The agent to contact.
   *
   * @exception NetSolveException thrown when the agent can't be contacted
   *    or some other data transmission error occurs.
   *
   * @see NetSolveException
   **/

  GetProblems(String hostname) throws NetSolveException {
    short encoding;
    int num_problems;

    try {
      try {
        // always XDR
        str = new NetSolveStream(hostname,AG_PORT,DATA_XDR);
      }
      catch(IOException e) {
        if(GlobalDefs.DEBUG)
          System.out.println("Could not connect to server: "+e);
        throw new NetSolveException(e.getMessage());
      }

     if(GlobalDefs.DEBUG)
       System.out.println("Sending some data to the server..");

     str.server_out.writeInt(POLL_LIST_PB); // send request
     str.server_out.writeInt(DETAIL);       // send detail level
     str.flushOutput();

     if(GlobalDefs.DEBUG) {
       System.out.println("...done.");
       System.out.println("Response from server:");
     }

     encoding = str.server_in.readShort();
     str.initRecv(encoding);

     if(GlobalDefs.DEBUG)
       System.out.println("encoding = " + encoding);

     num_problems = str.server_in.readInt();

     if(GlobalDefs.DEBUG)
       System.out.println("num_problems = " + num_problems);

     readProblems(num_problems);

     if(GlobalDefs.DEBUG)
       System.out.println("Sorting...");

     sortProblems();

     if(GlobalDefs.DEBUG)
       System.out.println("done.");
    }
    catch(IOException e) {
      if(GlobalDefs.DEBUG)
        System.out.println("Data transfer error: "+e);
      throw new NetSolveException(e.getMessage());
    }
    finally {
      // attempt to shut down the connection
      try { 
        if (str != null)  
          if (str.s != null) 
            str.shutDown(); 
      }
      catch (IOException e2) { 
         ;  // NULL BODY
      }
    }
  }
} // end class GetProblems
