/**
* GetServers
*
* This class provides methods for retrieving a list of
* currently available servers from the agent.  This list
* includes the machine's domain name, IP address, and
* status information.
* 
* @version 1.1, 26 Jan 1998 
* @author Keith Seymour (seymour@cs.utk.edu)
**/

import java.io.IOException;

public class GetServers extends GlobalDefs {
  private NetSolveStream str;
  private final int DETAIL = 3;
  private ServerStatus servers[];
  private int longest_hostname = 0;
  private String blank_string = "                                        ";

  // Field widths for formatting the server list
  private int IP_ADDR_FIELD_WIDTH       = 17;
  private int HOST_STATUS_FIELD_WIDTH   = 13;
  private int SERVER_STATUS_FIELD_WIDTH = 15;
  private int PROBLEMS_FIELD_WIDTH      = 10;

  /**
   * 
   * This is the method that actually reads the information 
   * from the agent.  The socket connection to the agent is
   * assumed to exist at this point.
   * 
   * @param num_serv The number of servers on which to read info.
   *
   * @exception java.io.IOException Thrown when an error is 
   *    encountered reading from the socket (e.g., broken pipe).
   *
   * @see ServerStatus
   **/

  private void readServers(int num_serv) throws java.io.IOException{  
     int i;
     String temp;
     int temp2;
     
     servers = new ServerStatus[num_serv];

     // for each server...
     for(i=0;i<num_serv;i++) {
       servers[i] = new ServerStatus();
       servers[i].desc = str.getString();

       /*
        * Determine if this hostname has the most characters of
        * all the hostnames we've read so far.  This is used in
        * formatting the server list.
        */

       if(servers[i].desc.length() > longest_hostname)
         longest_hostname = servers[i].desc.length();

       servers[i].ip_addr = str.readIPaddr();

       str.server_in.readInt();
       str.server_in.readInt();

       servers[i].host_status = str.server_in.readInt();
       servers[i].server_status = str.server_in.readInt();

       str.server_in.readInt();
       str.server_in.readInt();
       str.server_in.readInt();

       servers[i].number_problems = str.server_in.readInt();
     }
  }

  /**
   *
   * This method right-justifies a string to a certain field
   * length.  This is used in formatting the server information
   * report.
   * 
   * @param str The string to right-justify.
   * @param length The length of the field.
   *
   * @return Returns the right-justified string.
   **/

  private String rightJust(String str, int length) {
    if(str.length() > length)
      return(str);
    else
      return(blank_string.substring(0,length - str.length()) + str);
  }

  /**
   *
   * This method uses the server information that was returned
   * by the agent to produce a formatted text table listing
   * all the domain names, IP addresses, etc. for each server.
   * 
   * @return Returns string containing the text table.
   *
   **/

  public String statusString() {
    StringBuffer temp = new StringBuffer(700);
    int i;

    // format the column titles
    temp.append(rightJust("Machine Name",longest_hostname));
    temp.append(rightJust("IP address",IP_ADDR_FIELD_WIDTH));
    temp.append(rightJust("Host Status",HOST_STATUS_FIELD_WIDTH));
    temp.append(rightJust("Server Status",SERVER_STATUS_FIELD_WIDTH));
    temp.append(rightJust("Problems",PROBLEMS_FIELD_WIDTH) + "\n");

    // the tildes are used to "underline" the column titles
    temp.append(rightJust("~~~~~~~~~~~~",longest_hostname));
    temp.append(rightJust("~~~~~~~~~~",IP_ADDR_FIELD_WIDTH));
    temp.append(rightJust("~~~~~~~~~~~",HOST_STATUS_FIELD_WIDTH));
    temp.append(rightJust("~~~~~~~~~~~~~",SERVER_STATUS_FIELD_WIDTH));
    temp.append(rightJust("~~~~~~~~",PROBLEMS_FIELD_WIDTH) + "\n");

    // for each server...
    for(i=0;i<servers.length;i++) {
      temp.append(rightJust(servers[i].desc,longest_hostname));
      temp.append(rightJust(
            (long)  ((servers[i].ip_addr >> 24) & 0xFF) + "." +
                    ((servers[i].ip_addr >> 16) & 0xFF) + "." +
                    ((servers[i].ip_addr >>  8) & 0xFF) + "." +
                     (servers[i].ip_addr & 0xFF),IP_ADDR_FIELD_WIDTH));
      switch(servers[i].host_status) {
        case 0:
          temp.append(rightJust("DOWN",HOST_STATUS_FIELD_WIDTH));
          break;
        case 1:
          temp.append(rightJust("UP",HOST_STATUS_FIELD_WIDTH));
          break;
        default:
          temp.append(rightJust("UNKNOWN",HOST_STATUS_FIELD_WIDTH));
      }
      switch(servers[i].server_status) {
        case 1:
          temp.append(rightJust("RUNNING",SERVER_STATUS_FIELD_WIDTH));
          break;
        case 0:
          temp.append(rightJust("STOPPED",SERVER_STATUS_FIELD_WIDTH));
          break;
        case -2:
          temp.append(rightJust("FAILED",SERVER_STATUS_FIELD_WIDTH));
          break;
        default:
          temp.append(rightJust("UNKNOWN",SERVER_STATUS_FIELD_WIDTH));
      }
      if(servers[i].number_problems == 0)
        temp.append(rightJust("Agent",PROBLEMS_FIELD_WIDTH) + "\n");
      else
        temp.append( rightJust(
           new Integer(servers[i].number_problems).toString(),PROBLEMS_FIELD_WIDTH) 
           + "\n");
    }
    return(temp.toString());
  }

  /**
   *
   * This method sorts the list of servers by IP address.
   * Sorting by IP address should group the servers by domain.
   * This is not a sophisticated sort, but the server lists
   * are expected to be very short.
   * 
   * @see #swap
   **/

  private void sortServers() {
    int j, k, min_index, min_ip, cur_ip;
 
    for (k=0;k<servers.length;k++) {
      min_index=k;
      min_ip = servers[k].ip_addr;
  
      for(j = k+1;j<servers.length;j++) {
        cur_ip = servers[j].ip_addr;

        if(min_ip > cur_ip)
        {
          min_ip = cur_ip;
          min_index = j;
        }
      }
      swap(servers,k,min_index);
    }

  }

  /**
   *
   * This is a method used by the sorting routine...it
   * swaps two server information entries.
   * 
   * @param serv The array of server information entries.
   * @param x Index of element to be swapped.
   * @param y Index of element to be swapped.
   *
   * @see #sortServers
   **/

  private void swap(ServerStatus serv[], int x, int y) {
    ServerStatus temp = serv[x];
    serv[x] = serv[y];
    serv[y] = temp;
  }

  /**
   *
   * This is the constructor for the GetServers class.  It
   * first attempts to make a socket connection to the agent.
   * It will then read the server information from the agent
   * and create a sorted table of server information.
   * 
   * @param hostname The host name of the agent.
   *
   * @exception NetSolveException This exception is thrown when
   *   some problem is encountered retrieving the server information
   *   (e.g., connection refused, etc.).
   **/

  GetServers(String hostname) throws NetSolveException {
    short encoding;
    int num_servers;

    try {
      try {
        // always XDR
        str = new NetSolveStream(hostname,AG_PORT,DATA_XDR);
      }
      catch(IOException e) {
        if(GlobalDefs.DEBUG)
          System.out.println("Could not connect to server: "+e);
        throw new NetSolveException(e.getMessage());
      }

     if(GlobalDefs.DEBUG)
       System.out.println("Sending some data to the server..");

     str.server_out.writeInt(POLL_LIST_SV); // send request
     str.server_out.writeInt(DETAIL);       // send detail level
     str.flushOutput();

     if(GlobalDefs.DEBUG) {
       System.out.println("...done.");
       System.out.println("Response from server:");
     }

     encoding = str.server_in.readShort();
     str.initRecv(encoding);

     if(GlobalDefs.DEBUG)
       System.out.println("encoding = " + encoding);

     num_servers = str.server_in.readInt();

     if(GlobalDefs.DEBUG)
       System.out.println("num_servers = " + num_servers);

     readServers(num_servers);

     if(GlobalDefs.DEBUG)
       System.out.println("Sorting...");

     sortServers();

     if(GlobalDefs.DEBUG)
       System.out.println("done.");
    }
    catch(IOException e) {
      if(GlobalDefs.DEBUG)
        System.out.println("Data transfer error: "+e);
      throw new NetSolveException(e.getMessage());
    }
    finally {
      // attempt to shut down the connection
      try { 
        if (str != null)  
          if (str.s != null) 
            str.shutDown(); 
      }
      catch (IOException e2) { 
        ;  // NULL BODY
      }
    }
  }
} // end class GetServers
