/**
* NetSolveByteMatrix
*   
* This class contains a byte matrix and methods for reading,
* writing, and parsing byte data.
* 
* @version 1.1, 8 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see NetSolveMatrix
**/

import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedReader;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedOutputStream;
import java.lang.Thread;
import java.util.Vector;
import java.util.Random;
import java.util.StringTokenizer;

public class NetSolveByteMatrix extends NetSolveMatrix {
  private byte [][] val;
  private Thread parentThread = null;
  private static byte [][] byte_array = {
     { 1,  2,  3,  4},
     { 5,  6,  7,  8},
     { 9, 10, 11, 12},
     {13, 14, 15, 16}
  };
  private static byte [][] rand_array = null;

  public String sampleData(boolean rand) {
    int i;

    if(rand) {
       Random rnd = new Random();

       if(rand_array == null)
         rand_array = new byte[RANDOM_SIZE][RANDOM_SIZE];
          
       for(i=0; i < RANDOM_SIZE; i++)
         rnd.nextBytes(rand_array[i]);

       return(byteMatrix2string(rand_array));
    } else {
       return(byteMatrix2string(byte_array));
    }
  }

  /**
   *
   *  This method returns the current value of the 
   * byte matrix.  The matrix should have been
   * initialized before calling this function (either
   * through readData or parseStream).
   *
   * @return The byte matrix.
   *
   * @see #readData
   * @see #parseStream
   *
   **/

  public byte [][] getval() { 
    return val; 
  }


  /**
   *
   *  This method prints the current value of the 
   * byte matrix to stdout.
   *
   **/

  public void printData() {
    int i,j;

    System.out.print(this.toString());
  }

  /**
   *
   *  This method sets the data type and object type
   * of this object.
   *
   **/

  public void setDataType() {
    data_type = NETSOLVE_B;
    object_type = NETSOLVE_MATRIX;
  }

  /**
   *
   *  This method returns a String representation of
   * the byte matrix.
   *
   * @return String representation of the matrix.
   *
   **/

  public String toString() {
    if(order == GlobalDefs.ROW_MAJOR)
      return(byteMatrix2string(val));
    else
      return(colMajorByteMatrix2string(val));
  }

  private String byteMatrix2string(byte [][]mat) {
    StringBuffer tmp = new StringBuffer(100);
    int i,j;

    if(mat.length == 0) 
      return("Empty result");

    for(i = 0; i < mat.length; i++) {
      for(j = 0; j < mat[i].length; j++)
        tmp.append((int)(mat[i][j] & 0xFF) + " ");
      tmp.append("\n");
    }

    return(tmp.toString());
  }

  private String colMajorByteMatrix2string(byte [][] mat) {
    StringBuffer tmp = new StringBuffer(100);
    int i,j;

    if(mat.length == 0) 
      return("Empty result");

    try {
      for( i=0 ; i < mat[0].length ; i++ ) {
        for( j=0 ; j < mat.length ; j++ )
          tmp.append((int)(mat[j][i] & 0xFF) + " ");
        tmp.append("\n");
      }
    }catch(NullPointerException e) {
      return("Bad matrix");
    }

    return(tmp.toString());
  }

  public NetSolveByteMatrix () 
  {
    super();
  }

  /**
   *
   *  This is the constructor for the NetSolveByteMatrix
   * class.  Given a stream of text, this constructor 
   * attempts to create a byte matrix.  If the
   * text is invalid (e.g., uneven rows, invalid numeric
   * format, etc.), a NetSolveException is thrown.
   *
   * @param in A stream containing the text representation
   *   of the matrix.
   * @param p The parent thread.  This is passed in so
   *   that we may call yield() and allow other threads
   *   a chance to run.
   *
   * @exception NetSolveException is thrown when the text
   *   stream does not parse into a byte matrix.
   *
   **/

  public NetSolveByteMatrix (InputStream in, Thread p) 
   throws NetSolveException 
  {
    initialize(in,p);
  }

  public void initialize(InputStream in, Thread p) throws NetSolveException
  {
    StringBuffer err = new StringBuffer(80);
    BufferedReader s = new BufferedReader(new InputStreamReader(in));
    Vector vec = null;

    setDataType();

    cur_lang = Labels.language;
    parentThread = p;

    try {
      vec = parseStream(s,parentThread,err);
    }catch(IOException e) {
      System.err.println("IOException in NetSolveByteMatrix()");
    }

    if(vec == null)
      throw new NetSolveException(err.toString());
  
    val = new byte[vec.size()][];
    for(int i = 0; i < vec.size(); i++)
      val[i] = (byte []) (vec.elementAt(i));
  }
  
  public NetSolveByteMatrix ( byte [][] mat ) throws NetSolveException {
    super();
    initialize(mat);
  }

  public void initialize ( byte [][] mat ) throws NetSolveException {
    checkDims(mat);
    val = mat;
    setDims(val.length,val[0].length,0);
    setDataType();
  }

  public void checkDims(byte [][] mat) throws NetSolveException {
    int i, prev;

    try {
      prev = mat[0].length;

      for(i = 0; i < mat.length; i++) {
        if(mat[i].length != prev)
          throw new NetSolveException("Rows of matrix are uneven");
      }
    }catch(NullPointerException e) {
      throw new NetSolveException(
        "Badly formed matrix (null or containing null element)");
    }
  }

  /**
   *
   *  This method returns the number of byte elements
   * in a 'line' of input.  Actually, the input is an array
   * representation of the line of input.
   *
   * @param item An array of bytes, cast to Object, so
   *  that we may call this from NetSolveMatrix, the
   *  parent class, without knowing the data type.
   *
   * @return The number of bytes in the array.
   *
   **/

  public int lineLength(Object item)
  {
    return( ((byte [])item).length );
  }

  /**
   *
   *  This method parses a line of text into an array of
   * bytes.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of bytes, cast to Object.
   *
   **/

  public Object parseLine(String line, StringBuffer err) {
    return ByteParser.parseLine(line,err);
  }

  /**
   *
   *  This method reads data from the specified stream
   * and stores it into the byte matrix, accessible
   * through the getval method.
   *
   * @param s The stream from which the data should be read.
   *
   * @exception IOException is thrown upon some error reading
   *   from the stream.
   *
   * @see #getval
   *
   **/

  public void readData ( NetSolveStream s ) throws IOException {
    int j, k, cur_encoding;

    cur_encoding = s.getEncoding();

    readMatrixHeader(s);     //  read data type, dimensions, etc

    val = new byte [m][n];

    if(cur_encoding == GlobalDefs.DATA_XDR) {
      for(j=0;j<m;j++) {
        parentThread.yield();
        for(k=0;k<n;k++)
          val[j][k] = s.server_in.readByte();
        for(int kk=0;kk < s.padXDR(n); kk++)
          s.server_in.readByte();
      }
    } else {
      for(j=0;j<m;j++) {
        parentThread.yield();
        for(k=0;k<n;k++)
          val[j][k] = s.server_in.readByte();
      }
    }
  }

  /**
   *
   *  This method writes the byte matrix to the specified 
   * stream.
   *
   * @param s The stream to which the data should be written.
   *
   * @exception IOException is thrown upon some error writing
   *   to the stream.
   *
   **/

  public void writeData ( NetSolveStream s ) throws IOException {
    int j, k, cur_encoding;

    cur_encoding = s.getEncoding();
 
    writeMatrixHeader(s);    //  send data type, dimensions, etc

    if(cur_encoding == GlobalDefs.DATA_XDR) {
      for(j=0;j< m ; j++) {
        parentThread.yield();

        if(GlobalDefs.DEBUG)
          for(int f=0; f < n; f++)
            System.out.println("sending byte: " + val[j][f]);

        s.server_out.write(val[j], 0, n);

        for(int kk=0;kk < s.padXDR(n); kk++)
          s.server_out.writeByte(0);
      }
    }
    else {
      for(j=0;j< m ; j++) {
        parentThread.yield();

        if(GlobalDefs.DEBUG)
          for(int f=0; f < n; f++)
            System.out.println("sending byte: " + val[j][f]);

        s.server_out.write(val[j], 0, n);
      }
    }
  }
  
  public void copy (NetSolveByteMatrix obj) {
    super.copy(obj);
    val = obj.val;
    parentThread = obj.parentThread;
  }
}
