/**
* NetSolveByteVector
*   
* This class contains a byte vector and methods for reading,
* writing, and parsing byte data.
* 
* @version 1.1, 8 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see NetSolveVector
**/

import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedReader;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedOutputStream;
import java.lang.Thread;
import java.util.Vector;
import java.util.Random;
import java.util.StringTokenizer;

public class NetSolveByteVector extends NetSolveVector {
  private byte [] val;
  private Thread parentThread = null;
  private static byte [] byte_array = { 1, 2, 3, 4};
  private static byte [] rand_array = null;

  public String sampleData(boolean rand) {
    int i;

    if(rand) {
       Random rnd = new Random();

       if(rand_array == null)
         rand_array = new byte[RANDOM_SIZE];

       rnd.nextBytes(rand_array);

       return(byteVector2string(rand_array));
    } else {
       return(byteVector2string(byte_array));
    }
  }

  /**
   *
   *  This method returns the current value of the 
   * byte vector.  The vector should have been
   * initialized before calling this function (either
   * through readData or parseStream).
   *
   * @return The byte vector.
   *
   * @see #readData
   * @see #parseStream
   *
   **/

  public byte [] getval() { 
    return val; 
  }


  /**
   *
   *  This method prints the current value of the 
   * byte vector to stdout.
   *
   **/

  public void printData() {
    System.out.print(this.toString());
  }

  /**
   *
   *  This method sets the data type and object type
   * of this object.
   *
   **/

  public void setDataType() {
    data_type = NETSOLVE_B;
    object_type = NETSOLVE_VECTOR;
  }

  /**
   *
   *  This method returns a String representation of
   * the byte vector.
   *
   * @return String representation of the vector.
   *
   **/

  public String toString() {
    return byteVector2string(val);
  }

  private String byteVector2string(byte [] vec) {
    StringBuffer tmp = new StringBuffer(100);
    int i;

    if(vec.length == 0) 
      return("Empty result");

    for(i = 0; i < vec.length; i++)
      tmp.append((int)(vec[i] & 0xFF) + "\n");

    return(tmp.toString());
  }

  public NetSolveByteVector () 
  {
    super();
  }

  /**
   *
   *  This is the constructor for the NetSolveByteVector
   * class.  Given a stream of text, this constructor 
   * attempts to create a byte vector.  If the
   * text is invalid (e.g., uneven rows, invalid numeric
   * format, etc.), a NetSolveException is thrown.
   *
   * @param in A stream containing the text representation
   *   of the vector.
   * @param p The parent thread.  This is passed in so
   *   that we may call yield() and allow other threads
   *   a chance to run.
   *
   * @exception NetSolveException is thrown when the text
   *   stream does not parse into a byte vector.
   *
   **/

  public NetSolveByteVector (InputStream in, Thread p) 
   throws NetSolveException 
  {
    initialize(in,p);
  }

  public NetSolveByteVector (byte []vec)throws NetSolveException  
  {
    super();
    initialize(vec);
  }

  public void initialize ( byte [] vec ) throws NetSolveException {
    if(vec == null)
      throw new NetSolveException("Null vector");

    val = vec;
    setDims(0,0,val.length);
    setDataType();
  }

  public void initialize(InputStream in, Thread p) throws NetSolveException
  {
    StringBuffer err = new StringBuffer(80);
    BufferedReader s = new BufferedReader(new InputStreamReader(in));
    Vector vec = null;
    byte [] tmp_arr;
    int idx;

    setDataType();

    cur_lang = Labels.language;
    parentThread = p;

    try {
      vec = parseStream(s,parentThread,err);
    }catch(IOException e) {
      System.err.println("IOException in NetSolveByteVector()");
    }

    if(vec == null)
      throw new NetSolveException(err.toString());
  
    idx = 0;
    for(int i = 0; i < vec.size(); i++)
      idx += ((byte []) (vec.elementAt(i))).length;

    val = new byte[idx];

    idx = 0;
    for(int i = 0; i < vec.size(); i++) {
      tmp_arr = (byte []) (vec.elementAt(i));
      System.arraycopy(tmp_arr, 0, val, idx, tmp_arr.length);
      idx += tmp_arr.length;
    }
  }
  
  /**
   *
   *  This method returns the number of byte elements
   * in a 'line' of input.  Actually, the input is an array
   * representation of the line of input.
   *
   * @param item An array of bytes, cast to Object, so
   *  that we may call this from NetSolveVector, the
   *  parent class, without knowing the data type.
   *
   * @return The number of byte elements in the array.
   *
   **/

  public int lineLength(Object item)
  {
    return( ((byte [])item).length );
  }

  /**
   *
   *  This method parses a line of text into an array of
   * bytes.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of bytes, cast to Object.
   *
   **/

  public Object parseLine(String line, StringBuffer err) {
    return ByteParser.parseLine(line,err);
  }

  /**
   *
   *  This method reads data from the specified stream
   * and stores it into the byte vector, accessible
   * through the getval method.
   *
   * @param s The stream from which the data should be read.
   *
   * @exception IOException is thrown upon some error reading
   *   from the stream.
   *
   * @see #getval
   *
   **/

  public void readData ( NetSolveStream s ) throws IOException {
    int j;

    readVectorHeader(s);     //  read data type, dimensions, etc

    val = new byte[n];

    for(j=0;j<n;j++)
      val[j] = s.server_in.readByte();

    if(s.getEncoding() == GlobalDefs.DATA_XDR) {
      for(int kk = 0; kk < s.padXDR(n); kk++)
        s.server_in.readByte();   // read the padding bytes
    }
  }

  /**
   *
   *  This method writes the byte vector to the specified 
   * stream.
   *
   * @param s The stream to which the data should be written.
   *
   * @exception IOException is thrown upon some error writing
   *   to the stream.
   *
   **/

  public void writeData ( NetSolveStream s ) throws IOException {
    int j,k;
 
    writeVectorHeader(s);    //  send data type, dimensions, etc

    s.server_out.write(val, 0, val.length);

    if(s.getEncoding() == GlobalDefs.DATA_XDR) {
      for(int kk = 0; kk < s.padXDR(n); kk++)
        s.server_out.writeByte(0);   // write the padding bytes
    }

    if(GlobalDefs.DEBUG)
    {
      System.out.print("sending byte vector: ");
      for(k=0;k< val.length;k++)
        System.out.print(val[k] + " ");
      System.out.println();
    }
  }

  public void copy (NetSolveByteVector obj) {
    super.copy(obj);
    val = obj.val;
    parentThread = obj.parentThread;
  }
}
