/**
* NetSolveCharMatrix
*   
* This class contains a character matrix and methods for reading,
* writing, and parsing character data.
* 
* @version 1.1, 8 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see NetSolveMatrix
**/

import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedReader;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedOutputStream;
import java.lang.Thread;
import java.util.Vector;
import java.util.Random;
import java.util.StringTokenizer;

public class NetSolveCharMatrix extends NetSolveMatrix {
  private char [][] val;
  private Thread parentThread = null;
  private static char [][] char_array = {
     { 'A', 'B', 'C', 'D'},
     { 'E', 'F', 'G', 'H'},
     { 'I', 'J', 'K', 'L'},
     { 'M', 'N', 'O', 'P'}
  };
  private static char [][] rand_array = null;


  public String sampleData(boolean rand) {
    int i,j;

    if(rand) {
       Random rnd = new Random();

       if(rand_array == null)
         rand_array = new char[RANDOM_SIZE][RANDOM_SIZE];

       for(i=0; i < RANDOM_SIZE; i++)
         for(j=0; j < RANDOM_SIZE; j++)
           rand_array[i][j] = (char) ((rnd.nextInt() % 13) + 12 + 'A');

       return(charMatrix2string(rand_array));
    } else {
       return(charMatrix2string(char_array));
    }
  }

  /**
   *
   *  This method returns the current value of the 
   * character matrix.  The matrix should have been
   * initialized before calling this function (either
   * through readData or parseStream).
   *
   * @return The character matrix.
   *
   * @see #readData
   * @see #parseStream
   *
   **/

  public char [][] getval() { 
    return val; 
  }


  /**
   *
   *  This method prints the current value of the 
   * character matrix to stdout.
   *
   **/

  public void printData() {
    System.out.print(this.toString());
  }

  /**
   *
   *  This method sets the data type and object type
   * of this object.
   *
   **/

  public void setDataType() {
    data_type = NETSOLVE_CHAR;
    object_type = NETSOLVE_MATRIX;
  }

  /**
   *
   *  This method returns a String representation of
   * the character matrix.
   *
   * @return String representation of the matrix.
   *
   **/

  public String toString() {
    if(order == GlobalDefs.ROW_MAJOR)
      return(charMatrix2string(val));
    else
      return(colMajorCharMatrix2string(val));
  }

  private String charMatrix2string(char [][] mat) {
    StringBuffer tmp = new StringBuffer(100);
    int i,j;

    if(mat.length == 0) 
      return("Empty result");

    for(i = 0; i < mat.length; i++) {
      for(j = 0; j < mat[i].length; j++)
        tmp.append(mat[i][j] + " ");
      tmp.append("\n");
    }

    return(tmp.toString());
  }

  private String colMajorCharMatrix2string(char [][] mat) {
    StringBuffer tmp = new StringBuffer(100);
    int i,j;

    if(mat.length == 0)
      return("Empty result");

    try {
      for( i=0 ; i < mat[0].length ; i++ ) {
        for( j=0 ; j < mat.length ; j++ )
          tmp.append(mat[j][i] + " ");
        tmp.append("\n");
      }
    }catch(NullPointerException e) {
      return("Bad matrix");
    }

    return(tmp.toString());
  }

  public NetSolveCharMatrix () 
  {
    super();
  }

  /**
   *
   *  This is the constructor for the NetSolveCharMatrix
   * class.  Given a stream of text, this constructor 
   * attempts to create a character matrix.  If the
   * text is invalid (e.g., uneven rows, invalid numeric
   * format, etc.), a NetSolveException is thrown.
   *
   * @param in A stream containing the text representation
   *   of the matrix.
   * @param p The parent thread.  This is passed in so
   *   that we may call yield() and allow other threads
   *   a chance to run.
   *
   * @exception NetSolveException is thrown when the text
   *   stream does not parse into a character matrix.
   *
   **/

  public NetSolveCharMatrix (InputStream in, Thread p) 
   throws NetSolveException 
  {
    initialize(in,p);
  }

  public NetSolveCharMatrix (char [][] mat) throws NetSolveException  
  {
    super();
    initialize(mat);
  }

  public void initialize ( char [][] mat ) throws NetSolveException {
    checkDims(mat);
    val = mat;
    setDims(val.length,val[0].length,0);
    setDataType();
  }

  public void checkDims(char [][] mat) throws NetSolveException {
    int i, prev;

    try {
      prev = mat[0].length;

      for(i = 0; i < mat.length; i++) {
        if(mat[i].length != prev)
          throw new NetSolveException("Rows of matrix are uneven");
      }
    }catch(NullPointerException e) {
      throw new NetSolveException(
        "Badly formed matrix (null or containing null element)");
    }
  }

  public void initialize(InputStream in, Thread p) throws NetSolveException
  {
    StringBuffer err = new StringBuffer(80);
    BufferedReader s = new BufferedReader(new InputStreamReader(in));
    Vector vec = null;

    setDataType();

    cur_lang = Labels.language;
    parentThread = p;

    try {
      vec = parseStream(s,parentThread,err);
    }catch(IOException e) {
      System.err.println("IOException in NetSolveCharMatrix()");
    }

    if(vec == null)
      throw new NetSolveException(err.toString());
  
    val = new char[vec.size()][];
    for(int i = 0; i < vec.size(); i++)
      val[i] = (char []) (vec.elementAt(i));
  }
  
  /**
   *
   *  This method returns the number of character elements
   * in a 'line' of input.  Actually, the input is an array
   * representation of the line of input.
   *
   * @param item An array of characters, cast to Object, so
   *  that we may call this from NetSolveMatrix, the
   *  parent class, without knowing the data type.
   *
   * @return The number of characters in the array.
   *
   **/

  public int lineLength(Object item)
  {
    return( ((char [])item).length );
  }

  /**
   *
   *  This method parses a line of text into an array of
   * characters.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of characters, cast to Object.
   *
   **/

  public Object parseLine(String line, StringBuffer err) {
    return CharParser.parseLine(line,err);
  }

  /**
   *
   *  This method reads data from the specified stream
   * and stores it into the character matrix, accessible
   * through the getval method.
   *
   * @param s The stream from which the data should be read.
   *
   * @exception IOException is thrown upon some error reading
   *   from the stream.
   *
   * @see #getval
   *
   **/

  public void readData ( NetSolveStream s ) throws IOException {
    int j, k, cur_encoding;

    cur_encoding = s.getEncoding();

    readMatrixHeader(s);     //  read data type, dimensions, etc

    val = new char [m][n];

    for(j=0;j<m;j++) {
      parentThread.yield();
      for(k=0;k<n;k++)
        if(cur_encoding == GlobalDefs.DATA_XDR)
          val[j][k] = (char) s.server_in.readInt();
        else
          val[j][k] = (char) s.server_in.readByte();
    }
  }

  /**
   *
   *  This method writes the character matrix to the specified 
   * stream.
   *
   * @param s The stream to which the data should be written.
   *
   * @exception IOException is thrown upon some error writing
   *   to the stream.
   *
   **/

  public void writeData ( NetSolveStream s ) throws IOException {
    int j, k, cur_encoding;
 
    cur_encoding = s.getEncoding();

    writeMatrixHeader(s);    //  send data type, dimensions, etc

    for(j=0;j< m; j++) {
      parentThread.yield();

      if(cur_encoding == GlobalDefs.DATA_XDR) {
        for(k=0;k< n;k++) {
          s.server_out.writeInt( (int) val[j][k]);
          if(GlobalDefs.DEBUG)
            System.out.println("sending XDR-encoded char "+ val[j][k]);
        }
      }
      else {
        if(GlobalDefs.DEBUG)
          for(k=0;k< n;k++)
            System.out.println("sending char "+ val[j][k]);

        s.server_out.writeBytes(new String(val[j]));
      }
    }
  }

  public void copy (NetSolveCharMatrix obj) {
    super.copy(obj);
    val = obj.val;
    parentThread = obj.parentThread;
  }
}
