/**
* NetSolveCharVector
*   
* This class contains a character vector and methods for reading,
* writing, and parsing character data.
* 
* @version 1.1, 8 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see NetSolveVector
**/

import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedReader;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedOutputStream;
import java.lang.Thread;
import java.util.Vector;
import java.util.Random;
import java.util.StringTokenizer;

public class NetSolveCharVector extends NetSolveVector {
  protected char [] val;
  private Thread parentThread = null;
  private static char [] char_array = { 'A', 'B', 'C', 'D'};
  private static char [] rand_array = null;

  public String sampleData(boolean rand) {
    int i;

    if(rand) {
       Random rnd = new Random();

       if(rand_array == null)
         rand_array = new char[RANDOM_SIZE];

       for(i=0; i < RANDOM_SIZE; i++)
         rand_array[i] = (char)((rnd.nextInt() % 13) + 12 + 'A');

       return(charVector2string(rand_array));
    } else {
       return(charVector2string(char_array));
    }
  }

  /**
   *
   *  This method returns the current value of the 
   * character vector.  The vector should have been
   * initialized before calling this function (either
   * through readData or parseStream).
   *
   * @return The character vector.
   *
   * @see #readData
   * @see #parseStream
   *
   **/

  public char [] getval() { 
    return val; 
  }


  /**
   *
   *  This method prints the current value of the 
   * character vector to stdout.
   *
   **/

  public void printData() {
    System.out.print(this.toString());
  }

  /**
   *
   *  This method sets the data type and object type
   * of this object.
   *
   **/

  public void setDataType() {
    data_type = NETSOLVE_CHAR;
    object_type = NETSOLVE_VECTOR;
  }

  /**
   *
   *  This method returns a String representation of
   * the character vector.
   *
   * @return String representation of the vector.
   *
   **/

  public String toString() {
    return charVector2string(val);
  }

  private String charVector2string(char [] vec) {
    StringBuffer tmp = new StringBuffer(100);
    int i;

    if(vec.length == 0) 
      return("Empty result");

    for(i = 0; i < vec.length; i++)
      tmp.append(vec[i] + "\n");

    return(tmp.toString());
  }

  public NetSolveCharVector () 
  {
    super();
  }

  /**
   *
   *  This is the constructor for the NetSolveCharVector
   * class.  Given a stream of text, this constructor 
   * attempts to create a character vector.  If the
   * text is invalid (e.g., uneven rows, invalid numeric
   * format, etc.), a NetSolveException is thrown.
   *
   * @param in A stream containing the text representation
   *   of the vector.
   * @param p The parent thread.  This is passed in so
   *   that we may call yield() and allow other threads
   *   a chance to run.
   *
   * @exception NetSolveException is thrown when the text
   *   stream does not parse into a character vector.
   *
   **/

  public NetSolveCharVector (InputStream in, Thread p) 
   throws NetSolveException 
  {
    initialize(in,p);
  }

  public NetSolveCharVector (char [] vec)throws NetSolveException  
  {
    super();
    initialize(vec);
  }

  public void initialize ( char [] vec ) throws NetSolveException {
    if(vec == null)
      throw new NetSolveException("Null vector");

    val = vec;
    setDims(0,0,val.length);
    setDataType();
  }

  public void initialize(InputStream in, Thread p) throws NetSolveException
  {
    StringBuffer err = new StringBuffer(80);
    BufferedReader s = new BufferedReader(new InputStreamReader(in));
    Vector vec = null;
    char [] tmp_arr;
    int idx;

    setDataType();

    cur_lang = Labels.language;
    parentThread = p;

    try {
      vec = parseStream(s,parentThread,err);
    }catch(IOException e) {
      System.err.println("IOException in NetSolveCharVector()");
    }

    if(vec == null)
      throw new NetSolveException(err.toString());
  
    idx = 0;
    for(int i = 0; i < vec.size(); i++)
      idx += ((char []) (vec.elementAt(i))).length;

    val = new char[idx];

    idx = 0;
    for(int i = 0; i < vec.size(); i++) {
      tmp_arr = (char []) (vec.elementAt(i));
      System.arraycopy(tmp_arr, 0, val, idx, tmp_arr.length);
      idx += tmp_arr.length;
    }
  }
  
  /**
   *
   *  This method returns the number of character elements
   * in a 'line' of input.  Actually, the input is an array
   * representation of the line of input.
   *
   * @param item An array of characters, cast to Object, so
   *  that we may call this from NetSolveVector, the
   *  parent class, without knowing the data type.
   *
   * @return The number of characters in the array.
   *
   **/

  public int lineLength(Object item)
  {
    return( ((char [])item).length );
  }

  /**
   *
   *  This method parses a line of text into an array of
   * characters.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of characters, cast to Object.
   *
   **/

  public Object parseLine(String line, StringBuffer err) {
    return CharParser.parseLine(line,err);
  }

  /**
   *
   *  This method reads data from the specified stream
   * and stores it into the character vector, accessible
   * through the getval method.
   *
   * @param s The stream from which the data should be read.
   *
   * @exception IOException is thrown upon some error reading
   *   from the stream.
   *
   * @see #getval
   *
   **/

  public void readData ( NetSolveStream s ) throws IOException {
    int j;

    readVectorHeader(s);     //  read data type, dimensions, etc

    val = new char[n];

    for(j=0;j<val.length;j++)
      if(s.getEncoding() == GlobalDefs.DATA_XDR)
        val[j] = (char)s.server_in.readInt();
      else
        val[j] = (char)s.server_in.readByte();
  }

  /**
   *
   *  This method writes the character vector to the specified 
   * stream.
   *
   * @param s The stream to which the data should be written.
   *
   * @exception IOException is thrown upon some error writing
   *   to the stream.
   *
   **/

  public void writeData ( NetSolveStream s ) throws IOException {
    int j, cur_encoding;
 
    cur_encoding = s.getEncoding();

    writeVectorHeader(s);    //  send data type, dimensions, etc

    if(cur_encoding == GlobalDefs.DATA_XDR) {
      for(j=0;j<val.length;j++) {
        if(GlobalDefs.DEBUG)
          System.out.println( "sending XDR-encoded char: " + val[j]);

        s.server_out.writeInt( (int) val[j]);
      }
    }
    else {
      if(GlobalDefs.DEBUG)
        for(int k = 0; k < val.length; k++)
          System.out.println("sending char: " + val[k]);

      s.server_out.writeBytes(new String(val));
    }
  }

  public void copy (NetSolveCharVector obj) {
    super.copy(obj);
    val = obj.val;
    parentThread = obj.parentThread;
  }
}
