/**
* NetSolveDcomplexMatrix
*   
* This class contains a double precision complex  matrix and methods for reading,
* writing, and parsing double precision complex  data.
* 
* @version 1.1, 8 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see NetSolveMatrix
**/

import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedReader;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedOutputStream;
import java.lang.Thread;
import java.util.Vector;
import java.util.Random;
import java.util.StringTokenizer;

public class NetSolveDcomplexMatrix extends NetSolveMatrix {
  private double [][] val;
  private Thread parentThread = null;
  private static double [][] dcomp_array = {
     {1,0, 2,0, 3,0, 4,0},
     {5,0, 6,0, 7,0, 8,0},
     {9,0, 10,0, 11,0, 12,0},
     {13,0, 14,0, 15,0, 16,0}
  };

  private static double [][] rand_array = null;

  public String sampleData(boolean rand) {
    int i,j;

    if(rand) {
       Random rnd = new Random();

       if(rand_array == null)
         rand_array = new double[RANDOM_SIZE][RANDOM_SIZE*2];

       for(i=0; i < RANDOM_SIZE; i++)
         for(j=0; j < RANDOM_SIZE*2; j++)
           rand_array[i][j] = rnd.nextDouble() * 10.0 - 5.0;

       return(dcomplexMatrix2string(rand_array));
    } else {
       return(dcomplexMatrix2string(dcomp_array));
    }
  }

  /**
   *
   *  This method returns the current value of the 
   * double precision complex matrix.  The matrix should have been
   * initialized before calling this function (either
   * through readData or parseStream).
   *
   * @return The double precision complex matrix.
   *
   * @see #readData
   * @see #parseStream
   *
   **/

  public double [][] getval() { 
    return val; 
  }


  /**
   *
   *  This method prints the current value of the 
   * double precision complex matrix to stdout.
   *
   **/

  public void printData() {
    System.out.print(this.toString());
  }

  /**
   *
   *  This method sets the data type and object type
   * of this object.
   *
   **/

  public void setDataType() {
    data_type = NETSOLVE_Z;
    object_type = NETSOLVE_MATRIX;
  }

  /**
   *
   *  This method returns a String representation of
   * the double precision complex matrix.
   *
   * @return String representation of the matrix.
   *
   **/

  public String toString() {
    if(order == GlobalDefs.ROW_MAJOR)
      return dcomplexMatrix2string(val);
    else
      return colMajorDcomplexMatrix2string(val);
  }

  private String dcomplexMatrix2string(double [][] mat) {
    StringBuffer tmp = new StringBuffer(100);
    int kk,mm;

    if(mat.length == 0)
      return("Empty result");

    for(kk=0;kk< mat.length;kk++) {
      for(mm=0;mm < mat[kk].length;mm+=2){
        if(GlobalDefs.DEBUG) {
          System.out.print(String.valueOf( mat[kk][mm]));
          System.out.print(String.valueOf( mat[kk][mm+1]));
        }

        if(mat[kk][mm+1] >= 0.0) {
          tmp.append(String.valueOf( mat[kk][mm]) + " + ");
          tmp.append(String.valueOf( mat[kk][mm+1]));
        }
        else {
          tmp.append(String.valueOf( mat[kk][mm]) + " ");
          tmp.append(String.valueOf( mat[kk][mm+1]));
        }
        tmp.append("i");
        tmp.append("\t");
      }
      tmp.append("\n");

      if(GlobalDefs.DEBUG)
        System.out.println();
    }

    return(tmp.toString());
  }

  private String colMajorDcomplexMatrix2string(double [][] mat) {
    StringBuffer tmp = new StringBuffer(100);
    int i,j;

    if(mat.length == 0)
      return("Empty result");

    try {
      for( i=0 ; i < mat[0].length ; i+=2 ) {
        for( j=0 ; j < mat.length ; j++ ) {
          if(mat[j][i+1] >= 0.0)
            tmp.append(mat[j][i] + " + " + mat[j][i+1]);
          else
            tmp.append(mat[j][i] + " " + mat[j][i+1]);
          tmp.append("i ");
        }

        tmp.append("\n");
      }
    }catch(NullPointerException e) {
      return("Bad matrix");
    }

    return(tmp.toString());
  }

  public NetSolveDcomplexMatrix () 
  {
    super();
  }

  /**
   *
   *  This is the constructor for the NetSolveDcomplexMatrix
   * class.  Given a stream of text, this constructor 
   * attempts to create a double precision complex matrix.  If the
   * text is invalid (e.g., uneven rows, invalid numeric
   * format, etc.), a NetSolveException is thrown.
   *
   * @param in A stream containing the text representation
   *   of the matrix.
   * @param p The parent thread.  This is passed in so
   *   that we may call yield() and allow other threads
   *   a chance to run.
   *
   * @exception NetSolveException is thrown when the text
   *   stream does not parse into an integer matrix.
   *
   **/

  public NetSolveDcomplexMatrix (InputStream in, Thread p) 
   throws NetSolveException 
  {
    initialize(in,p);
  }

  public NetSolveDcomplexMatrix (double [][] mat)throws NetSolveException  
  {
    super();
    initialize(mat);
  }

  public void initialize ( double [][] mat ) throws NetSolveException {
    checkDims(mat);
    val = mat;
    setDims(val.length,val[0].length / 2,0);
    setDataType();
  }

  public void checkDims(double [][] mat) throws NetSolveException {
    int i, prev;

    try {
      prev = mat[0].length;

      for(i = 0; i < mat.length; i++) {
        if(mat[i].length != prev)
          throw new NetSolveException("Rows of matrix are uneven");

        if((mat[i].length % 2) != 0)
          throw new NetSolveException("Badly formed complex matrix");

      }
    }catch(NullPointerException e) {
      throw new NetSolveException(
        "Badly formed matrix (null or containing null element)");
    }
  }

  public void initialize(InputStream in, Thread p) throws NetSolveException
  {
    StringBuffer err = new StringBuffer(80);
    BufferedReader s = new BufferedReader(new InputStreamReader(in));
    Vector vec = null;

    setDataType();

    cur_lang = Labels.language;
    parentThread = p;

    try {
      vec = parseStream(s,parentThread,err);
    }catch(IOException e) {
      System.err.println("IOException in NetSolveDcomplexMatrix()");
    }

    if(vec == null)
      throw new NetSolveException(err.toString());
  
    val = new double[vec.size()][];
    for(int i = 0; i < vec.size(); i++)
      val[i] = (double []) (vec.elementAt(i));
  }
  
  /**
   *
   *  This method returns the number of double precision complex elements
   * in a 'line' of input.  Actually, the input is an array
   * representation of the line of input.
   *
   * @param item An array of double precision complex numbers, cast to 
   *   Object, so that we may call this from NetSolveMatrix, the
   *   parent class, without knowing the data type.
   *
   * @return The number of double precision complex numbers in the array.
   *
   **/

  public int lineLength(Object item)
  {
    return( ((double [])item).length / 2 );
  }

  /**
   *
   *  This method parses a line of text into an array of
   * double precision complex numbers.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of double precision complex numbers, cast to Object.
   *
   **/

  public Object parseLine(String line, StringBuffer err) {
    return DcomplexParser.parseLine(line,err);
  }

  /**
   *
   *  This method reads data from the specified stream and stores 
   * it into the double precision complex matrix, accessible
   * through the getval method.
   *
   * @param s The stream from which the data should be read.
   *
   * @exception IOException is thrown upon some error reading
   *   from the stream.
   *
   * @see #getval
   *
   **/

  public void readData ( NetSolveStream s ) throws IOException {
    int j,k;

    readMatrixHeader(s);     //  read data type, dimensions, etc

    val = new double [m][n*2];

    for(j=0;j<m;j++) {
      parentThread.yield();
      for(k=0;k<n*2;k+=2) {
        val[j][k] = s.server_in.readDouble();
        val[j][k+1] = s.server_in.readDouble();
      }
    }
  }

  /**
   *
   *  This method writes the double precision complex matrix to 
   * the specified stream.
   *
   * @param s The stream to which the data should be written.
   *
   * @exception IOException is thrown upon some error writing
   *   to the stream.
   *
   **/

  public void writeData ( NetSolveStream s ) throws IOException {
    int j,k;
 
    writeMatrixHeader(s);    //  send data type, dimensions, etc

    for(j=0;j< m; j++) {
      parentThread.yield();
      for(k=0;k< n * 2;k+=2) {
        // send the real part
        s.server_out.writeDouble( val[j][k]);
        if(GlobalDefs.DEBUG)
          System.out.println("sending Double "+ val[j][k]);

        // send the imaginary part
        s.server_out.writeDouble( val[j][k+1]);
        if(GlobalDefs.DEBUG)
          System.out.println("sending Double "+ val[j][k+1]);
      }
    }
  }

  public void copy (NetSolveDcomplexMatrix obj) {
    super.copy(obj);
    val = obj.val;
    parentThread = obj.parentThread;
  }
}
