/**
* NetSolveDcomplexScalar
*   
* This class contains a double precision complex scalar and methods 
* for reading, writing, and parsing double precision complex data.
* 
* @version 1.1, 8 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see NetSolveScalar
**/

import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedReader;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedOutputStream;
import java.lang.Thread;
import java.util.Vector;
import java.util.Random;
import java.util.StringTokenizer;

public class NetSolveDcomplexScalar extends NetSolveScalar {
  private double [] val;
  private Thread parentThread = null;
  private static double dcompScalarReal = 1.0,
                        dcompScalarImag = 0.0;

  public String sampleData(boolean rand) {
    if(rand) {
       Random rnd = new Random();

       return(Double.toString(rnd.nextDouble() * 10.0 - 5.0) + " + " +
              Double.toString(rnd.nextDouble() * 10.0 - 5.0) + "i");
    } else {
       return(new Double(dcompScalarReal).toString() + " + " +
           new Double(dcompScalarImag).toString()  + "i");
    }
  }

  /**
   *
   *  This method returns the current value of the 
   * double precision complex scalar.  The scalar should have been
   * initialized before calling this function (either
   * through readData or parseStream).
   *
   * @return The double precision complex scalar.
   *
   * @see #readData
   * @see #parseStream
   *
   **/

  public double [] getval() { 
    return val; 
  }


  /**
   *
   *  This method prints the current value of the 
   * double precision complex scalar to stdout.
   *
   **/

  public void printData() {
    System.out.print(this.toString());
  }

  /**
   *
   *  This method sets the data type and object type
   * of this object.
   *
   **/

  public void setDataType() {
    data_type = NETSOLVE_Z;
    object_type = NETSOLVE_SCALAR;
  }

  /**
   *
   *  This method returns a String representation of
   * the double precision complex scalar.
   *
   * @return String representation of the scalar.
   *
   **/

  public String toString() {
    StringBuffer tmp = new StringBuffer(100);
    int k;

    if(val.length == 0) 
      return("Empty result");

    if(val[1] >= 0.0) {
      tmp.append(String.valueOf(val[0]) + " + ");
      tmp.append(String.valueOf(val[1]) + "i");
    }
    else {
      tmp.append(String.valueOf(val[0]) + " ");
      tmp.append(String.valueOf(val[1]) + "i");
    }

    tmp.append("\n");

    return(tmp.toString());
  }

  public NetSolveDcomplexScalar () 
  {
    super();
  }

  /**
   *
   *  This is the constructor for the NetSolveDcomplexScalar
   * class.  Given a stream of text, this constructor 
   * attempts to create a double precision complex scalar.  If the
   * text is invalid (e.g., uneven rows, invalid numeric
   * format, etc.), a NetSolveException is thrown.
   *
   * @param in A stream containing the text representation
   *   of the scalar.
   * @param p The parent thread.  This is passed in so
   *   that we may call yield() and allow other threads
   *   a chance to run.
   *
   * @exception NetSolveException is thrown when the text
   *   stream does not parse into a double precision complex scalar.
   *
   **/

  public NetSolveDcomplexScalar (InputStream in, Thread p) 
   throws NetSolveException 
  {
    initialize(in,p);
  }

  public NetSolveDcomplexScalar (double [] sca)throws NetSolveException  
  {
    super();
    initialize(sca);
  }

  public void initialize ( double [] sca ) throws NetSolveException {
    if(sca == null)
      throw new NetSolveException("Null complex scalar");

    if(sca.length != 2)
      throw new NetSolveException("Badly formed complex scalar");
   
    val = sca;
    setDims(0,0,0);
    setDataType();
  }

  public void initialize(InputStream in, Thread p) throws NetSolveException
  {
    StringBuffer err = new StringBuffer(80);
    BufferedReader s = new BufferedReader(new InputStreamReader(in));
    Vector vec = null;
    double [] tmp_arr = null;
    int idx;

    setDataType();

    cur_lang = Labels.language;
    parentThread = p;

    try {
      vec = parseStream(s,parentThread,err);
    }catch(IOException e) {
      System.err.println("IOException in NetSolveDcomplexScalar()");
    }

    if(vec == null)
      throw new NetSolveException(err.toString());

    for(int i = 0; i < vec.size(); i++)
      if(vec.elementAt(i) != null) {
        tmp_arr = (double []) (vec.elementAt(i));
        break;
      }
    
    if(tmp_arr == null)
      System.err.println("bad news.");

    if((vec.size() > 1) || (tmp_arr.length > 2))
      System.err.println("Warning: Scalar input requires only 1 item.");

    val = new double[2];

    val[0] = tmp_arr[0];
    val[1] = tmp_arr[1];
  }
  
  /**
   *
   *  This method returns the number of double precision complex elements
   * in a 'line' of input.  Actually, the input is an array
   * representation of the line of input.
   *
   * @param item An array of double, cast to Object, so
   *  that we may call this from NetSolveScalar, the
   *  parent class, without knowing the data type.
   *
   * @return The number of double precision complex elements in the array.
   *
   **/

  public int lineLength(Object item)
  {
    return( ((double [])item).length / 2);
  }

  /**
   *
   *  This method parses a line of text into an array of
   * double.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of double, cast to Object.
   *
   **/

  public Object parseLine(String line, StringBuffer err) {
    return DcomplexParser.parseLine(line,err);
  }

  /**
   *
   *  This method reads data from the specified stream
   * and stores it into the double precision complex scalar, accessible
   * through the getval method.
   *
   * @param s The stream from which the data should be read.
   *
   * @exception IOException is thrown upon some error reading
   *   from the stream.
   *
   * @see #getval
   *
   **/

  public void readData ( NetSolveStream s ) throws IOException {
    int j;

    readScalarHeader(s);     //  read data type, dimensions, etc

    val = new double[2];

    val[0] = s.server_in.readDouble();
    val[1] = s.server_in.readDouble();
  }

  /**
   *
   *  This method writes the double precision complex scalar to the specified 
   * stream.
   *
   * @param s The stream to which the data should be written.
   *
   * @exception IOException is thrown upon some error writing
   *   to the stream.
   *
   **/

  public void writeData ( NetSolveStream s ) throws IOException {
    int j;
 
    writeScalarHeader(s);    //  send data type, dimensions, etc

    if(GlobalDefs.DEBUG)
      System.out.println( "sending double: " + val[0]);

    s.server_out.writeDouble( val[0]);

    // send the imaginary part
    if(GlobalDefs.DEBUG)
      System.out.println( "sending double: " + val[1]);

    s.server_out.writeDouble( val[1]);
  }

  public void copy (NetSolveDcomplexScalar obj) {
    super.copy(obj);
    val = obj.val;
    parentThread = obj.parentThread;
  }
}
