/**
* NetSolveDcomplexVector
*   
* This class contains a double precision complex vector and methods 
* for reading, writing, and parsing double precision complex data.
* 
* @version 1.1, 8 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see NetSolveVector
**/

import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedReader;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedOutputStream;
import java.lang.Thread;
import java.util.Vector;
import java.util.Random;
import java.util.StringTokenizer;

public class NetSolveDcomplexVector extends NetSolveVector {
  private double [] val;
  private Thread parentThread = null;
  private static double [] dcomp_array = {1,0, 2,0, 3,0, 4,0,};
  private static double [] rand_array = null;

  public String sampleData(boolean rand) {
    int i;

    if(rand) {
       Random rnd = new Random();

       if(rand_array == null)
         rand_array = new double[RANDOM_SIZE*2];

       for(i=0; i < RANDOM_SIZE*2; i++)
         rand_array[i] = rnd.nextDouble();

       return(dcomplexVector2string(rand_array));
    } else {
       return(dcomplexVector2string(dcomp_array));
    }
  }

  /**
   *
   *  This method returns the current value of the 
   * double precision complex vector.  The vector should have been
   * initialized before calling this function (either
   * through readData or parseStream).
   *
   * @return The double precision complex vector.
   *
   * @see #readData
   * @see #parseStream
   *
   **/

  public double [] getval() { 
    return val; 
  }


  /**
   *
   *  This method prints the current value of the 
   * double precision complex vector to stdout.
   *
   **/

  public void printData() {
    System.out.print(this.toString());
  }

  /**
   *
   *  This method sets the data type and object type
   * of this object.
   *
   **/

  public void setDataType() {
    data_type = NETSOLVE_Z;
    object_type = NETSOLVE_VECTOR;
  }

  /**
   *
   *  This method returns a String representation of
   * the double precision complex vector.
   *
   * @return String representation of the vector.
   *
   **/

  public String toString() {
    return dcomplexVector2string(val); 
  }

  private String dcomplexVector2string(double [] vec) {
    StringBuffer tmp = new StringBuffer(100);
    int k;

    if(vec.length == 0) 
      return("Empty result");

    for(k=0;k<vec.length;k+=2) {
      if(vec[k+1] >= 0.0) {
        tmp.append(String.valueOf(vec[k]) + " + ");
        tmp.append(String.valueOf(vec[k+1]) + "i");
      }
      else {
        tmp.append(String.valueOf(vec[k]) + " ");
        tmp.append(String.valueOf(vec[k+1]) + "i");
      }
      if(k != vec.length - 1)
        tmp.append("\n");
    }

    return(tmp.toString());
  }

  public NetSolveDcomplexVector () 
  {
    super();
  }

  /**
   *
   *  This is the constructor for the NetSolveDcomplexVector
   * class.  Given a stream of text, this constructor 
   * attempts to create a double precision complex vector.  If the
   * text is invalid (e.g., uneven rows, invalid numeric
   * format, etc.), a NetSolveException is thrown.
   *
   * @param in A stream containing the text representation
   *   of the vector.
   * @param p The parent thread.  This is passed in so
   *   that we may call yield() and allow other threads
   *   a chance to run.
   *
   * @exception NetSolveException is thrown when the text
   *   stream does not parse into a double precision complex vector.
   *
   **/

  public NetSolveDcomplexVector (InputStream in, Thread p) 
   throws NetSolveException 
  {
    initialize(in,p);
  }

  public NetSolveDcomplexVector (double [] vec)throws NetSolveException  
  {
    super();
    initialize(vec);
  }

  public void initialize ( double [] vec ) throws NetSolveException {
    if(vec == null)
      throw new NetSolveException("Null vector");

    if((vec.length % 2) != 0)
      throw new NetSolveException("Badly formed complex vector");

    val = vec;
    setDims(0,0,val.length /2);
    setDataType();
  }

  public void initialize(InputStream in, Thread p) throws NetSolveException
  {
    StringBuffer err = new StringBuffer(80);
    BufferedReader s = new BufferedReader(new InputStreamReader(in));
    Vector vec = null;
    double [] tmp_arr;
    int idx;

    setDataType();

    cur_lang = Labels.language;
    parentThread = p;

    try {
      vec = parseStream(s,parentThread,err);
    }catch(IOException e) {
      System.err.println("IOException in NetSolveDcomplexVector()");
    }

    if(vec == null)
      throw new NetSolveException(err.toString());
  
    idx = 0;
    for(int i = 0; i < vec.size(); i++)
      idx += ((double []) (vec.elementAt(i))).length;

    val = new double[idx];

    idx = 0;
    for(int i = 0; i < vec.size(); i++) {
      tmp_arr = (double []) (vec.elementAt(i));
      System.arraycopy(tmp_arr, 0, val, idx, tmp_arr.length);
      idx += tmp_arr.length;
    }
  }
  
  /**
   *
   *  This method returns the number of double precision complex elements
   * in a 'line' of input.  Actually, the input is an array
   * representation of the line of input.
   *
   * @param item An array of double, cast to Object, so
   *  that we may call this from NetSolveVector, the
   *  parent class, without knowing the data type.
   *
   * @return The number of double precision complex elements in the array.
   *
   **/

  public int lineLength(Object item)
  {
    return( ((double [])item).length / 2);
  }

  /**
   *
   *  This method parses a line of text into an array of
   * double.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of double, cast to Object.
   *
   **/

  public Object parseLine(String line, StringBuffer err) {
    return DcomplexParser.parseLine(line,err);
  }

  /**
   *
   *  This method reads data from the specified stream
   * and stores it into the double precision complex vector, accessible
   * through the getval method.
   *
   * @param s The stream from which the data should be read.
   *
   * @exception IOException is thrown upon some error reading
   *   from the stream.
   *
   * @see #getval
   *
   **/

  public void readData ( NetSolveStream s ) throws IOException {
    int j;

    readVectorHeader(s);     //  read data type, dimensions, etc

    val = new double[n*2];

    for(j=0;j<n*2;j+=2) {
      val[j] = s.server_in.readDouble();
      val[j+1] = s.server_in.readDouble();
    }
  }

  /**
   *
   *  This method writes the double precision complex vector to the specified 
   * stream.
   *
   * @param s The stream to which the data should be written.
   *
   * @exception IOException is thrown upon some error writing
   *   to the stream.
   *
   **/

  public void writeData ( NetSolveStream s ) throws IOException {
    int j;
 
    writeVectorHeader(s);    //  send data type, dimensions, etc

    for(j=0;j<n*2;j+=2) {
      if(GlobalDefs.DEBUG)
        System.out.println( "sending double: " + val[j]);

      s.server_out.writeDouble( val[j]);

      // send the imaginary part
      if(GlobalDefs.DEBUG)
        System.out.println( "sending double: " + val[j+1]);

      s.server_out.writeDouble( val[j+1]);
    }
  }

  public void copy (NetSolveDcomplexVector obj) {
    super.copy(obj);
    val = obj.val;
    parentThread = obj.parentThread;
  }
}
