/**
* NetSolveDoubleVector
*   
* This class contains a double vector and methods for reading,
* writing, and parsing double data.
* 
* @version 1.1, 8 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see NetSolveVector
**/

import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedReader;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedOutputStream;
import java.lang.Thread;
import java.util.Vector;
import java.util.Random;
import java.util.StringTokenizer;

public class NetSolveDoubleVector extends NetSolveVector {
  private double [] val;
  private Thread parentThread = null;
  private static double [] double_array = { 1.1, 2.2, 3.3, 4.4};
  private static double [] rand_array = null;

  public String sampleData(boolean rand) {
    int i;

    if(rand) {
       Random rnd = new Random();

       if(rand_array == null)
         rand_array = new double[RANDOM_SIZE];

       for(i=0; i < RANDOM_SIZE; i++)
         rand_array[i] = rnd.nextDouble();

       return(doubleVector2string(rand_array));
    } else {
       return(doubleVector2string(double_array));
    }
  }

  /**
   *
   *  This method returns the current value of the 
   * double vector.  The vector should have been
   * initialized before calling this function (either
   * through readData or parseStream).
   *
   * @return The double vector.
   *
   * @see #readData
   * @see #parseStream
   *
   **/

  public double [] getval() { 
    return val; 
  }


  /**
   *
   *  This method prints the current value of the 
   * double vector to stdout.
   *
   **/

  public void printData() {
    System.out.print(this.toString());
  }

  /**
   *
   *  This method sets the data type and object type
   * of this object.
   *
   **/

  public void setDataType() {
    data_type = NETSOLVE_D;
    object_type = NETSOLVE_VECTOR;
  }

  /**
   *
   *  This method returns a String representation of
   * the double vector.
   *
   * @return String representation of the vector.
   *
   **/

  public String toString() {
    return doubleVector2string(val);
  }

  private String doubleVector2string(double [] vec) {
    StringBuffer tmp = new StringBuffer(100);
    int i;

    if(vec.length == 0) 
      return("Empty result");

    for(i = 0; i < vec.length; i++)
      tmp.append(vec[i] + "\n");

    return(tmp.toString());
  }

  public NetSolveDoubleVector () 
  {
    super();
  }

  /**
   *
   *  This is the constructor for the NetSolveDoubleVector
   * class.  Given a stream of text, this constructor 
   * attempts to create a double vector.  If the
   * text is invalid (e.g., uneven rows, invalid numeric
   * format, etc.), a NetSolveException is thrown.
   *
   * @param in A stream containing the text representation
   *   of the vector.
   * @param p The parent thread.  This is passed in so
   *   that we may call yield() and allow other threads
   *   a chance to run.
   *
   * @exception NetSolveException is thrown when the text
   *   stream does not parse into a double vector.
   *
   **/

  public NetSolveDoubleVector (InputStream in, Thread p) 
   throws NetSolveException 
  {
    initialize(in,p);
  }

  public NetSolveDoubleVector (double [] vec)throws NetSolveException  
  {
    super();
    initialize(vec);
  }

  public void initialize ( double [] vec ) throws NetSolveException {
    if(vec == null)
      throw new NetSolveException("Null vector");

    val = vec;
    setDims(0,0,val.length);
    setDataType();
  }

  public void initialize(InputStream in, Thread p) throws NetSolveException
  {
    StringBuffer err = new StringBuffer(80);
    BufferedReader s = new BufferedReader(new InputStreamReader(in));
    Vector vec = null;
    double [] tmp_arr;
    int idx;

    setDataType();

    cur_lang = Labels.language;
    parentThread = p;

    try {
      vec = parseStream(s,parentThread,err);
    }catch(IOException e) {
      System.err.println("IOException in NetSolveDoubleVector()");
    }

    if(vec == null)
      throw new NetSolveException(err.toString());
  
    idx = 0;
    for(int i = 0; i < vec.size(); i++)
      idx += ((double []) (vec.elementAt(i))).length;

    val = new double[idx];

    idx = 0;
    for(int i = 0; i < vec.size(); i++) {
      tmp_arr = (double []) (vec.elementAt(i));
      System.arraycopy(tmp_arr, 0, val, idx, tmp_arr.length);
      idx += tmp_arr.length;
    }
  }
  
  /**
   *
   *  This method returns the number of double elements
   * in a 'line' of input.  Actually, the input is an array
   * representation of the line of input.
   *
   * @param item An array of double, cast to Object, so
   *  that we may call this from NetSolveVector, the
   *  parent class, without knowing the data type.
   *
   * @return The number of double in the array.
   *
   **/

  public int lineLength(Object item)
  {
    return( ((double [])item).length );
  }

  /**
   *
   *  This method parses a line of text into an array of
   * double.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of double, cast to Object.
   *
   **/

  public Object parseLine(String line, StringBuffer err) {
    return DoubleParser.parseLine(line,err);
  }

  /**
   *
   *  This method reads data from the specified stream
   * and stores it into the double vector, accessible
   * through the getval method.
   *
   * @param s The stream from which the data should be read.
   *
   * @exception IOException is thrown upon some error reading
   *   from the stream.
   *
   * @see #getval
   *
   **/

  public void readData ( NetSolveStream s ) throws IOException {
    int j;

    readVectorHeader(s);     //  read data type, dimensions, etc

    val = new double[n];

    for(j=0;j<n;j++)
      val[j] = s.server_in.readDouble();
  }

  /**
   *
   *  This method writes the double vector to the specified 
   * stream.
   *
   * @param s The stream to which the data should be written.
   *
   * @exception IOException is thrown upon some error writing
   *   to the stream.
   *
   **/

  public void writeData ( NetSolveStream s ) throws IOException {
    int k;
 
    writeVectorHeader(s);    //  send data type, dimensions, etc

    for(k=0;k< val.length;k++) {
      if(GlobalDefs.DEBUG)
        System.out.println( "sending double: " + val[k]);
      s.server_out.writeDouble( val[k]);
    }
  }

  public void copy (NetSolveDoubleVector obj) {
    super.copy(obj);
    val = obj.val;
    parentThread = obj.parentThread;
  }
}
