/**
* NetSolveFloatMatrix
*   
* This class contains a float matrix and methods for reading,
* writing, and parsing float data.
* 
* @version 1.1, 8 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see NetSolveMatrix
**/

import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedReader;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedOutputStream;
import java.lang.Thread;
import java.util.Vector;
import java.util.Random;
import java.util.StringTokenizer;

public class NetSolveFloatMatrix extends NetSolveMatrix {
  private float [][] val;
  private Thread parentThread = null;
  private float [][] float_array = { 
      { (float)  1.1, (float)  2.2, (float)  3.3, (float)  4.4},
      { (float)  5.1, (float)  6.2, (float)  7.3, (float)  8.4},
      { (float)  9.1, (float) 10.2, (float) 11.3, (float) 12.4},
      { (float) 13.1, (float) 14.2, (float) 15.3, (float) 16.4}
  };
  private static float [][] rand_array = null;

  public String sampleData(boolean rand) {
    int i,j;

    if(rand) {
       Random rnd = new Random();

       if(rand_array == null)
         rand_array = new float[RANDOM_SIZE][RANDOM_SIZE];

       for(i=0; i < RANDOM_SIZE; i++)
         for(j=0; j < RANDOM_SIZE; j++)
           rand_array[i][j] = rnd.nextFloat();

       return(floatMatrix2string(rand_array));
    } else {
       return(floatMatrix2string(float_array));
    }
  }

  /**
   *
   *  This method returns the current value of the 
   * float matrix.  The matrix should have been
   * initialized before calling this function (either
   * through readData or parseStream).
   *
   * @return The float matrix.
   *
   * @see #readData
   * @see #parseStream
   *
   **/

  public float [][] getval() { 
    return val; 
  }


  /**
   *
   *  This method prints the current value of the 
   * float matrix to stdout.
   *
   **/

  public void printData() {
    int i,j;

    System.out.print(this.toString());
  }

  /**
   *
   *  This method sets the data type and object type
   * of this object.
   *
   **/

  public void setDataType() {
    data_type = NETSOLVE_S;
    object_type = NETSOLVE_MATRIX;
  }

  /**
   *
   *  This method returns a String representation of
   * the float matrix.
   *
   * @return String representation of the matrix.
   *
   **/

  public String toString() {
    if(order == GlobalDefs.ROW_MAJOR)
      return floatMatrix2string(val);
    else
      return colMajorFloatMatrix2string(val);
  }

  private String floatMatrix2string(float [][] mat) {
    StringBuffer tmp = new StringBuffer(100);
    int i,j;

    if(mat.length == 0)
      return("Empty result");

    for(i = 0; i < mat.length; i++) {
      for(j = 0; j < mat[i].length; j++)
        tmp.append(mat[i][j] + " ");
      tmp.append("\n");
    }

    return(tmp.toString());
  }

  private String colMajorFloatMatrix2string(float [][] mat) {
    StringBuffer tmp = new StringBuffer(100);
    int i,j;

    if(mat.length == 0)
      return("Empty result");

    try {
      for( i=0 ; i < mat[0].length ; i++ ) {
        for( j=0 ; j < mat.length ; j++ )
          tmp.append(mat[j][i] + " ");
        tmp.append("\n");
      }
    }catch(NullPointerException e) {
      return("Bad matrix");
    }

    return(tmp.toString());
  }

  public NetSolveFloatMatrix () 
  {
    super();
  }

  /**
   *
   *  This is the constructor for the NetSolveFloatMatrix
   * class.  Given a stream of text, this constructor 
   * attempts to create a float matrix.  If the
   * text is invalid (e.g., uneven rows, invalid numeric
   * format, etc.), a NetSolveException is thrown.
   *
   * @param in A stream containing the text representation
   *   of the matrix.
   * @param p The parent thread.  This is passed in so
   *   that we may call yield() and allow other threads
   *   a chance to run.
   *
   * @exception NetSolveException is thrown when the text
   *   stream does not parse into an integer matrix.
   *
   **/

  public NetSolveFloatMatrix (InputStream in, Thread p) 
   throws NetSolveException 
  {
    initialize(in,p);
  }

  public NetSolveFloatMatrix (float [][] mat)throws NetSolveException  
  {
    super();
    initialize(mat);
  }

  public void initialize ( float [][] mat ) throws NetSolveException {
    checkDims(mat);
    val = mat;
    setDims(val.length,val[0].length,0);
    setDataType();
  }

  public void checkDims(float [][] mat) throws NetSolveException {
    int i, prev;

    try {
      prev = mat[0].length;

      for(i = 0; i < mat.length; i++) {
        if(mat[i].length != prev)
          throw new NetSolveException("Rows of matrix are uneven");
      }
    }catch(NullPointerException e) {
      throw new NetSolveException(
        "Badly formed matrix (null or containing null element)");
    }
  }

  public void initialize(InputStream in, Thread p) throws NetSolveException
  {
    StringBuffer err = new StringBuffer(80);
    BufferedReader s = new BufferedReader(new InputStreamReader(in));
    Vector vec = null;

    setDataType();

    cur_lang = Labels.language;
    parentThread = p;

    try {
      vec = parseStream(s,parentThread,err);
    }catch(IOException e) {
      System.err.println("IOException in NetSolveFloatMatrix()");
    }

    if(vec == null)
      throw new NetSolveException(err.toString());
  
    val = new float[vec.size()][];
    for(int i = 0; i < vec.size(); i++)
      val[i] = (float []) (vec.elementAt(i));
  }
  
  /**
   *
   *  This method returns the number of float elements
   * in a 'line' of input.  Actually, the input is an array
   * representation of the line of input.
   *
   * @param item An array of floats, cast to Object, so
   *  that we may call this from NetSolveMatrix, the
   *  parent class, without knowing the data type.
   *
   * @return The number of floats in the array.
   *
   **/

  public int lineLength(Object item)
  {
    return( ((float [])item).length );
  }

  /**
   *
   *  This method parses a line of text into an array of
   * floats.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of floats, cast to Object.
   *
   **/

  public Object parseLine(String line, StringBuffer err) {
    return FloatParser.parseLine(line,err);
  }

  /**
   *
   *  This method reads data from the specified stream
   * and stores it into the float matrix, accessible
   * through the getval method.
   *
   * @param s The stream from which the data should be read.
   *
   * @exception IOException is thrown upon some error reading
   *   from the stream.
   *
   * @see #getval
   *
   **/

  public void readData ( NetSolveStream s ) throws IOException {
    int j,k;

    readMatrixHeader(s);     //  read data type, dimensions, etc

    val = new float [m][n];

    for(j=0;j<m;j++) {
      parentThread.yield();
      for(k=0;k<n;k++)
        val[j][k] = s.server_in.readFloat();
    }
  }

  /**
   *
   *  This method writes the float matrix to the specified 
   * stream.
   *
   * @param s The stream to which the data should be written.
   *
   * @exception IOException is thrown upon some error writing
   *   to the stream.
   *
   **/

  public void writeData ( NetSolveStream s ) throws IOException {
    int j,k;
 
    writeMatrixHeader(s);    //  send data type, dimensions, etc

    for(j=0;j< m ; j++) {
      parentThread.yield();
      for(k=0;k< n;k++) {
        s.server_out.writeFloat(val[j][k]);
        if(GlobalDefs.DEBUG)
          System.out.println("sending float "+ val[j][k]);
      }
    }
  }

  public void copy (NetSolveFloatMatrix obj) {
    super.copy(obj);
    val = obj.val;
    parentThread = obj.parentThread;
  }
}
