/**
* NetSolveIntScalar
*   
* This class contains an integer scalar and methods for reading,
* writing, and parsing integer data.
* 
* @version 1.1, 8 Apr 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see NetSolveScalar
**/

import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedReader;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedOutputStream;
import java.lang.Thread;
import java.util.Vector;
import java.util.Random;
import java.util.StringTokenizer;

public class NetSolveIntScalar extends NetSolveScalar {
  private int val;
  private Thread parentThread = null;
  private static int int_scalar = 1;

  public String sampleData(boolean rand) {
    if(rand) {
       Random rnd = new Random();

       return(new Integer(rnd.nextInt()).toString());
    } else {
       return(new Integer(int_scalar).toString());
    }
  }

  /**
   *
   *  This method returns the current value of the 
   * integer scalar.  The scalar should have been
   * initialized before calling this function (either
   * through readData or parseStream).
   *
   * @return The integer scalar.
   *
   * @see #readData
   * @see #parseStream
   *
   **/

  public int getval() { 
    return val; 
  }


  /**
   *
   *  This method prints the current value of the 
   * integer scalar to stdout.
   *
   **/

  public void printData() {
    System.out.print(this.toString());
  }

  /**
   *
   *  This method sets the data type and object type
   * of this object.
   *
   **/

  public void setDataType() {
    data_type = NETSOLVE_I;
    object_type = NETSOLVE_SCALAR;
  }

  /**
   *
   *  This method returns a String representation of
   * the integer scalar.
   *
   * @return String representation of the scalar.
   *
   **/

  public String toString() {
    int i;

    return(Integer.toString(val) + "\n");
  }

  public NetSolveIntScalar () 
  {
    super();
  }

  /**
   *
   *  This is the constructor for the NetSolveIntScalar
   * class.  Given a stream of text, this constructor 
   * attempts to create an integer scalar.  If the
   * text is invalid (e.g., uneven rows, invalid numeric
   * format, etc.), a NetSolveException is thrown.
   *
   * @param in A stream containing the text representation
   *   of the scalar.
   * @param p The parent thread.  This is passed in so
   *   that we may call yield() and allow other threads
   *   a chance to run.
   *
   * @exception NetSolveException is thrown when the text
   *   stream does not parse into an integer scalar.
   *
   **/

  public NetSolveIntScalar (InputStream in, Thread p) 
   throws NetSolveException 
  {
    initialize(in,p);
  }

  public NetSolveIntScalar (int sca) 
  {
    super();
    initialize(sca);
  }

  public void initialize ( int sca ) {
    val = sca;
    setDims(0,0,0);
    setDataType();
  }

  public void initialize(InputStream in, Thread p) throws NetSolveException
  {
    StringBuffer err = new StringBuffer(80);
    BufferedReader s = new BufferedReader(new InputStreamReader(in));
    Vector vec = null;
    int [] tmp_arr = null;

    setDataType();

    cur_lang = Labels.language;
    parentThread = p;

    try {
      vec = parseStream(s,parentThread,err);
    }catch(IOException e) {
      System.err.println("IOException in NetSolveIntScalar()");
    }

    if(vec == null)
      throw new NetSolveException(err.toString());
  
    for(int i=0;i < vec.size(); i++)
      if(vec.elementAt(i) != null) {
        tmp_arr = (int []) (vec.elementAt(i));
        break;
      }

    if(tmp_arr == null)
      System.err.println("bad news.");

    if((vec.size() > 1) || (tmp_arr.length > 1))
      System.err.println("Warning: Scalar input requires only 1 item.");

    val = tmp_arr[0];
  }
  
  /**
   *
   *  This method returns the number of integer elements
   * in a 'line' of input.  Actually, the input is an array
   * representation of the line of input.
   *
   * @param item An array of integers, cast to Object, so
   *  that we may call this from NetSolveScalar, the
   *  parent class, without knowing the data type.
   *
   * @return The number of integers in the array.
   *
   **/

  public int lineLength(Object item)
  {
    return( ((int [])item).length );
  }

  /**
   *
   *  This method parses a line of text into an array of
   * integers.
   *
   * @param line A string containing the input.
   * @param err A buffer to which error messages are appended.
   *
   * @return Array of integers, cast to Object.
   *
   **/

  public Object parseLine(String line, StringBuffer err) {
    return IntegerParser.parseLine(line,err);
  }

  /**
   *
   *  This method reads data from the specified stream
   * and stores it into the integer scalar, accessible
   * through the getval method.
   *
   * @param s The stream from which the data should be read.
   *
   * @exception IOException is thrown upon some error reading
   *   from the stream.
   *
   * @see #getval
   *
   **/

  public void readData ( NetSolveStream s ) throws IOException {
    int j;

    readScalarHeader(s);     //  read data type, dimensions, etc

    val = s.server_in.readInt();
  }

  /**
   *
   *  This method writes the integer scalar to the specified 
   * stream.
   *
   * @param s The stream to which the data should be written.
   *
   * @exception IOException is thrown upon some error writing
   *   to the stream.
   *
   **/

  public void writeData ( NetSolveStream s ) throws IOException {
    int k;
 
    writeScalarHeader(s);    //  send data type, dimensions, etc

    s.server_out.writeInt(val);
  }

  public void copy (NetSolveIntScalar obj) {
    super.copy(obj);
    val = obj.val;
    parentThread = obj.parentThread;
  }
}
