/**
* NetSolveObject
*   
* This class defines some constants used in the
* NetSolve system.
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.util.Vector;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;

public abstract class NetSolveObject {

  public static final int        NETSOLVE_I = 1;
  public static final int        NETSOLVE_S = 2;
  public static final int        NETSOLVE_D = 3;
  public static final int        NETSOLVE_C = 4;
  public static final int        NETSOLVE_Z = 5;
  public static final int     NETSOLVE_CHAR = 6;
  public static final int NETSOLVE_EXTERNAL = 7;
  public static final int        NETSOLVE_B = 8;
  public static final int    NETSOLVE_ASCII = 9;

  public static final int   NETSOLVE_MATRIX = 1;
  public static final int   NETSOLVE_VECTOR = 2;
  public static final int   NETSOLVE_SCALAR = 3;
  public static final int   NETSOLVE_UPF    = 4;
  public static final int   NETSOLVE_FILE   = 5;

  public static final int       RANDOM_SIZE = 4;

  public int data_type;
  public int object_type;

  public String description;

  protected int m, n;
  protected int cur_lang;
  
  public abstract void setDims(int lines, int last, int total_items);

  public abstract Object parseLine(String line, StringBuffer err);

  public abstract int lineLength(Object item);

  public abstract void readData(NetSolveStream s) throws IOException;

  public abstract void writeData(NetSolveStream s) throws IOException;

  public abstract void setDataType();

  public abstract void initialize(InputStream in, Thread p) 
    throws NetSolveException;

  public abstract void printData();

  public abstract String sampleData(boolean rand);

  public NetSolveObject()
  {
    super();
  }

  /**
   *
   * This method returns a text description of the given data type.
   * 
   * @param data_type The data type.
   *
   * @return Returns string describing that data type.
   *
   **/

  public static String datatype2str(int data_type) {
    switch(data_type) {
      case NETSOLVE_I:
        return(new String("Integer"));
      case NETSOLVE_S:
        return(new String("Float"));
      case NETSOLVE_D:
        return(new String("Double"));
      case NETSOLVE_C:
        return(new String("Complex"));
      case NETSOLVE_Z:
        return(new String("Double Complex"));
      case NETSOLVE_CHAR:
        return(new String("Char"));
      case NETSOLVE_EXTERNAL:
        return(new String("User Provided Function"));
      case NETSOLVE_B:
        return(new String("Byte"));
      case NETSOLVE_ASCII:
        return(new String("ASCII"));
      default:
        return(new String("UNKNOWN DATA TYPE"));
    }
  }
 
  /**
   *
   * This method returns a text description of the given object type.
   * 
   * @param obj_type The object type.
   *
   * @return Returns string describing that object type.
   *
   **/

  public static String objtype2str(int obj_type) {
    switch(obj_type) {
      case NETSOLVE_MATRIX:
        return(new String("Matrix"));
      case NETSOLVE_VECTOR:
        return(new String("Vector"));
      case NETSOLVE_SCALAR:
        return(new String("Scalar"));
      case NETSOLVE_UPF:
        return(new String("UPF"));
      case NETSOLVE_FILE:
        return(new String("FILE"));
      default:
        return(new String("UNKNOWN OBJECT TYPE"));
    }
  }

  public static NetSolveObject newObject(int dt, int ot) {
     switch(ot) {
       case NETSOLVE_MATRIX:
         switch(dt) {
           case NETSOLVE_I:         return new NetSolveIntMatrix();
           case NETSOLVE_S:         return new NetSolveFloatMatrix();
           case NETSOLVE_D:         return new NetSolveDoubleMatrix();
           case NETSOLVE_C:         return new NetSolveScomplexMatrix();
           case NETSOLVE_Z:         return new NetSolveDcomplexMatrix();
           case NETSOLVE_CHAR:      return new NetSolveCharMatrix();
           case NETSOLVE_B:         return new NetSolveByteMatrix();

           case NETSOLVE_EXTERNAL:
           case NETSOLVE_ASCII:
             System.err.println("GetProblems: Bad case (external/ascii matrix)");
             break;
           default:
             System.err.println("GetProblems: Bad case (within matrix)");
             break;
         }
         break;
       case NETSOLVE_VECTOR:
         switch(dt) {
           case NETSOLVE_I:        return new NetSolveIntVector();
           case NETSOLVE_S:        return new NetSolveFloatVector();
           case NETSOLVE_D:        return new NetSolveDoubleVector();
           case NETSOLVE_C:        return new NetSolveScomplexVector();
           case NETSOLVE_Z:        return new NetSolveDcomplexVector();
           case NETSOLVE_CHAR:     return new NetSolveCharVector();
           case NETSOLVE_B:        return new NetSolveByteVector();

           case NETSOLVE_EXTERNAL:
           case NETSOLVE_ASCII:
             System.err.println("GetProblems: Bad case (external/ascii vector)");
             break;
           default:
             System.err.println("GetProblems: Bad case (within vector)");
             break;
         }
         break;
       case NETSOLVE_SCALAR:
         switch(dt) {
           case NETSOLVE_I:         return new NetSolveIntScalar();
           case NETSOLVE_S:         return new NetSolveFloatScalar();
           case NETSOLVE_D:         return new NetSolveDoubleScalar();
           case NETSOLVE_C:         return new NetSolveScomplexScalar();
           case NETSOLVE_Z:         return new NetSolveDcomplexScalar();
           case NETSOLVE_CHAR:      return new NetSolveCharScalar();
           case NETSOLVE_B:         return new NetSolveByteScalar();

           case NETSOLVE_EXTERNAL:
           case NETSOLVE_ASCII:
             System.err.println("GetProblems: Bad case (external/ascii scalar)");
             break;
           default:
             System.err.println("GetProblems: Bad case (within scalar)");
             break;
         }
         break;
       case NETSOLVE_UPF:
         switch(dt) {
           case NETSOLVE_EXTERNAL:   return new NetSolveUPF();

           default:
             System.err.println("GetProblems: Bad case (within UPF)");
             break;
         }
         break;
       case NETSOLVE_FILE:
         switch(dt) {
           case NETSOLVE_ASCII:      return new NetSolveASCII();

           default:
             System.err.println("GetProblems: Bad case (within FILE)");
             break;
         }
         break;
     }

     return null;
  }

  public Vector parseStream( BufferedReader s, Thread p, StringBuffer err )
    throws IOException
  {
    int last = 0, lines = 0, total_items = 0, prev = 0;
    boolean err_status = false, uneven = false;
    String temp=null;
    Vector vec = new Vector(100);
    Object items;

    while((temp = s.readLine()) != null) {     // for each line of input
      if(p != null)
        p.yield();
      lines++;

      if(GlobalDefs.DEBUG)
        System.out.println("line "+lines);

      if((items = parseLine(temp,err)) == null) {
        err_status = true;
        total_items++;
        continue;
      }

      vec.addElement(items);

      prev = lineLength(items);
      total_items += prev;

      if( (this instanceof NetSolveMatrix) && (last != 0) && (last != prev) )
      {
        // the rows of the matrix aren't even, flag an error
        err_status = true;
        uneven = true;
      }

      last = prev;
    }

    this.setDims(lines, last, total_items);

    if((this instanceof NetSolveMatrix) && uneven)
      err.append(Labels.uneven_matrix[cur_lang] + "\n");

    if(total_items == 0) {
      // this input is empty
      err_status = true;
      err.append(Labels.empty_input[cur_lang] + "\n");
    }

    if(err_status)
      return null;
    else
      return vec;
  }

  public void copy(NetSolveObject obj) {
    data_type = obj.data_type;
    object_type = obj.object_type;
    description = new String(obj.description);
    m = obj.m;
    n = obj.n;
    cur_lang = obj.cur_lang;
  }
} // End class NetSolveObject
