/**
* NetSolveStream
*   
* This class defines a socket class which includes features
* useful to the NetSolve Java interaface (such as the ability
* to read IP addresses, read vectors, write vectors, etc.).
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.net.Socket;
import java.io.IOException;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.util.Vector;

public class NetSolveStream {
  private int encoding;
  private int order;
  private boolean source_gui;

  public Socket s = null;
  public DataInputStream server_in;
  public DataOutputStream server_out; 

  private Thread thisThread = null;

  public static final int DEFAULT_ORIENTATION = GlobalDefs.ROW_MAJOR;
  public static final boolean DEFAULT_GUI     = true;

  /**
   *
   * This method returns the encoding currently in use - either DATA_RAW
   * or DATA_XDR.
   * 
   * @return The current encoding in use.
   *
   **/

  public int getEncoding() {
    return encoding;
  }

  /**
   *
   * This method prepares to read data from the socket
   * by setting the encoding: XDR or RAW.  If not sure,
   * use XDR.
   * 
   * @param enc The encoding to use.
   *
   **/

  public void initRecv(int enc) {
    if((enc != GlobalDefs.DATA_RAW) && (enc != GlobalDefs.DATA_XDR))
    {
      // If the encoding isn't something we recognize, default to XDR

      System.err.println("NetSolveStream:initRecv() Warning: " + 
          "Invalid encoding ("+enc+"), using XDR.");

      encoding = GlobalDefs.DATA_XDR;
    }
    else
      encoding = enc;

    if(GlobalDefs.DEBUG)
      System.out.println("in initRecv, setting encoding to " +
        ((encoding == GlobalDefs.DATA_XDR) ? "DATA_XDR" : "DATA_RAW"));
  }
      
  public void flushOutput() throws java.io.IOException {
    server_out.flush();
  }

  /**
   *
   * This method reads an IP address from the socket.  In XDR
   * mode, read four integers:                         
   *    ___________________________________________________   
   *   | Integer 1  | Integer 2  | Integer 3  | Integer 4  |  
   *   |XXXXXX|byte1|XXXXXX|byte2|XXXXXX|byte3|XXXXXX|byte4|  
   *    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   
   *                                                          
   * Then mask out all but the least significant bytes in each   
   * integer to get the return value:                         
   *             _______________________________              
   *            |      Integer Return Value     |             
   *            | byte1 | byte2 | byte3 | byte4 |             
   *             ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~              
   *
   * In RAW mode the IP address is represented by only 4 bytes,
   * so we only need to read one integer.
   * 
   * @return Returns the IP address (stored in an int).
   *
   * @exception java.io.IOException Thrown when an error is 
   *    encountered reading from the socket (e.g., broken pipe).
   *
   **/

  public int readIPaddr() throws java.io.IOException {
    int [] b = new int [4];
    byte [] c = new byte [16];
    int i;

    if(encoding == GlobalDefs.DATA_XDR) {
      server_in.readFully(c,0,16);            // read 16 bytes

      return(((0xFF & c[3]) << 24) | 
             ((0xFF & c[7]) << 16) | 
             ((0xFF & c[11]) << 8) |
              (0xFF & c[15]));
    } else {
      return(server_in.readInt());
    }
  }

  /**
   *
   * This method writes an IP address to the socket.  As with
   * readIPaddr, the IP address is written as one int in RAW
   * mode, but as four in XDR mode.
   * 
   * @param addr The IP addressed stored in an int.
   *
   * @exception java.io.IOException Thrown when an error is 
   *    encountered writing to the socket (e.g., broken pipe).
   *
   **/

  public void writeIPaddr(int addr) throws java.io.IOException {
    if(encoding == GlobalDefs.DATA_XDR) {
      // As with readIPaddr, an IP address must be transmitted as
      // four integers.  So here we just shift over the byte we
      // want to transmit, mask out the other bits, and send it as
      // an integer.

      server_out.writeInt( (addr >> 24) & 0xFF);
      server_out.writeInt( (addr >> 16) & 0xFF);
      server_out.writeInt( (addr >>  8) & 0xFF);
      server_out.writeInt( addr & 0xFF);
    } else {
      server_out.writeInt(addr);
    }
  }

  /**
   *
   * This method calculates the amount of padding required 
   * for an item of a given length to be aligned on a 4-byte
   * boundary.
   *
   * @param len The length of the item.
   *
   * @return Returns the length of the padding.
   *
   */

  public int padXDR(int len) {
    return((4 * (len/4 + 1) - len) % 4);
  }

  /**
   *
   * This method writes a string to the socket.  Since we are 
   * communicating with servers written in C, we terminate the
   * string with a NULL char by writing a zero byte.  XDR
   * requires first sending the string length, then sending
   * the string, then padding with zeroes to align the data
   * on a 4 byte boundary.
   * 
   * @param str The string to write.
   *
   * @exception java.io.IOException Thrown when an error is 
   *    encountered writing to the socket (e.g., broken pipe).
   *
   **/

  public void writeString(String str) throws java.io.IOException {  
    int i, pad, length;

    if(encoding == GlobalDefs.DATA_XDR) {
      length = str.length();
      pad = padXDR(length);

      if(GlobalDefs.DEBUG) {
        System.out.println("Writing string in XDR mode....");
        System.out.println("length = " +length+ " pad = " + pad);
        System.out.println(str);
      }

      server_out.writeInt(str.length());   // send the length
      server_out.writeBytes(str);          // send the string
      for(i=0;i<pad;i++)
        server_out.writeByte(0);           // send the padding
    } else {
      if(GlobalDefs.DEBUG) {
        System.out.println("Writing string in RAW mode....");
        System.out.println(str);
      }

      server_out.writeBytes(str);   // just write the string...
      server_out.writeByte(0);      // and null-terminate
    }
  }

  /**
   *
   * This method reads a string from the socket.  In XDR mode, we
   * first read an int representing the string length.  We can
   * then allocate the appropriate number of bytes and read in
   * exactly that amount, including any necessary padding.  In RAW
   * mode, we dont know the length of the string ahead of time 
   * so we have to just read until we get a NULL char.  In this
   * case we read one character at a time until we hit the end.
   * Clearly there are better ways to write this, but in the
   * NetSolve interface, strings are almost always sent in XDR
   * mode, so I dont think it's causing a big problem.
   * 
   * @return Returns the string read from the socket.
   *
   * @exception java.io.IOException Thrown when an error is 
   *    encountered reading from the socket (e.g., broken pipe).
   *
   **/

  public String getString() throws java.io.IOException {  
    StringBuffer temp;
    byte [] temp2;
    String str;
    int i,length, pad=0;
    byte resp;

    if(encoding == GlobalDefs.DATA_XDR) {
      length = this.server_in.readInt();    // read the length

      pad = padXDR(length);

      temp2 = new byte[length];
      server_in.readFully(temp2,0,length);  // read the string

      // str = new String(temp2,0);
      str = new String(temp2);

      for(i=0;i<pad;i++)
        server_in.readByte();    // read the padding

      return(str);
    } else {
      temp= new StringBuffer(200);
  
      // read characters until we hit NULL.
      // not the best way to write this, but the simplest

      while( (resp = server_in.readByte()) != 0 )
        temp.append((char)resp);
      return(temp.toString());
    }
  }

  /**
   *
   * This method reads a string of the given length from the 
   * socket.  This performs the same function as the getString
   * method above, but in this case we know the length of the
   * string ahead of time.
   * 
   * @return Returns the string read from the socket.
   *
   * @exception java.io.IOException Thrown when an error is 
   *    encountered reading from the socket (e.g., broken pipe).
   *
   **/

  public String getString(int length) throws java.io.IOException {  
    byte [] temp = new byte[length];
    int i;

    if(encoding == GlobalDefs.DATA_XDR)
      for(i = 0; i < length; i++)
        temp[i] = (byte) this.server_in.readInt();
    else
      server_in.readFully(temp);  // read the string

    return(new String(temp));
  }
    
  /**
   *
   * This method shuts down the connection by setting the
   * streams to null and closing the socket.
   * 
   * @exception java.io.IOException Thrown when an error is 
   *    encountered closing the connection.
   *
   **/

  public void shutDown() throws java.io.IOException {
    server_in = null;
    if(server_out != null) {
      server_out.flush();    // flush any remaining output
      server_out = null;
    }
    s.close();
  }

  /** 
   *
   * This method writes the data type and object type
   * to the socket.
   *
   * @param dataType The data type of this object.
   * @param objType The object type of this object.
   *
   * @exception java.io.IOException Thrown when an error is 
   *    encountered writing to the socket (e.g., broken pipe).
   */

  public void sendType(int dataType, int objType) 
    throws java.io.IOException
  {
    if(GlobalDefs.DEBUG)
      System.out.println("sending data_type = " + 
        NetSolveObject.datatype2str(dataType));

    this.server_out.writeInt(dataType);
 
    if(GlobalDefs.DEBUG)
      System.out.println("sending obj_type = " + 
        NetSolveObject.objtype2str(objType));
 
    this.server_out.writeInt(objType);
  }

  /**
   *
   * This is the constructor for the NetSolveStream class.
   * It attempts to create a socket connection to the 
   * specified host/port.  It then creates two streams:
   * a DataInputStream for reading and a DataOutputStream
   * for writing.  It also sets the data format so that
   * we can determine whether to use XDR or not.
   * 
   * @param hostname The name of the host to connect to.
   * @param port The port number on that host.
   * @param data_format The data format we're using.
   *
   * @exception java.io.IOException Thrown when an error is 
   *    encountered creating the socket.
   *
   **/

  NetSolveStream(String hostname, int port, int data_format)
   throws java.io.IOException {

    this(hostname,port,data_format,DEFAULT_ORIENTATION,DEFAULT_GUI);
  }

  /**
   *
   * This is the constructor for the NetSolveStream class.
   * It attempts to create a socket connection to the 
   * specified host/port.  It then creates two streams:
   * a DataInputStream for reading and a DataOutputStream
   * for writing.  It also sets the data format so that
   * we can determine whether to use XDR or not.
   * 
   * @param hostname The name of the host to connect to.
   * @param port The port number on that host.
   * @param data_format The data format we're using.
   * @param orient The orientation of matrices (row or 
   *    column-major)
   *
   * @exception java.io.IOException Thrown when an error is 
   *    encountered creating the socket.
   *
   **/

  NetSolveStream(String hostname, int port, int data_format, int orient)
   throws java.io.IOException {

    this(hostname,port,data_format,orient,DEFAULT_GUI);
  }

  /**
   *
   * This is the constructor for the NetSolveStream class.
   * It attempts to create a socket connection to the 
   * specified host/port.  It then creates two streams:
   * a DataInputStream for reading and a DataOutputStream
   * for writing.  It also sets the data format so that
   * we can determine whether to use XDR or not.
   * 
   * @param hostname The name of the host to connect to.
   * @param port The port number on that host.
   * @param data_format The data format we're using.
   * @param orient The orientation of matrices (row or 
   *    column-major)
   * @param gui The current execution mode - true represents
   *    GUI and false represents API.
   *
   * @exception java.io.IOException Thrown when an error is 
   *    encountered creating the socket.
   *
   **/

  NetSolveStream(String hostname, int port, int data_format, int orient, boolean gui)
   throws java.io.IOException {
 
     thisThread = Thread.currentThread();
     if(thisThread == null)
       thisThread = new Thread();
  
     s = new Socket(hostname,port);

     server_in = 
       new DataInputStream(new BufferedInputStream(s.getInputStream()));

     server_out = 
       new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));

     if((data_format != GlobalDefs.DATA_RAW) &&
            (data_format != GlobalDefs.DATA_XDR)) {
       System.err.println("NetSolveStream warning: Invalid data format, " +
            "using XDR.");
       encoding = GlobalDefs.DATA_XDR;
     }
     else
       encoding = data_format;

     if((order != GlobalDefs.COL_MAJOR) &&
            (order != GlobalDefs.ROW_MAJOR)) {
       System.err.println("NetSolveStream warning: Invalid data ordering, " +
            "using row major.");
       order = GlobalDefs.ROW_MAJOR;
     }
     else
       order = orient;

     source_gui = gui;

     this.server_out.writeChar(encoding);    // send encoding

     return;
  }
} // end class NetSolveStream
