/**
* Netsolve
*   
* This class provides a Java API to the NetSolve system.
* Before creating a new instance of this class, first
* create a new NetSolve session (see class NetsolveSession).
*
* For example:
*  ns = new NetsolveSession("kenner.cs.utk.edu");
*  n  = new Netsolve(ns);
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see NetsolveSession
**/

import java.util.Vector;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class Netsolve 
 implements ActionListener {

  public static final String DEFAULT_HOST = "kenner.cs.utk.edu";
  private GetProblems gp;
  private String host;
  private int prob_index = -1;
  private int cur_idx = 0;
  private int upf_idx = 0;
  private NetSolveObject [] obj;
  private NetSolveObject [] output_obj;
  private SubmitJob sub;
  private int order;

  /**
   *
   * This is the constructor for the Netsolve class.
   * The first parameter (the "session") provides
   * information including the host name of the NetSolve 
   * agent and a list of the problems that are handled
   * by this agent.
   * 
   * @param s The session to use for this submission.
   *
   * @see NetsolveSession#getProbList
   * @see NetsolveSession#getHost
   *
   * @exception NetSolveException thrown when the agent
   *   cannot be contacted or returns a null list of
   *   problems.
   *
   **/

  public Netsolve (NetsolveSession s) throws NetSolveException {
    if(s == null)
      throw new NetSolveException ("NetSolve Error: " +
        "null session.  Initialize a new NetsolveSession.");

    gp = s.getProbList();
    host = s.getHost();

    if(host == null) 
    {
      host = DEFAULT_HOST;
      if(GlobalDefs.DEBUG) 
        System.out.println("Netsolve: NULL host, resetting to default: " + 
          DEFAULT_HOST);
    }

    if(gp == null) 
      throw new NetSolveException("NetSolve Error: " +
        "Retrieved null problem list.");

    order = GlobalDefs.ROW_MAJOR;
  }

  /**
   *
   * This method searches for the problem that the user wants to
   * solve.  If found, the vector which will hold the data is
   * allocated (along with two arrays to store dimension info).
   * 
   * @param prob_name The name of the problem (e.g. "iqsort");
   *
   * @exception NetSolveException thrown if the problem cannot
   *    be found in the list of problems returned by the agent.
   *
   **/

  public void setProblem(String prob_name) throws NetSolveException {
    int i;

    for(i=0;i<gp.problems.length;i++) {
      if(gp.problems[i].name.equalsIgnoreCase(prob_name)) {
        if(GlobalDefs.DEBUG)
          System.out.println("Problem found!");
        prob_index = i;
        cur_idx = 0;
        upf_idx = 0;
        obj = new NetSolveObject[gp.problems[i].nb_input];
        return;
      }
    }     

    throw new NetSolveException("NetSolve Error: Problem '" + prob_name +
        "' NOT found!");
  }

  /**
   *
   * This method sets the current storage scheme to be used
   * in interpreting the user's matrices.
   * 
   * @param major A string representing the storage scheme
   *   being used.  Acceptable values are:
   *     'row' - signifies row-major layout
   *     'col' or 'column' - signifies column-major layout
   *   This routine is not case sensitive.
   *
   **/

  public void setMajor(String major)
  {
    if( major.equalsIgnoreCase("row") )
      order = GlobalDefs.ROW_MAJOR;
    else if(( major.equalsIgnoreCase("col") ) ||
            ( major.equalsIgnoreCase("column") ))
      order = GlobalDefs.COL_MAJOR;
    else {
      System.err.println("setMajor: Warning, unknown type: \""+major+"\"..." +
         "use \"row\" or \"col\".");
      order = GlobalDefs.ROW_MAJOR;
    }
  }

  /**
   *
   * This method sets the current storage scheme to be used
   * in interpreting the user's matrices.
   * 
   * @param major An integer representing the storage scheme
   *   being used.  Acceptable values are:
   *     GlobalDefs.ROW_MAJOR - signifies row-major layout
   *     GlobalDefs.COL_MAJOR - signifies column-major layout
   *
   * @see GlobalDefs
   *
   **/

  public void setMajor(int major)
  {
    if((major != GlobalDefs.ROW_MAJOR) && (major != GlobalDefs.COL_MAJOR))
    {
      System.err.println("setMajor: Warning, unknown type ("+major+")..." +
         "use 0 for col major, 1 for row major.");
      order = GlobalDefs.ROW_MAJOR;
    }
    else
      order = major;
  }

  /**
   *
   * When the user calls pushArg to specify the data for a particular
   * input object, we should check that the data provided matches the
   * requirements of the routine.  First, we look up the expected 
   * data type and object type and then we check them against the
   * actual data type and object type provided as parameters.
   * 
   * @param idx The index of the input object expected by the routine.
   * @param obj_type The object type of the actual item received.
   * @param data_type The data type of the actual item received.
   *
   * @exception NetSolveException thrown if the argument does not
   *    match the expected type.
   *
   **/

  private void checkArg(int idx, int obj_type, int data_type) 
   throws NetSolveException
  {
    int pDataType;
    int pObjType;

    // ensure that the user has not provided too many args.
 
    if(cur_idx > gp.problems[prob_index].nb_input - 1)
      throw new NetSolveException(
        "NetSolve Error: Attempting to provide too many arguments");

    pDataType = gp.problems[prob_index].inputs[idx].data_type;
    pObjType  = gp.problems[prob_index].inputs[idx].object_type;

    if((pDataType == NetSolveObject.NETSOLVE_C) ||
       (pDataType == NetSolveObject.NETSOLVE_Z))
    {
      // When the user passes a float or double vector to pushArg, we
      // don't know whether it is intended to be a scalar or vector
      // since both complex scalars and complex vectors are represented
      // by one dimensional arrays.  so we add a special case here to 
      // allow for that ambiguity.  this is somewhat lax, but it should
      // not cause any major problems.

      if(pDataType == NetSolveObject.NETSOLVE_C)
      {
        // we're expecting a single precision complex scalar or vector,
        // so data type will come in as float.

        if((obj_type == NetSolveObject.NETSOLVE_SCALAR) ||
           (data_type != NetSolveObject.NETSOLVE_S))
               throw new NetSolveException(
                 argErrMsg(data_type,NetSolveObject.NETSOLVE_C));
        else
          return;
      }

      if(pDataType == NetSolveObject.NETSOLVE_Z)
      {
        // expecting double precision complex data type, so data type
        // will come in as double.

        if((obj_type == NetSolveObject.NETSOLVE_SCALAR) ||
           (data_type != NetSolveObject.NETSOLVE_D))
               throw new NetSolveException(
                 argErrMsg(data_type,NetSolveObject.NETSOLVE_Z));
        else
          return;
      }

      if((pObjType == NetSolveObject.NETSOLVE_MATRIX) &&
         (obj_type != NetSolveObject.NETSOLVE_MATRIX))
      {
        throw new NetSolveException(
          argErrMsg(data_type, pDataType, obj_type, pObjType));
      }
    }

    // For all cases other than complex numbers, this small
    // bit of code is all we need.

    if ((pObjType != obj_type) || (pDataType != data_type))
      throw new NetSolveException(
        argErrMsg(data_type, pDataType, obj_type, pObjType));
  }

  /**
   *
   * This method generates an error string based on the
   * expected data type of the routine.  This method 
   * is called by checkArg.
   * 
   * @param dataType The type of data provided by the user.
   * @param dataTypeExpected  The type of data expected by the routine.
   *
   **/

  private String argErrMsg(int dataType, int dataTypeExpected)
  {
    return(new String("NetSolve Error: Input " + (cur_idx+1) +
        " of problem '" + gp.problems[prob_index].name + 
        "' expects: " + 
        NetSolveObject.datatype2str(dataTypeExpected) + " " + 
        "\nYou provided: " + 
        NetSolveObject.datatype2str(dataType))); 
  }

  /**
   *
   * This method generates an error string based on the
   * expected data type and object type of the routine.  
   * This method is called by checkArg.
   * 
   * @param dataType The type of data provided by the user.
   * @param dataTypeExpected  The type of data expected by the routine.
   * @param objType The type of object provided by the user.
   * @param objTypeExpected  The type of object expected by the routine.
   *
   **/

  private String argErrMsg(int dataType, int dataTypeExpected,
                           int  objType, int  objTypeExpected) 
  {

      return(new String("NetSolve Error: Input " + (cur_idx+1) +
        " of problem '" + gp.problems[prob_index].name + 
        "' expects: " + 
        NetSolveObject.datatype2str(dataTypeExpected) + " " + 
        NetSolveObject.objtype2str(objTypeExpected) + "\nYou provided: " + 
        NetSolveObject.datatype2str(dataType) + " " + 
        NetSolveObject.objtype2str(objType)));
  }

  /**
   *
   * This method stores an integer item to be sent to the
   * server.  Since this is just a plain 'int', the value
   * is first wrapped in an Integer object and then another
   * call to pushObj actually stores the object.
   * 
   * @param int_item The integer to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is expecting a different type of input for this item.
   *
   **/

  public void pushArg(int int_item) throws NetSolveException {
    checkArg(cur_idx,NetSolveObject.NETSOLVE_SCALAR,NetSolveObject.NETSOLVE_I);

    obj[cur_idx++] = new NetSolveIntScalar(int_item);
  }

  /**
   *
   * This method stores a byte item to be sent to the
   * server.  The byte value is first wrapped in an Integer 
   * object (since there's no Byte object) and then another
   * call to pushObj actually stores the object.
   * 
   * @param byte_item The byte to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is expecting a different type of input for this item.
   *
   **/

  public void pushArg(byte byte_item) throws NetSolveException {
    checkArg(cur_idx,NetSolveObject.NETSOLVE_SCALAR,NetSolveObject.NETSOLVE_B);

    obj[cur_idx++] = new NetSolveByteScalar(byte_item);
  }

  /**
   *
   * This method stores an integer vector to be sent to the
   * server.  
   * 
   * @param intVec The integer vector to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting an integer vector for this item.
   *
   **/

  public void pushArg(int [] intVec) throws NetSolveException {
    checkArg(cur_idx,NetSolveObject.NETSOLVE_VECTOR,NetSolveObject.NETSOLVE_I);

    obj[cur_idx++] = new NetSolveIntVector(intVec);
  }

  /**
   *
   * This method stores a byte vector to be sent to the
   * server.  
   * 
   * @param byteVec The byte vector to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting a byte vector for this item.
   *
   **/

  public void pushArg(byte [] byteVec) throws NetSolveException {
    checkArg(cur_idx,NetSolveObject.NETSOLVE_VECTOR,NetSolveObject.NETSOLVE_B);

    obj[cur_idx++] = new NetSolveByteVector(byteVec);
  }

  /**
   *
   * This method stores an integer matrix to be sent to the
   * server.  
   * 
   * @param intMat The integer matrix to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting an integer matrix for this item.
   *
   **/

  public void pushArg(int [][] intMat) throws NetSolveException {
    checkArg(cur_idx,NetSolveObject.NETSOLVE_MATRIX,NetSolveObject.NETSOLVE_I);

    obj[cur_idx++] = new NetSolveIntMatrix(intMat);
  }

  /**
   *
   * This method stores a byte matrix to be sent to the
   * server.  
   * 
   * @param byteMat The byte matrix to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting a byte matrix for this item.
   *
   **/

  public void pushArg(byte [][] byteMat) throws NetSolveException {
    checkArg(cur_idx,NetSolveObject.NETSOLVE_MATRIX,NetSolveObject.NETSOLVE_B);

    obj[cur_idx++] = new NetSolveByteMatrix(byteMat);
  }

  /**
   *
   * This method stores a float item to be sent to the server.
   * Since this is just a plain 'float', the value
   * is first wrapped in an Float object and then another
   * call to pushArg actually stores the object.
   * 
   * @param float_item The float to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting a floating point scalar for this item.
   *
   **/

  public void pushArg(float float_item) throws NetSolveException {
    checkArg(cur_idx,NetSolveObject.NETSOLVE_SCALAR,NetSolveObject.NETSOLVE_S);

    obj[cur_idx++] = new NetSolveFloatScalar(float_item);
  }

  /**
   *
   * This method stores a single precision floating point matrix to 
   * be sent to the server.  
   * 
   * @param floatVec The float vector to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting a floating point vector for this item.
   *
   **/

  public void pushArg(float [] floatVec) throws NetSolveException {
    NetSolveObject input_obj = gp.problems[prob_index].inputs[cur_idx];

    checkArg(cur_idx,NetSolveObject.NETSOLVE_VECTOR,NetSolveObject.NETSOLVE_S);

    // This could be either a complex scalar, a complex vector, or
    // a regular float vector.

    if(input_obj instanceof NetSolveFloatVector)
      obj[cur_idx++] = new NetSolveFloatVector(floatVec);
    else if(input_obj instanceof NetSolveScomplexVector)
      obj[cur_idx++] = new NetSolveScomplexVector(floatVec);
    else if(input_obj instanceof NetSolveScomplexScalar)
      obj[cur_idx++] = new NetSolveScomplexScalar(floatVec);
    else
      throw new NetSolveException("pushArg: Unknown object type");

  }

  /**
   *
   * This method stores a single precision floating point matrix to 
   * be sent to the server.  
   * 
   * @param floatMat The float matrix to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting a floating point matrix for this item.
   *
   **/

  public void pushArg(float [][] floatMat) throws NetSolveException {
    NetSolveObject input_obj = gp.problems[prob_index].inputs[cur_idx];

    checkArg(cur_idx,NetSolveObject.NETSOLVE_MATRIX,NetSolveObject.NETSOLVE_S);

    // This could be either a regular matrix or a complex matrix

    if(input_obj instanceof NetSolveFloatMatrix)
      obj[cur_idx++] = new NetSolveFloatMatrix(floatMat);
    else if(input_obj instanceof NetSolveScomplexMatrix)
      obj[cur_idx++] = new NetSolveScomplexMatrix(floatMat);
    else
      throw new NetSolveException("pushArg: Unknown object type");

  }

  /**
   *
   * This method stores a double item to be sent to the server.
   * Since this is just a plain 'double', the value
   * is first wrapped in an Double object and then another
   * call to pushArg actually stores the object.
   * 
   * @param double_item The double to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting a double precision floating point scalar 
   *   for this item.
   *
   **/

  public void pushArg(double double_item) throws NetSolveException {
    checkArg(cur_idx,NetSolveObject.NETSOLVE_SCALAR,NetSolveObject.NETSOLVE_D);

    obj[cur_idx++] = new NetSolveDoubleScalar(double_item);
  }

  /**
   *
   * This method stores a double precision floating point vector to 
   * be sent to the server.  
   * 
   * @param doubleVec The double vector to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting a double precision floating point vector 
   *   for this item.
   *
   **/

  public void pushArg(double [] doubleVec) throws NetSolveException {
    NetSolveObject input_obj = gp.problems[prob_index].inputs[cur_idx];

    checkArg(cur_idx,NetSolveObject.NETSOLVE_VECTOR,NetSolveObject.NETSOLVE_D);

    // This could be either a complex scalar, a complex vector, or
    // a regular double vector.

    if(input_obj instanceof NetSolveDoubleVector)
      obj[cur_idx++] = new NetSolveDoubleVector(doubleVec);
    else if(input_obj instanceof NetSolveDcomplexVector)
      obj[cur_idx++] = new NetSolveDcomplexVector(doubleVec);
    else if(input_obj instanceof NetSolveDcomplexScalar)
      obj[cur_idx++] = new NetSolveDcomplexScalar(doubleVec);
    else
      throw new NetSolveException("pushArg: Unknown object type");

  }

  /**
   *
   * This method stores a double precision floating point matrix to 
   * be sent to the server.  
   * 
   * @param doubleMat The double matrix to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting a double precision floating point matrix
   *   for this item.
   *
   **/

  public void pushArg(double [][] doubleMat) throws NetSolveException {
    NetSolveObject input_obj = gp.problems[prob_index].inputs[cur_idx];

    checkArg(cur_idx,NetSolveObject.NETSOLVE_MATRIX,NetSolveObject.NETSOLVE_D);

    // This could be either a regular matrix or a complex matrix

    if(input_obj instanceof NetSolveDoubleMatrix)
      obj[cur_idx++] = new NetSolveDoubleMatrix(doubleMat);
    else if(input_obj instanceof NetSolveDcomplexMatrix)
      obj[cur_idx++] = new NetSolveDcomplexMatrix(doubleMat);
    else
      throw new NetSolveException("pushArg: Unknown object type");
  }

  /**
   *
   * This method stores a char item to be sent to the server.
   * Since this is just a plain 'char', the value
   * is first wrapped in an Character object and then another
   * call to pushArg actually stores the object.
   * 
   * @param char_item The char to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting a char scalar for this item.
   *
   **/

  public void pushArg(char char_item) throws NetSolveException {
    checkArg(cur_idx,NetSolveObject.NETSOLVE_SCALAR,NetSolveObject.NETSOLVE_CHAR);

    obj[cur_idx++] = new NetSolveCharScalar(char_item);
  }

  /**
   *
   * This method stores a character vector to be sent to the server.  
   * 
   * @param charVec The character vector to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting a char vector for this item.
   *
   **/

  public void pushArg(char [] charVec) throws NetSolveException {
    checkArg(cur_idx,NetSolveObject.NETSOLVE_VECTOR,NetSolveObject.NETSOLVE_CHAR);

    obj[cur_idx++] = new NetSolveCharVector(charVec);
  }

  /**
   *
   * This method stores a character matrix to be sent to the server.  
   * 
   * @param charMat The character matrix to store.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting a char matrix for this item.
   *
   **/

  public void pushArg(char [][] charMat) throws NetSolveException {
    checkArg(cur_idx,NetSolveObject.NETSOLVE_MATRIX,NetSolveObject.NETSOLVE_CHAR);

    obj[cur_idx++] = new NetSolveCharMatrix(charMat);
  }

  /**
   *
   * This method stores and prepares a UPF to be sent to the
   * server.  First, we create a new Vector and add the 
   * integer designating the language in which the UPF is
   * written followed by the UPF code itself.  Here we just
   * add the item directly to the vector rather than calling
   * pushObj.
   * 
   * @param upf_code The actual code of the UPF.
   * @param lang The language of this UPF 
   *    for C, pass in GlobalDefs.LANG_C
   *    for FORTRAN, pass in GlobalDefs.LANG_FORTRAN
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting a UPF for this item.
   *
   **/

  public void pushArg(String upf_code, int lang) throws NetSolveException {

    NetSolveUPF upf = new NetSolveUPF(upf_code,lang,upf_idx);

    obj[cur_idx++] = upf;
    upf_idx++;
  }

  /**
   *
   * This method stores and prepares ASCII text to be sent to the
   * server.   We actually store the string as a char array.
   *
   * @param text The text string to push.
   *
   * @exception NetSolveException thrown if the selected problem
   *   is not expecting a FILE for this item.
   *
   **/

  public void pushArg(String text) throws NetSolveException {
    checkArg(cur_idx,NetSolveObject.NETSOLVE_FILE,NetSolveObject.NETSOLVE_ASCII);

    obj[cur_idx++] = new NetSolveASCII(text.toCharArray());
  }

  /**
   *
   * @deprecated
   *
   * This method returns the results of the computation.
   * The results are stored in a Vector - each element of
   * the Vector containing one output object.  So, for
   * example if a problem returns two double precision
   * matrices, the results would be accessed as follows:
   *
   *    Vector v = n.getOutput();
   *    mat1 = (double [][]) (v.elementAt(0));
   *    mat2 = (double [][]) (v.elementAt(1));
   *
   * The explicit cast is necessary since Vector.elementAt()
   * always returns java.lang.Object.
   *
   * @return Returns a Vector containing the results of the
   *   computation
   *
   * @exception NetSolveException thrown when there was
   *   a problem submitting the job (i.e., there is no
   *   data to return).
   **/

  public Vector getOutput() throws NetSolveException {
    NetSolveObject [] tmp = getOutputObject();
    Vector output = new Vector(tmp.length);
    int i;

    // this next bit of code is pretty cheesy, but it
    // maintains compatibility with older versions of
    // the API.

    for(i=0;i < tmp.length; i++) {
      if(tmp[i] instanceof NetSolveASCII)
        output.addElement(((NetSolveASCII) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveUPF)
        output.addElement(((NetSolveUPF) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveIntScalar)
        output.addElement(
           new Integer(((NetSolveIntScalar) tmp[i]).getval()));
      else if(tmp[i] instanceof NetSolveIntVector)
        output.addElement(((NetSolveIntVector) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveIntMatrix)
        output.addElement(((NetSolveIntMatrix) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveFloatScalar)
        output.addElement(
           new Float(((NetSolveFloatScalar) tmp[i]).getval()));
      else if(tmp[i] instanceof NetSolveFloatVector)
        output.addElement(((NetSolveFloatVector) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveFloatMatrix)
        output.addElement(((NetSolveFloatMatrix) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveDoubleScalar)
        output.addElement(
           new Double(((NetSolveDoubleScalar) tmp[i]).getval()));
      else if(tmp[i] instanceof NetSolveDoubleVector)
        output.addElement(((NetSolveDoubleVector) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveDoubleMatrix)
        output.addElement(((NetSolveDoubleMatrix) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveByteScalar)
        output.addElement(
            new Byte(((NetSolveByteScalar) tmp[i]).getval()));
      else if(tmp[i] instanceof NetSolveByteVector)
        output.addElement(((NetSolveByteVector) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveByteMatrix)
        output.addElement(((NetSolveByteMatrix) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveCharScalar)
        output.addElement(
            new Character(((NetSolveCharScalar) tmp[i]).getval()));
      else if(tmp[i] instanceof NetSolveCharVector)
        output.addElement(((NetSolveCharVector) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveCharMatrix)
        output.addElement(((NetSolveCharMatrix) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveScomplexScalar)
        output.addElement(((NetSolveScomplexScalar) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveScomplexVector)
        output.addElement(((NetSolveScomplexVector) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveScomplexMatrix)
        output.addElement(((NetSolveScomplexMatrix) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveDcomplexScalar)
        output.addElement(((NetSolveDcomplexScalar) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveDcomplexVector)
        output.addElement(((NetSolveDcomplexVector) tmp[i]).getval());
      else if(tmp[i] instanceof NetSolveDcomplexMatrix)
        output.addElement(((NetSolveDcomplexMatrix) tmp[i]).getval());
      else
        throw new NetSolveException("Bad object in output array");
    }

    return output;
  }


  public NetSolveObject [] getOutputObject() throws NetSolveException {
    String err;

    if((err = sub.getErrString()) != null)
      throw new NetSolveException(err);

    if(output_obj == null)
      throw new NetSolveException("Error: Null output.");

    return output_obj;
  }

  /**
   *
   * This method is used when submitting the problem in 
   * non-blocking mode.  It waits until the results have
   * been computed and then returns.  After this method
   * returns, the user may call getOutput() to retrieve
   * the results.
   * 
   * @see SubmitJob
   * 
   * @exception NetSolveException thrown when the SubmitJob
   *    thread is interrupted.
   *
   **/

  public void waitFor() throws NetSolveException {
 
    try {  
      sub.join();   // wait for SubmitJob to finish
    }catch(InterruptedException e) {
       throw new NetSolveException("NetSolve Error: " +
          "SubmitJob thread interrupted ("+ e.getMessage() +
          ").  Try again.");
    }

    output_obj = sub.getOutputVec();
  }

  /**
   *
   * This method submits the data in non-blocking mode - 
   * that is, it does not wait for the computation to
   * complete before returning.  It returns immediately
   * after submitting the problem.  This allows the user
   * to overlap some of their computations with the job
   * submission tasks (sending data, waiting for reply 
   * from the server, retrieving data).  The user may
   * at any time wait for the results by calling the
   * waitFor() method, which blocks until the results
   * are ready.
   * 
   * @see SubmitJob
   * @see #waitFor
   * 
   * @exception NetSolveException thrown if the inputs
   *   were not fully specified, if there was no valid
   *   problem selected (see setProblem()), or if there
   *   was a NetSolveException thrown by SubmitJob.
   *
   **/

  public void submitProbNB() throws NetSolveException {
    int i;

    for(i=0;i< gp.problems[prob_index].nb_input;i++) {
      if( obj[i] == null )
        throw new NetSolveException("Cannot submit job: " +
           "Not enough input item(s) specified");

      if( obj[i] instanceof NetSolveMatrix )
        ((NetSolveMatrix) obj[i]).setMajor(order);
    }

    if(prob_index < 0)
      throw new NetSolveException("No valid problem selected");

    if(host == null) 
    {
      host = new String(DEFAULT_HOST);
      if(GlobalDefs.DEBUG) 
        System.out.println("submitProbNB: NULL host, resetting to default: " 
           + DEFAULT_HOST);
    }
    
    if(GlobalDefs.DEBUG)
      System.out.println("now i do my thing...");

    try {
      sub = new SubmitJob(null,host,gp.problems[prob_index],obj,
          "1.1",order);
      sub.start();
    }catch(NetSolveException e) {
      // we'll just go ahead and throw e back to the caller

      throw e;
    }
  }

  /**
   *
   * This method submits the data in blocking mode - that is,
   * it waits for the computation to complete before returning.
   * After submitProb() returns, the results may be accessed
   * by calling getOutput().
   * 
   * @see SubmitJob
   * @see #getOutput
   * 
   * @exception NetSolveException thrown if the inputs
   *   were not fully specified, if there was no valid
   *   problem selected (see setProblem()), or if there
   *   was a NetSolveException thrown by SubmitJob.
   *
   **/

  public void submitProb() throws NetSolveException {
    int i;

    for(i=0;i< gp.problems[prob_index].nb_input;i++) {
      if( obj[i] == null )
        throw new NetSolveException("Cannot submit job: " +
           "Not enough input item(s) specified");

      if( obj[i] instanceof NetSolveMatrix )
        ((NetSolveMatrix) obj[i]).setMajor(order);
    }

    if(prob_index < 0)
      throw new NetSolveException("No valid problem selected");

    if(host == null) 
    {
      host = new String(DEFAULT_HOST);
      if(GlobalDefs.DEBUG) 
        System.out.println("submitProb: NULL host, resetting to default: " + 
          DEFAULT_HOST);
    }
    
    if(GlobalDefs.DEBUG)
      System.out.println("now i do my thing...");

    try {
      sub = new SubmitJob(null,host,gp.problems[prob_index],obj,
          "1.1",order);
      sub.start();

      try {  // wait for SubmitJob to finish
        sub.join();
      }catch(InterruptedException e) {
       throw new NetSolveException("NetSolve Error: " +
          "SubmitJob thread interrupted ("+ e.getMessage() +
          ").  Try again.");
      }

      output_obj = sub.getOutputVec();

    }catch(NetSolveException e) {
      // we'll just go ahead and throw e back to the caller

      throw e;
    }
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of this event.  Then, execute the code
   * associated with the event.
   *
   * @param event The action event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();
    int elem;

/*
    if ( (elem = subVec.indexOf(source)) >= 0 )
      handleDataOutput( (SubmitJob) subVec.elementAt(elem));
    else
      System.err.println("Netsolve (API): " +
         " I dont know where this event comes from!");
*/
  }
}
