/**
* NetsolveSession
*   
* This class contains methods to contact a NetSolve agent
* and retrieve its list of problems.  To use the Netsolve class
* first requires creating a new instance of NetsolveSession
* in order to specify the NetSolve agent.
* 
* @version 1.1, 26 Jan 1998
* @author Keith Seymour (seymour@cs.utk.edu)
*
* @see Netsolve
**/

public class NetsolveSession {
  public static final String DEFAULT_HOST = "kenner.cs.utk.edu";
  private GetProblems gp;
  private String host;

  /**
   *
   * This is the constructor for the NetsolveSession class.
   * The purpose of this method is to establish a "NetSolve
   * session".  This includes contacting the agent, getting
   * its list of problems, and setting the current agent
   * host name.  This version takes no parameters, so it
   * contacts the default agent, given in the class
   * variable DEFAULT_HOST.
   * 
   * @see GetProblems
   *
   * @exception NetSolveException soon to be implemented
   *
   **/

  public NetsolveSession() throws NetSolveException {
    this(DEFAULT_HOST);
  }

  /**
   *
   * This is the constructor for the NetsolveSession class.
   * The purpose of this method is to establish a "NetSolve
   * session".  This includes contacting the agent, getting
   * its list of problems, and setting the current agent
   * host name.
   * 
   * @param host The host name of the agent to contact (pass
   *   in null to get the default host)
   *
   * @see GetProblems
   *
   * @exception NetSolveException soon to be implemented
   *
   **/

  public NetsolveSession(String hostname) throws NetSolveException {

    if(hostname == null) 
      host = new String(DEFAULT_HOST);
    else
      host = hostname;

    try {
      if(GlobalDefs.DEBUG)
        System.out.println("getting problems from host: " + host);
      gp = new GetProblems(host);
      if(gp.problems.length == 0)
        throw new NetSolveException("NetSolve Error: " +
          "Agent '" + host + "' has no problems.");
    }
    catch(NetSolveException e) {
      // could not make connection...
      // go ahead and throw e back to the caller
      throw e;
    }
  }

  /**
   *
   * This method returns an object containing the list of
   * problems.  The actual array of problems may be accessed
   * directly as follows:
   *
   *   GetProblems gp = ns.getProbList();
   *   Problem p1 = gp.problems[0];
   * 
   * @return Returns an instance of GetProblems containing the
   *    list of problems.
   *
   * @see GetProblems
   **/

  public GetProblems getProbList() {
    return gp;
  }

  /**
   *
   * This method returns the host name of the agent contacted
   * for this session.
   * 
   * @return Returns a string containing the host name of the agent
   *
   **/

  public String getHost() {
    return host;
  }

  /**
   *
   * This method prints the problem name, path, and description
   * for each problem returned by the agent.  This method would
   * normally only be called in debugging situations.
   * 
   **/

  public void printProbDesc() {
    int i;

    if(gp != null) {
      for(i=0;i<gp.problems.length;i++) {
        System.out.println(gp.problems[i].path + gp.problems[i].name);
        System.out.println(gp.problems[i].description);
        System.out.println("--------------------------------------------");
      }     
    }
  }
}
