/**
* OutputData
* 
* This class controls the graphical user interface for the
* Output Screen.
* 
* @version 1.1, 26 Jan 1998 
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.io.FileOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import java.awt.*;
import java.awt.event.*;

import java.util.Vector;

public class OutputData extends Frame 
 implements ActionListener, ItemListener {

  private String [] data;

  private Button quit_button;
  private Button save_button;

  private List output_list;

  private TextArea data_box;
  private TextArea description;

  private Panel data_panel;

  private InputData parent;
  private Problem prob;

  private Label output_label;
  private Label desc_label;
  private Label data_label;

  private int cur_lang = Labels.language;

  private InitData id;

  private static final int DATA_WIDTH         = 35;
  private static final int DATA_HEIGHT        = 10;
  private static final int OUTPUT_LIST_SIZE   = 15;
  private static final int DESC_WIDTH         = 35;
  private static final int DESC_HEIGHT        = 10;

  private static final int SMALL              = 10;
  private static final int MEDIUM             = 14;

  private static final int DESC_FONT_SIZE     = SMALL;
  private static final int LABEL_FONT_SIZE    = MEDIUM;
  private static final int BUTTON_FONT_SIZE   = MEDIUM;
  private static final int DATA_BOX_FONT_SIZE = SMALL;

  /**
   *
   * This is the constructor for the OutputData class.  The purpose of
   * this method is to initialize and layout all the graphical components
   * in the Output Screen.  This is done with standard AWT components.
   * 
   * @param f The parent frame.
   * @param p The problem the results correspond to.
   * @param vec The vector of results.
   * @param ident Identifying string used in the title bar.
   *
   **/

  public OutputData(InputData f, Problem p, NetSolveObject [] obj, String ident) {
    super(); 
    super.setTitle(Labels.output_title[cur_lang]+p.name+"' ("+ident+")"); 

    parent = f;
    prob = p;

    if(parent.in_applet())
      parent.screen_list.addElement(this);  // register this screen

    if(GlobalDefs.DEBUG)
      System.out.println("Entering OutputData");

    this.setLayout(new BorderLayout(10,10)); 

    /* 
     * Start InitData (which formats the data into text)
     * and wait for it to complete since there's not much we
     * can do in the meantime.
     */

    id = new InitData(p,obj,this);

    if(GlobalDefs.DEBUG)
      System.out.println("After InitData - waiting for results to be prepared...");

  }
 
  /**
   *
   * This method is called when the InitData thread completes
   * and then retrieves the String array containing the
   * results.
   * 
   **/

  private void handleResults()
  {
    if(GlobalDefs.DEBUG)
      System.out.println("before getStrArray");

    data = id.getStrArray();

    if(GlobalDefs.DEBUG)
      System.out.println("Data Initialized");

    initScreen();
  }

  /**
   *
   * This method initializes all the graphical elements of
   * the Output Screen (buttons, labels, etc.).
   * 
   **/

  private void initScreen()
  {
    int i;

    //initialize all the checkbox and text field arrays
    data_box = new TextArea(DATA_HEIGHT,DATA_WIDTH);
    data_box.setFont(new Font("Helvetica",Font.BOLD,DATA_BOX_FONT_SIZE));

    //create some buttons
    quit_button = new Button(Labels.dismiss[cur_lang]);
    quit_button.addActionListener(this);
    quit_button.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));

    save_button = new Button(Labels.save_data[cur_lang]);
    save_button.addActionListener(this);
    save_button.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    save_button.setEnabled(false);

    Panel output_panel = new Panel();
    output_panel.setLayout(new BorderLayout(5,5));

    output_list = new List(OUTPUT_LIST_SIZE,false);
    output_list.addItemListener(this);
    output_panel.add("South",output_list);

    output_label = new Label(Labels.outputs[cur_lang]);
    output_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
    output_panel.add("North",output_label);
 
    // define description panel
    Panel desc_panel = new Panel();
    desc_panel.setLayout(new BorderLayout(5,5));

    description = new TextArea(DESC_HEIGHT,DESC_WIDTH);
    description.setFont(new Font("Helvetica",Font.BOLD,DESC_FONT_SIZE));
    description.setEditable(false);
   
    desc_label = new Label(Labels.desc[cur_lang]);
    desc_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    desc_panel.add("North",desc_label);
    desc_panel.add("South",description);

    data_panel = new Panel();
    data_panel.setLayout(new BorderLayout(5,5));

    data_panel.add("South",data_box);

    data_label = new Label(Labels.data[cur_lang]);
    data_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    Panel data_save_panel = new Panel();
    data_save_panel.setLayout(new BorderLayout(5,5));
    
    data_save_panel.add("West",data_label);
    data_save_panel.add("East",save_button);

    data_panel.add("North",data_save_panel);

    Panel right_panel = new Panel();
    right_panel.setLayout(new BorderLayout(5,5));

    right_panel.add("North",data_panel);
    right_panel.add("South",desc_panel);
 
    // define panel for buttons
    Panel left_panel = new Panel();
    left_panel.setLayout(new BorderLayout(5,5));

    left_panel.add("North",output_panel);
    left_panel.add("South",quit_button);

    this.add("West",left_panel);
    this.add("East",right_panel);

    for(i=0;i<prob.nb_output;i++)
      output_list.add( 
        NetSolveObject.datatype2str(prob.outputs[i].data_type) 
         + " - " +
        NetSolveObject.objtype2str(prob.outputs[i].object_type));
    
    if(data.length > prob.nb_output)
      output_list.add("Standard Output");

    data_box.setEditable(false);
    
    // if we're in an applet, we can't save to a file, so hide
    // the save button.
    if(parent.in_applet())
      save_button.setVisible(false);

    this.pack();
    this.setVisible(true);
  }  

  /**
   * 
   * This method saves data currently being displayed in the
   * Output Screen to a file.  Obviously this is only useful when
   * running as an application.
   * 
   * @param data The string to save to the file.
   * @param filename The name to be given to the newly created file.
   *
   **/

  private void saveData(String data, String filename) {
    FileOutputStream file_out;
    DataOutputStream out;

    if(GlobalDefs.DEBUG)
      System.out.println("saving to file: "+filename);

    try {
      file_out = new FileOutputStream(filename);
    } catch (IOException e) {
      if(GlobalDefs.DEBUG)
        System.out.println("Can't create file: '"+filename+"'");
      NetSolveWarning n1 = 
        new NetSolveWarning(this,"Can't create file: '"+filename+"'");
      return;
    }

    out = new DataOutputStream(file_out);
 
    // try to write the text to the file

    try {
      out.writeBytes(data);
    } catch (IOException e) {
      if(GlobalDefs.DEBUG)
        System.out.println("Error writing to file: "+filename);
      NetSolveWarning n1 = 
        new NetSolveWarning(this,"Error writing to file: '"+filename+"'");
      return;
    }

    try {
      out.close();
    } catch (IOException e) {
      // can't close the file...
      System.err.println("Warning: Could not close file.");
      return;
    }
  }

  /**
   * 
   * This method is called when the user presses the Dismiss button.
   * Dispose of the window.
   * 
   **/

  private void handleQuit()
  {
    this.setVisible(false);
    this.dispose(); 
  }

  /**
   * 
   * This method is called when the user presses the Save button.
   * First, open a FileDialog window so the user can enter the
   * filename.  Then, save the data to that file.
   * 
   **/

  private void handleSave()
  {
    int selected = output_list.getSelectedIndex();

    // call file dialog to save data here
    FileDialog fd = new FileDialog(this,"Save data",FileDialog.SAVE);
    fd.setVisible(true);
    if(fd.getFile() != null) {   // make sure user didn't click 'cancel'

      if(fd.getDirectory() == null) {
        // the full path is not specified, just use the filename
        saveData(data[selected], fd.getFile());
      } 
      else {
        // the full path is specified, use the whole thing
        saveData(data[selected],fd.getDirectory() + fd.getFile());
      }
    }
  }

  /**
   * 
   * This method is called whenever an action event is detected.
   * 
   * @param event The detected event.
   *
   **/

  public void actionPerformed(ActionEvent event)
  {
    Object source = event.getSource();

    if (source == quit_button)
      handleQuit();
    else if (source == save_button)
      handleSave();
    else if (source == id)
      handleResults();
    else
      System.err.println("OutputData: I dont know where that event came from.");
  }

  /**
   * 
   * This method is called when the user selects an item from 
   * the output list.  Display the appropriate data.
   * 
   **/

  private void handleOutputListSelection()
  {
    int selected = output_list.getSelectedIndex();

    data_box.setText(data[selected]);

    if((data.length > prob.nb_output) && (selected == data.length - 1))
      description.setText(
        "This is the standard output from the function");
    else
      description.setText(prob.outputs[selected].description);

    save_button.setEnabled(true); 
  }

  /**
   * 
   * This method is called when the user deselects an item from 
   * the output list.  Disable the save button.
   *
   **/

  private void handleOutputListDeselection()
  {
    save_button.setEnabled(false);
  }

  /**
   * 
   * This method is called whenever an item event is detected.
   * In this case, we are interested in listening for list
   * selection and deselection.
   * 
   * @param event The detected event.
   *
   **/

  public void itemStateChanged(ItemEvent event)
  {
    Object source = event.getSource();

    if (source == output_list) {
      if(event.getStateChange() == ItemEvent.SELECTED)
        handleOutputListSelection();
      else
        handleOutputListDeselection();
    }
  }
}  // end class OutputData
