/**
* SubmitJob
*
* This class handles all aspects of submitting a problem to the
* agent to be solved.
* 
* @version 1.1, 26 Jan 1998 
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.awt.Frame;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.util.Vector;

import java.awt.AWTEvent;
import java.awt.event.ActionEvent;

public class SubmitJob extends NetSolveThread {
  private NetSolveStream str;
  private ServerInfo [] servers;
  private InputData f;
  private InputData input_frame;
  private String hostname;
  private Problem prob;
  private NetSolveObject [] obj;
  private NetSolveObject [] output_obj;
  private String idString;
  private int current_data_format;
  private int my_data_format = 11;
  private int order = GlobalDefs.ROW_MAJOR;
  private boolean gui = true;
  private String err_string = null;
  public int cur_lang = Labels.language;

  /**
   *
   * This method provides access to the output array to
   * other classes.
   * 
   * @return Returns the array containing the results.
   *
   **/

  public NetSolveObject [] getOutputVec() {
    return(output_obj);
  }

  public String getidString() {
    return(idString);
  }

  public void shutDown() {
    try {
      str.shutDown();
    } catch(java.io.IOException e) {
      System.err.println("SubmitJob: Can't close stream.");
    }
  }

  /**
   *
   * This method returns any error logged while trying to
   * submit the job.  Currently this method is only called
   * from the NetSolve Java API (the return value would
   * always be null if called from the GUI).
   * 
   * @return Returns a string containing a description of
   *   the error or returns null if there was no error.
   *
   **/

  public String getErrString() {
    return(err_string);
  }

  /**
   *
   * This method attempts to contact the agent and retrieve a list
   * of servers that can solve the given problem.
   * 
   * @param server The host name of the agent.
   * @param prob The problem to be solved.
   *
   * @return Returns the status of the request to the agent.
   *
   * @exception java.io.IOException Thrown when an error is
   *    encountered reading from the socket (e.g., broken pipe).
   *
   **/

  private int getServers(String server,Problem prob) 
   throws java.io.IOException {
     int prob_size, data_size, result_size, status,
         num_servers, i, temp[], encoding;
     temp = new int[4];
   
     
     // First, try to make connection to the agent (always XDR)

     str = new NetSolveStream(server,GlobalDefs.AG_PORT,GlobalDefs.DATA_XDR,order,gui);

     if(GlobalDefs.DEBUG)
       System.out.println("Sending some data to the server..");

     str.server_out.writeInt(GlobalDefs.PROBLEM_REQUEST);
     str.server_out.writeInt(GlobalDefs.JAVA_PROG);        // send client type
     str.writeString(prob.name);                           // send problem name

     // some junk values (for now)
     prob_size = 1;
     data_size = 1;
     result_size = 1;

     str.server_out.writeInt(prob_size);   // send problem size
     str.server_out.writeInt(data_size);   // send data size
     str.server_out.writeInt(result_size); // send result size

     str.server_out.writeInt(prob.nb_input);  // send number of inputs

     for(i=0;i<prob.nb_input;i++) {
       str.server_out.writeInt(prob.inputs[i].data_type);   // send data type
       str.server_out.writeInt(prob.inputs[i].object_type); // send object type
     }
       
     str.server_out.writeInt(prob.nb_output); // send number of outputs

     for(i=0;i<prob.nb_output;i++) {
       str.server_out.writeInt(prob.outputs[i].data_type);   // send data type
       str.server_out.writeInt(prob.outputs[i].object_type); // send object type
     }

     str.flushOutput();

     if(GlobalDefs.DEBUG) {
       System.out.println("...done.");
       System.out.println("Response from server:");
     }

     /*
      * get the encoding that we need to use and initialize the
      * stream to receive in that mode.
      */

     encoding = str.server_in.readShort();
     str.initRecv(encoding);

     if(GlobalDefs.DEBUG)
       System.out.println("encoding = " + encoding);

     status = str.server_in.readInt();

     if(GlobalDefs.DEBUG)
       System.out.println("status = " + status);
     
     if((status == GlobalDefs.NO_SUCH_PB) || (status == GlobalDefs.BAD_INPUT) || 
        (status == GlobalDefs.BAD_OUTPUT) || (status == GlobalDefs.SV_FAILURE)) {

           // we got a bad return status, shutdown the connection and return

           str.shutDown();
           return status;
     }

     num_servers = str.server_in.readInt();    // get number of servers
     servers = new ServerInfo[num_servers];

     // for each server, read information about that server...

     for(i=0;i<num_servers;i++) {
       servers[i] = new ServerInfo();
       servers[i].name = str.getString();
       servers[i].ip_addr = str.readIPaddr();
       servers[i].data_format = str.server_in.readInt();
       servers[i].expected = str.server_in.readInt();
     }

     // finally, shut down the connection and return
     str.shutDown();
     return status;
  }

  /**
   *
   * This method sends the problem information to the computational
   * server.  We assume that the connection has already been made
   * prior to calling this method.
   * 
   * @param prob The problem to send.
   *
   * @return Returns the status of the problem submission.
   *
   * @exception java.io.IOException Thrown when an error is
   *    encountered reading from the socket (e.g., broken pipe).
   *
   **/

  private int sendProb(Problem prob) throws java.io.IOException {
    int i, encoding, status;

    str.server_out.writeInt(GlobalDefs.PROBLEM_SOLVE);  // send PROBLEM_SOLVE
    str.writeString(prob.name);                         // send problem name
    str.writeString(hostname);                          // send agent name
    str.server_out.writeInt(GlobalDefs.JAVA_PROG);      // send client type (java)

    str.server_out.writeInt(prob.nb_input);                 // num inputs
    
    for(i=0;i<prob.nb_input;i++) {
      str.server_out.writeInt(prob.inputs[i].data_type);    // data type
      str.server_out.writeInt(prob.inputs[i].object_type);  // object type
    }

    str.server_out.writeInt(prob.nb_output);                // num outputs
    for(i=0;i<prob.nb_output;i++) {
      str.server_out.writeInt(prob.outputs[i].data_type);   // data type
      str.server_out.writeInt(prob.outputs[i].object_type); // object type
    }

    str.flushOutput();

    if(GlobalDefs.DEBUG)
      System.out.println("Response from server:");

    // get the encoding that we need to use and initialize the
    // stream to receive in that mode.

    encoding = str.server_in.readShort();
    str.initRecv(encoding);

    if(GlobalDefs.DEBUG)
      System.out.println("encoding = " + encoding);

    status = str.server_in.readInt();

    if(GlobalDefs.DEBUG)
      System.out.println("status = " + status);

    return status;
  }

/*********** getInfo not really necessary for this interface *******

  private ProblemDesc getInfo(String server,String name) {
    int encoding, status, i, dummy, dummy2;
    ProblemDesc pb_desc;
    Constant current_constant;
    Formula current_form;

    try {
      str = new NetSolveStream(server,GlobalDefs.AG_PORT,GlobalDefs.DATA_RAW,order,gui);
    }
    catch (IOException e) {
      return null;
    }

    try {
      pb_desc = new ProblemDesc();

      str.server_out.writeInt(GlobalDefs.INFO_REQUEST); // send INFO_REQUEST
      str.writeString(name);                            // send problem name
    
      str.flushOutput();

      if(GlobalDefs.DEBUG)
        System.out.println("info_request sent...response from server:");

      encoding = str.server_in.readShort();
      if(GlobalDefs.DEBUG)
        System.out.println("encoding = " + encoding);
      status = str.server_in.readInt();
      if(GlobalDefs.DEBUG)
        System.out.println("status = " + status);

      if(status == GlobalDefs.NO_SUCH_PB) {
        if(GlobalDefs.DEBUG)
          System.out.println("No Such Problem");
        return null;
      }

      pb_desc.path = str.getString();
      pb_desc.desc = str.getString();
      pb_desc.num_upf = str.server_in.readInt();
      pb_desc.nb_obj_input = str.server_in.readInt();

      pb_desc.input_obj = new NetSolveObject[pb_desc.nb_obj_input];
      for(i=0;i<pb_desc.nb_obj_input;i++) {
        pb_desc.input_obj[i] = new NetSolveObject();
        pb_desc.input_obj[i].data_type = str.server_in.readInt();      
        pb_desc.input_obj[i].object_type = str.server_in.readInt();
        pb_desc.input_obj[i].description = str.getString();
      }

      pb_desc.nb_obj_output = str.server_in.readInt();
      pb_desc.output_obj = new NetSolveObject[pb_desc.nb_obj_output];

      for(i=0;i<pb_desc.nb_obj_output;i++) {
        pb_desc.output_obj[i] = new NetSolveObject();
        pb_desc.output_obj[i].data_type = str.server_in.readInt();      
        pb_desc.output_obj[i].object_type = str.server_in.readInt();
        pb_desc.output_obj[i].description = str.getString();
      }

      // only allocate one format, as in the C version
      pb_desc.formats = new FormatSpec[1];
      pb_desc.formats[0] = new FormatSpec();
      pb_desc.formats[0].nb_args = str.server_in.readInt();
      pb_desc.formats[0].args_desc = 
          new String[pb_desc.formats[0].nb_args];
      for(i=0;i<pb_desc.formats[0].nb_args;i++) {
        pb_desc.formats[0].args_desc[i] = str.getString();
      }

      pb_desc.formats[0].nb_constants = str.server_in.readInt();
      pb_desc.formats[0].consts = null;

      for(i=0;i<pb_desc.formats[0].nb_constants;i++) {
        current_constant = new Constant();
        current_constant.next = pb_desc.formats[0].consts;
        current_constant.mnemonic = str.getString();
        current_constant.value = str.server_in.readInt();
        pb_desc.formats[0].consts = current_constant;
      }

      pb_desc.formats[0].nb_formulae = str.server_in.readInt();
      pb_desc.formats[0].forms = null;

      for(i=0;i<pb_desc.formats[0].nb_formulae;i++) {
        current_form = new Formula();
        current_form.next = pb_desc.formats[0].forms;
        current_form.mnemonic = str.getString();
        current_form.expression = str.getString();
        pb_desc.formats[0].forms = current_form;
      }

      // ignore matlab information
      dummy = str.server_in.readInt();
      for(i=0;i<dummy;i++) {
        dummy2 = str.server_in.readInt();
        dummy2 = str.server_in.readInt();
      }
      pb_desc.name = new String(name);
    } 
    catch (IOException e) {
      return null;
    }
    finally {
      try {
        if (str != null)  
          if (str.s != null) 
            str.shutDown(); 
      }
      catch (IOException e2) { ; }
    }
  
    if(GlobalDefs.DEBUG)
      System.out.println("That's it for getInfo...");
    return pb_desc;
  }

**************** end getInfo *******************************/

  /**
   *
   * This method sends the user's data to the computational server.
   * 
   * @param prob The problem to be solved.
   * @param obj The objects containing the user's data.
   *
   * @exception java.io.IOException Thrown when an error is
   *    encountered reading from the socket (e.g., broken pipe).
   * @exception NetSolveException Thrown when the the UPF fails to 
   *    compile.
   * @exception UpfUnsafeException Thrown when the UPF compiles, but
   *    contains unsafe function calls.
   *
   **/

  private void sendData(Problem prob, NetSolveObject [] obj) 
   throws java.io.IOException, NetSolveException, UpfUnsafeException 
  {
    int upf_type, unsafe_index, encoding, status, i, j, k, count=0;
    double answer;
    String upf_code, upf_name;
    String unsafe_calls;
    
    if(prob.upf > 0) {   // if there are UPFs to send...

      if(GlobalDefs.DEBUG)
        System.out.println("There are " + prob.upf + "UPFs to send...");

      if(current_data_format == my_data_format) {
        if(GlobalDefs.DEBUG)
          System.out.println("Sending UPF using DATA_RAW");
        str.server_out.writeChar(GlobalDefs.DATA_RAW);         // send encoding
      }
      else {
        if(GlobalDefs.DEBUG)
          System.out.println("Sending UPF using DATA_XDR");
        str.server_out.writeChar(GlobalDefs.DATA_XDR);         // send encoding
      }

      // The UPFs may be in any position, so just loop through each 
      // input object and determine whether it is a UPF.

      for(j = 0;j < prob.nb_input; j++) {
        if(obj[j] instanceof NetSolveUPF) {
          if(GlobalDefs.DEBUG)
            System.out.println("Sending UPF number " + count++);

          obj[j].writeData(str);
        }
      }

      str.flushOutput();

      /*
       * The UPFs have been sent and before we try to send the rest of the
       * data, we must get the return code to make sure that the UPFs 
       * compiled successfully.
       */

      if(GlobalDefs.DEBUG)
        System.out.println("Response from server:");

      encoding = str.server_in.readShort();
      str.initRecv(encoding);

      if(GlobalDefs.DEBUG)
        System.out.println("encoding = " + encoding);
  
      status = str.server_in.readInt();

      if(GlobalDefs.DEBUG)
        System.out.println("status = " + status);

      if(status == GlobalDefs.UPF_FAILED)
        throw new NetSolveException(Labels.upf_failed[cur_lang]);
      else if(status == GlobalDefs.UPF_UNSAFE) {
        unsafe_index = str.server_in.readInt();
        unsafe_calls = str.getString();
        throw new UpfUnsafeException("UPF"+unsafe_index+": "+unsafe_calls);
      }
    }

    /*
     * UPFs have been sent and compiled, now move on to the
     * rest of the data.
     */

    // if UPF, are we resending encoding here?  seems to work.
    if(current_data_format == my_data_format) {
      if(GlobalDefs.DEBUG)
        System.out.println("Sending data using DATA_RAW");

      str.server_out.writeChar(GlobalDefs.DATA_RAW);         // send encoding
    }
    else {
      if(GlobalDefs.DEBUG)
        System.out.println("Sending data using DATA_XDR");

      str.server_out.writeChar(GlobalDefs.DATA_XDR);         // send encoding
    }

    if(GlobalDefs.DEBUG)
      System.out.println("sending "+ (prob.nb_input - prob.upf) +
        " input objs, not counting upf");

    // Send the non-UPF data - just call the appropriate write method
    for(i=0;i<prob.nb_input;i++) {
      this.yield();
      if(!(obj[i] instanceof NetSolveUPF))
        obj[i].writeData(str);
    }
    str.flushOutput();

    encoding = str.server_in.readShort();
    str.initRecv(encoding);
    int tmp = str.server_in.readInt();                            // ignore PID 
System.out.println("PID = " + tmp);
    str.server_out.writeChar(encoding);                 // write encoding
System.out.println("wrote encoding");
    str.server_out.writeInt(666);                       // write dummy val
System.out.println("wrote dummy val");
    str.flushOutput();
  }

  /**
   *
   * After the data has been submitted, this method is called to
   * get the results of the computation.
   * 
   * @param prob The problem that was solved.
   *
   * @return Returns a vector containing the results of the computation.
   *
   * @exception java.io.IOException Thrown when an error is
   *    encountered reading from the socket (e.g., broken pipe).
   * @exception NetSolveException Thrown when there is some error
   *    in solving the problem (e.g., dimension mismatch).
   *
   **/

  private NetSolveObject [] getAnswer(Problem prob) throws java.io.IOException, 
   NetSolveException {
    int i, j, encoding, status, stdoutlength;
    NetSolveObject [] obj;
    String stdoutText = null;
    
    if(GlobalDefs.DEBUG)
      System.out.println("done.  response from server:");

    encoding = str.server_in.readShort();
    str.initRecv(encoding);

    if(GlobalDefs.DEBUG)
      System.out.println("encoding = " + encoding);

    status = str.server_in.readInt();

    if(GlobalDefs.DEBUG)
      System.out.println("status = " + status);

    switch(status) {
      case GlobalDefs.SOLUTION:
        // this case is good - we got a solution, just continue
        break;
      case GlobalDefs.NO_SOLUTION:
        throw new NetSolveException(Labels.no_solution[cur_lang]);
      case GlobalDefs.LINEAR_FAILED:
        throw new NetSolveException(Labels.linear_failed[cur_lang]);
      case GlobalDefs.DM_MISMATCH:
        throw new NetSolveException(Labels.dm_mismatch[cur_lang]);
      case GlobalDefs.BAD_VALUES:
        throw new NetSolveException(Labels.bad_values[cur_lang]);
      case GlobalDefs.NO_SUCH_PB:
        return(null);
      case GlobalDefs.BAD_UPF:
        throw new NetSolveException(Labels.bad_upf[cur_lang]);
      case GlobalDefs.UPF_UNSAFE:
        throw new NetSolveException(Labels.upf_unsafe[cur_lang]);
      default:
        throw new NetSolveException(Labels.unknown_err[cur_lang]);
    }

    stdoutlength = str.server_in.readInt();

    if(GlobalDefs.DEBUG)
      System.out.println("trying to get answer...");

    /*
     * The results are available, initialize an array to hold the
     * data and read it all in.  If we have stdout text to display,
     * add an extra element to the array.  stdoutlength should 
     * always be at least one, but if not, we wont bother trying
     * to read anything.
     */

    if(stdoutlength > 0)
    {
      if(GlobalDefs.DEBUG)
        System.out.println("going to read the stdout text (length = " +
           stdoutlength + ")");

      stdoutText = str.getString(stdoutlength);
    }

    if(stdoutlength > 1)
      output_obj = new NetSolveObject[prob.nb_output + 1];
    else
      output_obj = new NetSolveObject[prob.nb_output];

    for(i=0;i < prob.nb_output;i++) {
      output_obj[i] = NetSolveObject.newObject(prob.outputs[i].data_type, 
           prob.outputs[i].object_type);
      output_obj[i].readData(str);
    }

    if(stdoutlength > 1) {
      ByteArrayInputStream b = new ByteArrayInputStream(stdoutText.getBytes());
      output_obj[i] = NetSolveObject.newObject(
        NetSolveObject.NETSOLVE_ASCII,NetSolveObject.NETSOLVE_FILE);
      output_obj[i].initialize(b,this);
    }

    return(output_obj);
  }

  /**
   *
   * Any time a computational server fails to solve the problem,
   * send a failure report to the agent mentioning the error.
   * This method attempts to contact the agent and deliver the
   * failure report.
   * 
   * @param host The agent to contact.
   * @param addr The IP address of the failed server.
   * @param type The type of failure.
   *
   **/

  private void reportFailure(String host, int addr, int type) {
    if(GlobalDefs.DEBUG)
      System.out.println("Sending failure report for host: " + host);
    try {
      // always XDR
      str = new NetSolveStream(host,GlobalDefs.AG_PORT,GlobalDefs.DATA_XDR,order,gui);

      str.server_out.writeInt(GlobalDefs.CLIENT_F_REPORT); // send fail report
      str.server_out.writeInt(type);                       // send type
      str.writeIPaddr(addr);                               // send IP addr

      str.shutDown();    

      if(GlobalDefs.DEBUG)
        System.out.println("*** It WORKED ***");

    } catch (IOException e) {
      if(GlobalDefs.DEBUG) {
        System.out.println("*** HERE ***");
        System.out.println(e.getMessage());
      }
      return;
    }
  }

  /**
   *
   * Each time a problem is successfully solved, notify the agent
   * so that it can update its workload information.  This method
   * attempts to contact the agent and deliver the job completion
   * message.
   * 
   * @param host The name of the agent.
   * @param addr The IP address of the server that solved the problem.
   *
   **/

  private void jobDone(String host, int addr) {
    try {
      // always XDR
      str = new NetSolveStream(host,GlobalDefs.AG_PORT,GlobalDefs.DATA_XDR,order,gui);

      str.server_out.writeInt(GlobalDefs.JOB_COMPLETED);  // send JOB_COMPLETED
      str.writeIPaddr(addr);                              // send IP address
      str.shutDown();    
    } catch (IOException e) {
      return;
    }
  }

  /**
   *
   * This is the constructor for the SubmitJob class.  It just copies the
   * parameters to class variables, starts this class as a thread, and returns
   * to the caller.  The parameters are badly named, but they are only used
   * once each in this constructor to initialize the class variables.
   *
   * @param a The parent frame.
   * @param b The host name of the agent.
   * @param c The problem to solve.
   * @param d The vector of input data.
   * @param g Identification string.
   * @param i Current orientation of matrices.
   *
   * @exception NetSolveException Actually thrown by other methods, but
   *   since we start the thread via this method, the compiler likes for
   *   NetSolveException to be listed here.
   **/

  public SubmitJob(InputData a, String b, Problem c, NetSolveObject [] d,
      String g, int i) throws NetSolveException {

    f = a;
    input_frame = (InputData) a;
    hostname = b;
    prob = c;
    obj = d;
    idString = g;
    order = i;

    gui = input_frame != null;
    addActionListener(input_frame);
  }

  /**
   *
   * This method is called when the class is started as a thread.
   * It first gets the list of servers that can solve the problem.
   * It loops through the server list trying each server until one
   * solves the problem or until the list is exhausted.  Once a
   * server has accepted the problem, the data is sent and the
   * results are retrieved.  Finally, if we're running graphically
   * (not through the API), start up the Output Screen so that the
   * user can view the results.
   * 
   **/

  public void run() {
    int i,resp=0, status;
    boolean made_connection = false;
    ProblemDesc pd = null;
    NetSolveWarning warn;

    // Keep looping until we make a connection or there are no more servers..
    while(!made_connection) {
      if(gui)  
        input_frame.setStatusText(Labels.contacting[cur_lang]);

      try {
        status = getServers(hostname,prob);   // grab a list of servers

        if((status == GlobalDefs.NO_SUCH_PB) || 
           (status == GlobalDefs.BAD_INPUT)  || 
           (status == GlobalDefs.BAD_OUTPUT) ||  
           (status == GlobalDefs.SV_FAILURE))
        {
          if(f != null)
            warn = new NetSolveWarning(f, Labels.internal_err[cur_lang]);
          else
            err_string = new String(Labels.internal_err[cur_lang]);
          return;
        }
      }
      catch(IOException e) {
        if(GlobalDefs.DEBUG)
          System.out.println("Cant contact agent: "+hostname);

        if(f != null) 
          warn = new NetSolveWarning(f,Labels.contact_err[cur_lang] + 
             "'" + hostname + "'" + " -- (" + e + ")");
        else
          err_string = new String(Labels.contact_err[cur_lang] + 
             "'" + hostname + "'" + " -- (" + e + ")");

        return;
      }

      if(servers.length == 0) {
        if(GlobalDefs.DEBUG)
          System.out.println("No servers available......");

        if(f != null)
          warn = new NetSolveWarning(f,Labels.no_servers[cur_lang]);
        else
          err_string = new String(Labels.no_servers[cur_lang]);
  
        return;
      }

      if(GlobalDefs.DEBUG) {
        System.out.println("got a list of "+servers.length+" servers: ");
        for(i=0;i<servers.length;i++) {
          System.out.println(i+": "+ servers[i].name);
          System.out.println( 
              (long)  ((servers[i].ip_addr >> 24) & 0xFF) + "." +
                      ((servers[i].ip_addr >> 16) & 0xFF) + "." +
                      ((servers[i].ip_addr >>  8) & 0xFF) + "." +
                       (servers[i].ip_addr & 0xFF));
          System.out.println("data format: "+servers[i].data_format);
          System.out.println("expected: "+servers[i].expected);
        }
      }

      if(GlobalDefs.DEBUG)
        System.out.println("Going to contact a server.");

      for(i=0;i<servers.length;i++) {
        this.yield();   // give other threads a chance to run

        if(GlobalDefs.DEBUG)
          System.out.println("Setting current_data_format to: "+
             servers[i].data_format);

        current_data_format = servers[i].data_format;
        if(gui)
          input_frame.setStatusText(Labels.trying[cur_lang] + servers[i].name);

        if(GlobalDefs.DEBUG)
          System.out.println("Trying "+servers[i].name+"...");

        // try to contact this server
        try {
          if(current_data_format == my_data_format) {
            if(GlobalDefs.DEBUG)
              System.out.println("...in DATA_RAW mode.");
            str = new NetSolveStream(servers[i].name,GlobalDefs.SV_PORT,
              GlobalDefs.DATA_RAW,order,gui);
          } else {
            if(GlobalDefs.DEBUG)
              System.out.println("...in DATA_XDR mode.");
            str = new NetSolveStream(servers[i].name,GlobalDefs.SV_PORT,
              GlobalDefs.DATA_XDR,order,gui);
          }
        }
        catch (IOException e) {
          // if something goes wrong contacting this server, send a failure report
          reportFailure(hostname, servers[i].ip_addr, GlobalDefs.STOPPED);
          continue;
        }

        if(GlobalDefs.DEBUG)
          System.out.println("Made a connection to: "+servers[i].name);

        if(gui)
          input_frame.setStatusText(Labels.connected[cur_lang] + servers[i].name);

        // A connection has been made to the server, now send the problem
        try {
          resp = sendProb(prob);
          if((resp == GlobalDefs.PB_REFUSED) || (resp == GlobalDefs.SV_FAILURE)) {
            // either the server refused the problem or something is wrong with
            // the server - either way, send a failure report

            reportFailure(hostname, servers[i].ip_addr, GlobalDefs.FAILED);
            continue;
          }
          else if(resp == GlobalDefs.TOKENS_OVERFLOW) {
            NetSolveWarning cant_solve = new NetSolveWarning(f,
              Labels.overflow_error[cur_lang]);
          }
          else if(resp == GlobalDefs.NOT_AUTHORIZED) {
            NetSolveWarning cant_solve = new NetSolveWarning(f,
              Labels.authorized_error[cur_lang]);
          }
        }
        catch (IOException e) {
          reportFailure(hostname, servers[i].ip_addr, GlobalDefs.FAILED);
          continue;
        }

        if(gui)
          input_frame.setStatusText(Labels.sending[cur_lang]);

        // The server accepted the problem, now send the data
        try {
          sendData(prob,obj);

          if(gui)
            input_frame.setStatusText(Labels.waiting[cur_lang]);

          output_obj = getAnswer(prob);   // get answer

          if(output_obj == null)  // if no data returned
            continue;

        }catch (IOException e) {
          if(GlobalDefs.DEBUG)
            System.out.println("IO exception on sendData or getAnswer: " +
               e.getMessage());

          reportFailure(hostname, servers[i].ip_addr, GlobalDefs.FAILED);

          if(GlobalDefs.DEBUG)
            System.out.println("reported failure of " + servers[i].ip_addr);
          continue;
        }catch(NetSolveException e) {
          // what to do here?
          if(GlobalDefs.DEBUG)
            System.out.println(e.getMessage());

          if(f != null) {
            NetSolveWarning cant_solve = new NetSolveWarning(f,
              Labels.solve_error[cur_lang]+e.getMessage());
          }
          else
            err_string = new String( Labels.solve_error[cur_lang]
                + e.getMessage());

        }catch(UpfUnsafeException e) {
          TextScreen unsafe = new TextScreen(Labels.upf_unsafe[cur_lang],
            "One or more of your User Provided Functions cannot be\n" +
            "compiled because of unsafe function calls.  The following\n" +
            "is a list of the unsafe calls in your code:\n\n" + 
            e.getMessage());
        }

        // Now that the results are in, send a job completion message
        // to the agent.
        jobDone(hostname,servers[i].ip_addr);

        made_connection = true;

        try {
          str.shutDown();
        }
        catch(IOException e) { ; }
        break; // break out of for loop
      }
      if(GlobalDefs.DEBUG)
        System.out.println("after for loop");
    }
    if(GlobalDefs.DEBUG)
      System.out.println("after while loop");

    if(made_connection) {
      if(GlobalDefs.DEBUG)
        System.out.println("SubmitJob done...calling processEvent");

      this.setCompleted(true);
      processEvent(
        new ActionEvent(this,(int)AWTEvent.ACTION_EVENT_MASK,"hi there"));
    }
  }
}  // end class SubmitJob
