/**
* ThreadList
* 
* This class controls the graphical user interface for the
* Thread Information Screen.
* 
* @version 1.1, 26 Jan 1998 
* @author Keith Seymour (seymour@cs.utk.edu)
*
**/

import java.awt.Frame;
import java.awt.Event;
import java.awt.Button;
import java.awt.Panel;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.List;
import java.awt.Label;
import java.awt.event.*;
import java.util.Vector;

public class ThreadList extends Frame
 implements ActionListener, ItemListener {
  private Button  close_button;
  private Button  update_button;
  private Button  kill_button;

  private Label running_label;
  private Label stopped_label;

  private List running_threads;
  private List stopped_threads;

  private InputData parent;
  private Vector threadVec;

  public int cur_lang = Labels.language;

  private final int LABEL_SIZE = 14;

  /**
   *
   * This is the constructor for the ThreadList class.  The purpose of
   * this method is to initialize and layout all the graphical components
   * in the Thread Information Screen.  This is all done with standard 
   * AWT components.
   * 
   * @param p The parent frame.
   *
   **/

  public ThreadList(InputData p) {

    super();
    super.setTitle(Labels.thread_title[cur_lang]);
    this.setLayout(new BorderLayout(5,5));

    parent = p;

    // define panel to hold the two buttons
    Panel button_panel = new Panel();
    button_panel.setLayout(new FlowLayout(FlowLayout.CENTER,15,15));

    close_button = new Button(Labels.close[cur_lang]);
    close_button.addActionListener(this);
    close_button.setFont(new Font("TimesRoman",Font.BOLD,LABEL_SIZE));

    update_button = new Button(Labels.update[cur_lang]);
    update_button.addActionListener(this);
    update_button.setFont(new Font("TimesRoman",Font.BOLD,LABEL_SIZE));

    kill_button = new Button(Labels.kill[cur_lang]);
    kill_button.addActionListener(this);
    kill_button.setFont(new Font("TimesRoman",Font.BOLD,LABEL_SIZE));

    button_panel.add(kill_button);
    button_panel.add(update_button);
    button_panel.add(close_button);

    running_threads = new List(20);
    running_threads.addItemListener(this);

    stopped_threads = new List(20);
    stopped_threads.addItemListener(this);

    running_label = new Label(Labels.running[cur_lang]);
    running_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_SIZE));

    stopped_label = new Label(Labels.stopped[cur_lang]);
    stopped_label.setFont(new Font("Helvetica",Font.BOLD,LABEL_SIZE));

    Panel running_panel = new Panel();
    running_panel.setLayout(new BorderLayout(5,5));

    running_panel.add("North",running_label);
    running_panel.add("South",running_threads);

    Panel stopped_panel = new Panel();
    stopped_panel.setLayout(new BorderLayout(5,5));

    stopped_panel.add("North",stopped_label);
    stopped_panel.add("South",stopped_threads);

    this.add("West",running_panel);
    this.add("East",stopped_panel);
    this.add("South",button_panel);

    handleUpdate();

    this.pack();
    this.setVisible(true);
    
  }

  /**
   *
   * This method is called when the user clicks the 'close' 
   * button.  Get rid of this window.
   *
   **/

  private void handleClose()
  {
    this.setVisible(false);
    this.dispose(); 
  }

  /**
   *
   * This method is called when the user clicks the 'update'
   * button.  It determines which threads are still running
   * and updates the display.  It is also called from InputData
   * whenever a thread completes.
   *
   **/

  public synchronized void handleUpdate()
  {
    NetSolveThread t;

    threadVec = parent.getThreadList();
    running_threads.removeAll();
    stopped_threads.removeAll();

    for(int i=0; i < threadVec.size(); i++)
      if( (t = (NetSolveThread)threadVec.elementAt(i)) != null) {
        if( !t.isAlive() || t.completed())
          stopped_threads.addItem( t.getName() );
        else
        {
          ((NetSolveThread)threadVec.elementAt(i)).addActionListener(this);
          running_threads.addItem( t.getName() );
        }
      }
  }

  private int findThread(String name) {
    Thread t;

    for(int i=0; i < threadVec.size(); i++)
      if( (t = (Thread)threadVec.elementAt(i)) != null)
        if(t.getName().equalsIgnoreCase(name))
          return i;

    return -1;
  }

  private void handleKill()
  {
    NetSolveWarning nw;
    int selected = running_threads.getSelectedIndex();
    String threadName;
    Thread t;

    if(selected == -1)
      nw = new NetSolveWarning(this,Labels.invalid_thread[cur_lang]);
    else if( (threadName = running_threads.getItem(selected)) == null )
      nw = new NetSolveWarning(this,Labels.no_thread[cur_lang]);
    else if( threadName != null ) {

      if( (selected = findThread(threadName)) == -1)
        nw = new NetSolveWarning(this,Labels.invalid_thread[cur_lang]);
      else if(threadVec.elementAt(selected) == null)
        nw = new NetSolveWarning(this,Labels.no_thread[cur_lang]);
      else {
        t = (Thread) threadVec.elementAt(selected);
     
        if(threadVec.elementAt(selected) instanceof SubmitJob)
        {
          System.out.println("going to try to shut down the socket..");
          ((SubmitJob) threadVec.elementAt(selected)).shutDown();
        }

        t.stop();

        if(t.isAlive())
          System.out.println("Thread not stopped"); 
        else 
          System.out.println("Thread stopped");
      }
    }
  }

  private void handleRunningSelection() {
    int selected = running_threads.getSelectedIndex();
  
    if(stopped_threads.getSelectedIndex() >= 0)
      stopped_threads.deselect(stopped_threads.getSelectedIndex());
  }

  private void handleStoppedSelection() {
    int selected = stopped_threads.getSelectedIndex();
  
    if(running_threads.getSelectedIndex() >= 0)
      running_threads.deselect(running_threads.getSelectedIndex());
  }

  /**
   *
   * This is the method which is called whenever an event happens.
   * First, determine what kind of event this is.  Next, determine the
   * target of this event.  Finally, execute the code associated with
   * the event.
   *
   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent e) {
    Object source = e.getSource();
 
    if (source == close_button)
      handleClose();
    else if (source == update_button)
      handleUpdate();
    else if (source == kill_button)
      handleKill();
  }

  public void itemStateChanged(ItemEvent e) {
    Object source = e.getSource();

    if (source == running_threads)
      handleRunningSelection();
    else if (source == stopped_threads)
      handleStoppedSelection();
  }

} // end class ThreadList
