/*****************************************************************/
/*      matlab_send.c                                            */
/*      Version 1.0                      Henri Casanova          */
/*****************************************************************/

#include "mex.h"
#include "interface.h"
#include "matlab_netsolve.h"

#define BYTE(x,y) (((unsigned char *)x)[(y)]&0xff)

int is_sigpipe;

/*
 * matlab_send()
 */
        
void matlab_send(int nlhs, mxArray *plhs[], int nrhs, mxArray *prhs[])
{
  int client_type = MATLAB;
  char code[CODE_LENGTH];
  int info,status;
  problem_desc *pb_desc;
  input_sizes *sizes_input;
  output_sizes *sizes_output;
  void **data_input, **data_output;
  double *returned_value;
#ifdef MATLABTHREADS
  thread_input *arguments;
  void *thread_do();
  int slot;
#endif


  int i,j;
  int length;
  double *ptr,*pti;

  
  /*
   * list of problems
   */
  if (nrhs == 0)
  {
    mexPrintf("More arguments needed\n");
    return;
  }

  if (!mxIsChar(prhs[0]))
  {
    mexPrintf("Invalid second parameter : should be a string\n");
    return;
  }
  mxGetString(prhs[0],code,CODE_LENGTH);
  pb_desc = (problem_desc *)mxCalloc(1,sizeof(problem_desc));

  mexMakeMemoryPersistent(pb_desc);

  strcpy(pb_desc->code,code);

  if (nlhs != 1)
  {
    mexPrintf("NetSolve : There should be one output argument\n");
    mexPrintf("           to catch the request number\n");
    mxFree(pb_desc);
    return;
  }
  plhs[0] = mxCreateDoubleMatrix(1,1,mxREAL);
  returned_value = mxGetPr(plhs[0]);

  mexnetslmajor(COL_MAJOR);

  /*
   * Real Netsolve stuff
   */

  status = mxnetsolve_info(code,pb_desc);

  if (status < 0)
  {
    mxFree(pb_desc);
    *returned_value = -1;
    return;
  }

  if ((nrhs-1) != pb_desc->nb_obj_input)
  {
    mexPrintf("Illegal number of input, should be :\n");
    for (i=0;i<pb_desc->nb_obj_input;i++)
    {
      mexPrintf("\t #%d : ",i);
      switch(pb_desc->input_obj[i].data_type)
      {
        case NETSOLVE_ASCII :
          mexPrintf("ASCII ");
          break;
        case NETSOLVE_I :
          mexPrintf("Integer ");
          break;
        case NETSOLVE_B :
          mexPrintf("Byte ");
          break;
        case NETSOLVE_CHAR :
          mexPrintf("Character ");
          break;
        case NETSOLVE_S :
          mexPrintf("Single precision real ");
          break;
        case NETSOLVE_D :
          mexPrintf("Double precision real ");
          break;
        case NETSOLVE_C :
          mexPrintf("Single precision complex ");
          break;
        case NETSOLVE_Z :
          mexPrintf("Double precision complex ");
          break;
        case NETSOLVE_EXTERNAL :
          mexPrintf("");
          break;
        default :
          mexPrintf("??? ");
          break;
      }
      switch(pb_desc->input_obj[i].object_type)
      {
        case NETSOLVE_FILE :
          mexPrintf("File.\n");
          break;
        case NETSOLVE_MATRIX :
          mexPrintf("Matrix.\n");
          break;
        case NETSOLVE_VECTOR :
          mexPrintf("Vector.\n");
          break;
        case NETSOLVE_SCALAR :
          mexPrintf("Scalar.\n");
          break;
        case NETSOLVE_UPF :
          mexPrintf("User Provided Function.\n");
          break;
        default :
          mexPrintf("???.\n");
          break;
      }
    }
    mxFree(pb_desc);
    *returned_value = -1;
    return;
  }

  sizes_input =
     (input_sizes *)mxCalloc(sizeof(input_sizes),pb_desc->nb_obj_input);
  sizes_output =
     (output_sizes *)mxCalloc(sizeof(input_sizes),pb_desc->nb_obj_output);
  mexMakeMemoryPersistent(sizes_input);
  mexMakeMemoryPersistent(sizes_output);
  for (i=0;i<pb_desc->nb_obj_output;i++)
  {
    sizes_output[i].m = (int *)mxCalloc(sizeof(int),1);
    sizes_output[i].n = (int *)mxCalloc(sizeof(int),1);
    mexMakeMemoryPersistent(sizes_output[i].m);
    mexMakeMemoryPersistent(sizes_output[i].n);
  }

  data_input = (void **)mxCalloc(sizeof(void *),pb_desc->nb_obj_input);
  data_output = (void **)mxCalloc(sizeof(void *),pb_desc->nb_obj_output);

  mexMakeMemoryPersistent(data_input);
  mexMakeMemoryPersistent(data_output);

  /*
   * Going from MATLAB to NetSolve data representation
   */

  for (i=0;i< pb_desc->nb_obj_input;i++)
  {
    int m,n;
    double *pr,*pi;
    
    m = mxGetM(prhs[i+1]);
    n = mxGetN(prhs[i+1]);
    pr = mxGetPr(prhs[i+1]);
    pi = mxGetPi(prhs[i+1]);

    if ((m==0)||(n==0))
    {
      mexPrintf("input '%s' empty !!\n",mxGetName(prhs[i+1]));
      *returned_value = -1;
      return;
    }
    switch(pb_desc->input_obj[i].object_type)
    {
      case NETSOLVE_VECTOR :
      {
        if (n != 1)
        {
          mexPrintf(
                  "Warning : input '%s' seems to be more than a vector\n",
                  mxGetName(prhs[i+1]));
          n = 1;
        }
     
        break;
      }
      case NETSOLVE_SCALAR :
      {
        if ((n!=1)||(m!= 1))
        {
          mexPrintf(
                  "Warning : input #%d seems to be more than a scalar\n",
                  mxGetName(prhs[i+1]));
          m = n = 1;
        }
        break;
      }
      case NETSOLVE_UPF :
      {
        if (m != 1)
        {
           mexPrintf(
                "Warning : input #%d appears not to be a string\n",i);
        }
        m = 1;
      }
      case NETSOLVE_FILE :
      {
        struct stat st;
        if (m != 1)
        {
           mexPrintf(
                "Warning : input #%d appears not to be a file name\n",i);
        }
      }
    }
    sizes_input[i].m = m; 
    sizes_input[i].n = n; 
    sizes_input[i].l = m; 
    switch(pb_desc->input_obj[i].data_type)
    {
      case NETSOLVE_I :
      {
        int k;
        int *tmp;

        if (mxIsChar(prhs[i+1]))
        {
          mexPrintf("input #%d should not be a string",i);
          *returned_value = -1;
          return;
        }
        tmp = (int *)mxCalloc(sizeof(int),m*n);
        mexMakeMemoryPersistent(tmp);
        for (k=0;k<m*n;k++)
          tmp[k] = (int)pr[k];
        data_input[i] = tmp;
        break;
      }
      case NETSOLVE_B :
      {
        int k;
        char *tmp;

        tmp = (char *)mxCalloc(sizeof(char),m*n+1);
        mexMakeMemoryPersistent(tmp);
        mxGetString(prhs[i+1],tmp,m*n+1);
        data_input[i] = tmp;
        break;
      }
      case NETSOLVE_CHAR :
      {
        int k;
        char *tmp;

        tmp = (char *)mxCalloc(sizeof(char),m*n+1);
        mexMakeMemoryPersistent(tmp);
        mxGetString(prhs[i+1],tmp,m*n+1);
        data_input[i] = tmp;
        break;
      }
      case NETSOLVE_EXTERNAL :
      {
        int k;
        char *tmp;

        tmp = (char *)mxCalloc(sizeof(char),m*n+1);
        mexMakeMemoryPersistent(tmp);
        mxGetString(prhs[i+1],tmp,m*n+1);
        data_input[i] = tmp;
        break;
      }
      case NETSOLVE_ASCII :
      {
        int k;
        char *tmp;
        struct stat st;

        tmp = (char *)mxCalloc(sizeof(char),m*n+1);
        mexMakeMemoryPersistent(tmp);
        mxGetString(prhs[i+1],tmp,m*n+1);
        data_input[i] = tmp;
        if (stat(data_input[i],&st) == -1)
        {
          mexPrintf("Impossible to open file '%s'\n",data_input[i]);
          *returned_value = -1;
          return;
        }
        fprintf(stderr,"data_input[i] = '%s'\n",data_input[i]);
        sizes_input[i].m = (int)st.st_size;
        sizes_input[i].n = 1;
        sizes_input[i].l = sizes_input[i].m; 
        break;
      }
      case NETSOLVE_S :
      {
        int k;
        float *tmp;

        if (mxIsChar(prhs[i+1]))
        {
          mexPrintf("input #%d should not be a string",i);
          *returned_value = -1;
          return;
        }
        tmp = (float *)mxCalloc(sizeof(float),m*n);
        mexMakeMemoryPersistent(tmp);
        for (k=0;k<m*n;k++)
          tmp[k] = (float)pr[k];
        data_input[i] = tmp;
        break;
      }
      case NETSOLVE_D :
      {
        double *tmp;
        int k;

        if (mxIsChar(prhs[i+1]))
        {
          mexPrintf("input #%d should not be a string",i);
          *returned_value = -1;
          return;
        }
        tmp = (double *)mxCalloc(sizeof(double),m*n);
        mexMakeMemoryPersistent(tmp);
        for (k=0;k<m*n;k++)
          tmp[k] = (double)pr[k];
        data_input[i] = tmp;
        break;
      }
      case NETSOLVE_C :
      {
        int k;
        scomplex *tmp;
  
        if (mxIsChar(prhs[i+1]))
        {
          mexPrintf("input #%d should not be a string",i);
          *returned_value = -1;
          return;
        }
        tmp = (scomplex *)mxCalloc(sizeof(scomplex),m*n);
        mexMakeMemoryPersistent(tmp);
        for (k=0;k<n*m;k++)
        {
          tmp[k].r = (float)pr[k];
          tmp[k].i = (float)pi[k];
        }
        data_input[i] = tmp;
        break;
      }
      case NETSOLVE_Z :
      {
        int k;
        dcomplex *tmp;

        if (mxIsChar(prhs[i+1]))
        {
          mexPrintf("input #%d should not be a string",i);
          *returned_value = -1;
          return;
        }
        tmp = (dcomplex *)mxCalloc(sizeof(dcomplex),m*n);
        mexMakeMemoryPersistent(tmp);
        for (k=0;k<n*m;k++)
        {
          tmp[k].r = pr[k];
          tmp[k].i = pi[k];
        }
        data_input[i] = tmp;
        break;
      }
    }
  }
  
#ifdef MATLABTHREADS

  /* Beginning of critical section */
  if (pthread_mutex_lock(&thread_mutex))
  {
    mexPrintf("Impossible to obtain a mutex lock\n");
    *returned_value = -1;
    return;
  }

    /* finding a slot in the thread list */

  slot =-1;
  for (i=0;i<MAX_REQUEST;i++)
    if (request[i] == NULL)
    {
      slot = i;
      break;
    }

  if (slot == -1)
  {
    mexPrintf("Too many pending requests\n");
    if (pthread_mutex_unlock(&thread_mutex))
    {
      mexPrintf("Impossible to unlock a mutex lock\n");
      *returned_value = -1;
    }
    *returned _value = -1;
    return;
  }
 
  request[slot] = (thread_list *)mxCalloc(1,sizeof(thread_list));
  mexMakeMemoryPersistent(request[slot]);
  if (pthread_mutex_unlock(&thread_mutex))
  {
    mexPrintf("Impossible to unlock a mutex lock\n");
    *returned _value = -1;
    return;
  }
  /* End of critical section */

  request[slot]->command_name = (char *)mxCalloc(strlen(pb_desc->code),sizeof(char));
  strcpy(request[slot]->command_name,pb_desc->code);
  mexMakeMemoryPersistent(request[slot]->command_name);

  request[slot]->thread_status = THREAD_RUNNING;

  arguments = (thread_input *)mxCalloc(1,sizeof(thread_input));
  mexMakeMemoryPersistent(arguments);

  arguments->pb_desc = pb_desc;
  arguments->sizes_input = sizes_input;
  arguments->sizes_output = sizes_output;
  arguments->data_input = data_input;
  arguments->data_output = data_output;
  arguments->my_slot = slot;

  if (pthread_create(&(request[slot]->thread_id),NULL,
                thread_do,(void*)arguments))
  {
    mexPrintf("Could not create a thread\n");
    *returned_value = -1;
    return;
  }
  sleep(1);
  *return_value = slot;
  return;
#endif

#ifdef MATLABNOTHREADS

  info = mxgateway(-1,pb_desc,sizes_input,
                 sizes_output,data_input,data_output);

  if (info <0)
  {
    *returned_value = info;
    return;
  }
  *returned_value = (double)info;
  return;
#endif
}
